@@ -46,6 +46,7 @@
 
 #include "config.h"
 
+#include <assert.h>
 #include <stdio.h>
 
 #include "libs/fvwmlib.h"
@@ -131,206 +132,6 @@ static void delete_client_context(FvwmWindow *fw)
 	return;
 }
 
-/*
- *
- *  Procedure:
- *	CaptureOneWindow
- *	CaptureAllWindows
- *
- *   Decorates windows at start-up and during recaptures
- *
- */
-
-static void CaptureOneWindow(
-	const exec_context_t *exc, FvwmWindow *fw, Window window,
-	Window keep_on_top_win, Window parent_win, Bool is_recapture)
-{
-	Window w;
-	unsigned long data[1];
-	initial_window_options_t win_opts;
-	evh_args_t ea;
-	exec_context_changes_t ecc;
-	XEvent e;
-
-	if (fw == NULL)
-	{
-		return;
-	}
-	if (IS_SCHEDULED_FOR_DESTROY(fw))
-	{
-		/* Fvwm might crash in complex functions if we really try to
-		 * the dying window here because AddWindow() may fail and leave
-		 * a destroyed window in some structures.  By the way, it is
-		 * pretty useless to recapture a window that will vanish in a
-		 * moment. */
-		return;
-	}
-	/* Grab the server to make sure the window does not die during the
-	 * recapture. */
-	MyXGrabServer(dpy);
-	if (
-		!XGetGeometry(
-			dpy, FW_W(fw), &JunkRoot, &JunkX, &JunkY,
-			(unsigned int*)&JunkWidth, (unsigned int*)&JunkHeight,
-			(unsigned int*)&JunkBW, (unsigned int*)&JunkDepth))
-	{
-		/* The window has already died, do not recapture it! */
-		MyXUngrabServer(dpy);
-		return;
-	}
-	if (XFindContext(dpy, window, FvwmContext, (caddr_t *)&fw) != XCNOENT)
-	{
-		Bool is_mapped = IS_MAPPED(fw);
-
-		memset(&win_opts, 0, sizeof(win_opts));
-		win_opts.initial_state = DontCareState;
-		win_opts.flags.do_override_ppos = 1;
-		win_opts.flags.is_recapture = 1;
-		if (IS_ICONIFIED(fw))
-		{
-			win_opts.initial_state = IconicState;
-			win_opts.flags.is_iconified_by_parent =
-				IS_ICONIFIED_BY_PARENT(fw);
-		}
-		else
-		{
-			win_opts.initial_state = NormalState;
-			win_opts.flags.is_iconified_by_parent = 0;
-			SetMapStateProp(fw, NormalState);
-		}
-		data[0] = (unsigned long) fw->Desk;
-		XChangeProperty(
-			dpy, FW_W(fw), _XA_WM_DESKTOP, _XA_WM_DESKTOP, 32,
-			PropModeReplace, (unsigned char *) data, 1);
-
-		XSelectInput(dpy, FW_W(fw), NoEventMask);
-		w = FW_W(fw);
-		XUnmapWindow(dpy, FW_W_FRAME(fw));
-		border_undraw_decorations(fw);
-		RestoreWithdrawnLocation(fw, is_recapture, parent_win);
-		SET_DO_REUSE_DESTROYED(fw, 1); /* RBW - 1999/03/20 */
-		destroy_window(fw);
-		win_opts.flags.is_menu =
-			(is_recapture && fw != NULL && IS_TEAR_OFF_MENU(fw));
-
-
-		fev_make_null_event(&e, dpy);
-		e.xmaprequest.window = w;
-		e.xmaprequest.parent = Scr.Root;
-		ecc.x.etrigger = &e;
-		ecc.w.fw = NULL;
-		ecc.w.w = w;
-		ecc.w.wcontext = C_ROOT;
-		ea.exc = exc_clone_context(
-			exc, &ecc, ECC_ETRIGGER | ECC_FW | ECC_W |
-			ECC_WCONTEXT);
-		HandleMapRequestKeepRaised(&ea, keep_on_top_win, fw, &win_opts);
-		exc_destroy_context(ea.exc);
-
-		/* HandleMapRequestKeepRaised may have destroyed the fw if the
-		 * window vanished while in AddWindow(), so don't access fw
-		 * anymore before checking if it is a valid window. */
-		if (check_if_fvwm_window_exists(fw))
-		{
-			if (!fFvwmInStartup)
-			{
-				SET_MAP_PENDING(fw, 0);
-				SET_MAPPED(fw, is_mapped);
-			}
-		}
-	}
-	MyXUngrabServer(dpy);
-
-	return;
-}
-
-/* Put a transparent window all over the screen to hide what happens below. */
-static void hide_screen(
-	Bool do_hide, Window *ret_hide_win, Window *ret_parent_win)
-{
-	static Bool is_hidden = False;
-	static Window hide_win = None;
-	static Window parent_win = None;
-	XSetWindowAttributes xswa;
-	unsigned long valuemask;
-
-	if (do_hide == is_hidden)
-	{
-		/* nothing to do */
-		if (ret_hide_win)
-		{
-			*ret_hide_win = hide_win;
-		}
-		if (ret_parent_win)
-		{
-			*ret_parent_win = parent_win;
-		}
-
-		return;
-	}
-	is_hidden = do_hide;
-	if (do_hide)
-	{
-		xswa.override_redirect = True;
-		xswa.cursor = Scr.FvwmCursors[CRS_WAIT];
-		xswa.backing_store = NotUseful;
-		xswa.save_under = False;
-		xswa.background_pixmap = None;
-		valuemask = CWOverrideRedirect | CWCursor | CWSaveUnder |
-			CWBackingStore | CWBackPixmap;
-		hide_win = XCreateWindow(
-			dpy, Scr.Root, 0, 0, monitor_get_all_widths(),
-			monitor_get_all_heights(), 0, Pdepth, InputOutput,
-			Pvisual, valuemask, &xswa);
-		if (hide_win)
-		{
-			/* When recapturing, all windows are reparented to this
-			 * window. If they are reparented to the root window,
-			 * they will flash over the hide_win with XFree.  So
-			 * reparent them to an unmapped window that looks like
-			 * the root window. */
-			parent_win = XCreateWindow(
-				dpy, Scr.Root, 0, 0, monitor_get_all_widths(),
-				monitor_get_all_heights(), 0, CopyFromParent,
-				InputOutput, CopyFromParent, valuemask, &xswa);
-			if (!parent_win)
-			{
-				XDestroyWindow(dpy, hide_win);
-				hide_win = None;
-			}
-			else
-			{
-				XMapWindow(dpy, hide_win);
-				XFlush(dpy);
-			}
-		}
-	}
-	else
-	{
-		if (hide_win != None)
-		{
-			XDestroyWindow(dpy, hide_win);
-		}
-		if (parent_win != None)
-		{
-			XDestroyWindow(dpy, parent_win);
-		}
-		XFlush(dpy);
-		hide_win = None;
-		parent_win = None;
-	}
-	if (ret_hide_win)
-	{
-		*ret_hide_win = hide_win;
-	}
-	if (ret_parent_win)
-	{
-		*ret_parent_win = parent_win;
-	}
-
-	return;
-}
-
 /*
  *
  *  Procedure:
@@ -387,10 +188,7 @@ static void setup_window_structure(
 	FvwmWindow save_state;
 	FvwmWindow *savewin = NULL;
 
-	/*
-	  Allocate space for the FvwmWindow struct, or reuse an
-	  old one (on Recapture).
-	*/
+	/* Allocate space for the FvwmWindow struct, or reuse an old one. */
 	if (ReuseWin == NULL)
 	{
 		*pfw = fxmalloc(sizeof(FvwmWindow));
@@ -402,12 +200,6 @@ static void setup_window_structure(
 		memcpy(savewin, ReuseWin, sizeof(FvwmWindow));
 	}
 
-	/*
-	  RBW - 1999/05/28 - modify this when we implement the preserving of
-	  various states across a Recapture. The Destroy function in misc.c may
-	  also need tweaking, depending on what you want to preserve.
-	  For now, just zap any old information, except the desk.
-	*/
 	memset(*pfw, 0, sizeof(FvwmWindow));
 	FW_W(*pfw) = w;
 	if (savewin != NULL)
@@ -916,8 +708,7 @@ void setup_numeric_vals(FvwmWindow *fw, window_style *pstyle)
 		fw->edge_delay_ms_resize = DEFAULT_RESIZE_DELAY;
 	}
 	fw->edge_resistance_move = pstyle->edge_resistance_move;
-	fw->edge_resistance_xinerama_move =
-		pstyle->edge_resistance_xinerama_move;
+	fw->edge_resistance_screen_move = pstyle->edge_resistance_screen_move;
 
 	return;
 }
@@ -1416,14 +1207,10 @@ static void setup_icon(FvwmWindow *fw, window_style *pstyle)
 		if (SHAS_ICON(&pstyle->flags) &&
 		    S_ICON_OVERRIDE(SCF(*pstyle)) == ICON_OVERRIDE)
 		{
-			ICON_DBG((stderr,"si: iwh ignored '%s'\n",
-				  fw->name.name));
 			fw->icon_bitmap_file = SGET_ICON_NAME(*pstyle);
 		}
 		else
 		{
-			ICON_DBG((stderr,"si: using iwh '%s'\n",
-				  fw->name.name));
 			fw->icon_bitmap_file = NULL;
 		}
 	}
@@ -1432,27 +1219,21 @@ static void setup_icon(FvwmWindow *fw, window_style *pstyle)
 		if (SHAS_ICON(&pstyle->flags) &&
 		    S_ICON_OVERRIDE(SCF(*pstyle)) != NO_ICON_OVERRIDE)
 		{
-			ICON_DBG((stderr,"si: iph ignored '%s'\n",
-				  fw->name.name));
 			fw->icon_bitmap_file = SGET_ICON_NAME(*pstyle);
 		}
 		else
 		{
-			ICON_DBG((stderr,"si: using iph '%s'\n",
-				  fw->name.name));
 			fw->icon_bitmap_file = NULL;
 		}
 	}
 	else if (SHAS_ICON(&pstyle->flags))
 	{
 		/* an icon was specified */
-		ICON_DBG((stderr,"si: using style '%s'\n", fw->name.name));
 		fw->icon_bitmap_file = SGET_ICON_NAME(*pstyle);
 	}
 	else
 	{
 		/* use default icon */
-		ICON_DBG((stderr,"si: using default '%s'\n", fw->name.name));
 		fw->icon_bitmap_file = Scr.DefaultIcon;
 	}
 
@@ -1618,16 +1399,6 @@ static void destroy_mini_icon(FvwmWindow *fw)
 
 static void setup_key_and_button_grabs(FvwmWindow *fw)
 {
-#ifdef BUGS_ARE_COOL
-	/* dv (29-May-2001): If keys are grabbed separately for C_WINDOW and
-	 * the other contexts, new windows have problems when bindings are
-	 * removed.  Therefore, grab all keys in a single pass through the
-	 * list. */
-	GrabAllWindowKeys(
-		dpy, FW_W_FRAME(fw), Scr.AllBindings,
-		C_WINDOW|C_TITLE|C_RALL|C_LALL|C_SIDEBAR, GetUnusedModifiers(),
-		True);
-#endif
 	GrabAllWindowKeys(
 		dpy, FW_W_FRAME(fw), Scr.AllBindings,
 		C_TITLE|C_RALL|C_LALL|C_SIDEBAR|C_WINDOW, GetUnusedModifiers(),
@@ -1806,6 +1577,8 @@ int setup_visible_names(FvwmWindow *fw, int what_changed)
 			&bits, fw, &style, False);
 		if ((changed_names & bits) || (force_update & 1))
 		{
+			if (fw->visible_name != NoName)
+				free(fw->visible_name);
 			fw->visible_name = ext_name;
 			affected_titles |= 1;
 		}
@@ -2228,8 +2001,6 @@ void increase_icon_hint_count(FvwmWindow *fw)
 		default:
 			break;
 		}
-		ICON_DBG((stderr,"icon hint count++ (%d) '%s'\n",
-			  (int)WAS_ICON_HINT_PROVIDED(fw), fw->name.name));
 	}
 
 	return;
@@ -2517,7 +2288,7 @@ FvwmWindow *AddWindow(
 		char *cmd;
 
 		xasprintf(&cmd, "GotoDesk %s 0 %d", tm->si->name, fw->Desk);
-		execute_function_override_window(NULL, NULL, cmd, 0, fw);
+		execute_function_override_window(NULL, NULL, cmd, NULL, 0, fw);
 		free(cmd);
 
 		/* read the requested absolute geometry */
@@ -2556,7 +2327,7 @@ FvwmWindow *AddWindow(
 	{
 		rectangle attr_g;
 
-		UPDATE_FVWM_SCREEN(fw);
+		update_fvwm_monitor(fw);
 
 		if (IS_SHADED(fw))
 		{
@@ -2659,7 +2430,7 @@ FvwmWindow *AddWindow(
 	setup_key_and_button_grabs(fw);
 
 	/****** inform modules of new window ******/
-	UPDATE_FVWM_SCREEN(fw);
+	update_fvwm_monitor(fw);
 	BroadcastConfig(M_ADD_WINDOW,fw);
 	BroadcastWindowIconNames(fw, True, False);
 
@@ -2717,7 +2488,8 @@ FvwmWindow *AddWindow(
 			SET_STICKY_ACROSS_PAGES(fw, 0);
 			SET_STICKY_ACROSS_DESKS(fw, 0);
 			handle_stick(
-				NULL, exc2, "", stick_page, stick_desk, 1, 0);
+				NULL, exc2, "", NULL, stick_page, stick_desk,
+				1, 0);
 			exc_destroy_context(exc2);
 		}
 	}
@@ -2749,7 +2521,7 @@ FvwmWindow *AddWindow(
 		ecc.w.wcontext = C_WINDOW;
 		exc2 = exc_clone_context(
 			exc, &ecc, ECC_ETRIGGER | ECC_FW | ECC_WCONTEXT);
-		CMD_Resize(NULL, exc2, "");
+		CMD_Resize(NULL, exc2, "", NULL);
 		exc_destroy_context(exc2);
 	}
 
@@ -2764,7 +2536,7 @@ FvwmWindow *AddWindow(
 	{
 		rectangle big_g;
 		rectangle new_g;
-		frame_move_resize_args mr_args;
+		frame_move_resize_args mr_args2;
 
 		if (state_args.used_title_dir_for_shading)
 		{
@@ -2781,13 +2553,13 @@ FvwmWindow *AddWindow(
 		new_g = big_g;
 		get_shaded_geometry_with_dir(
 			fw, &new_g, &new_g, state_args.shade_dir);
-		mr_args = frame_create_move_resize_args(
+		mr_args2 = frame_create_move_resize_args(
 			fw, FRAME_MR_SHRINK | FRAME_MR_DONT_DRAW, &big_g,
 			&new_g, 0, state_args.shade_dir);
-		frame_move_resize(fw, mr_args);
+		frame_move_resize(fw, mr_args2);
 		SET_SHADED(fw, 1);
 		SET_SHADED_DIR(fw, state_args.shade_dir);
-		frame_free_move_resize_args(fw, mr_args);
+		frame_free_move_resize_args(fw, mr_args2);
 	}
 	if (!IS_SHADED(fw) && !IS_ICONIFIED(fw))
 	{
@@ -2818,23 +2590,44 @@ FvwmWindow *AddWindow(
 			     EWMH_STATE_HAS_HINT) ? 100 : 0;
 			sprintf(cmd,"Maximize on %i %i", h, v);
 			execute_function_override_window(
-				NULL, NULL, cmd, 0, fw);
+				NULL, NULL, cmd, NULL, 0, fw);
 		}
 	}
 	if (HAS_EWMH_INIT_FULLSCREEN_STATE(fw) == EWMH_STATE_HAS_HINT)
 	{
 		EWMH_fullscreen(fw);
 	}
-	if (!XGetGeometry(
-		    dpy, FW_W(fw), &JunkRoot, &JunkX, &JunkY,
-		    (unsigned int*)&JunkWidth, (unsigned int*)&JunkHeight,
-		    (unsigned int*)&JunkBW, (unsigned int*)&JunkDepth))
 	{
-		/* The window has disappeared somehow.	For some reason we do
-		 * not always get a DestroyNotify on the window, so make sure
-		 * it is destroyed. */
-		destroy_window(fw);
-		fw = NULL;
+		unsigned int nchildren;
+		Window parent, *children;
+		Status rc;
+
+		rc = XQueryTree(
+			dpy, w, &JunkRoot, &parent, &children, &nchildren);
+		if (nchildren > 0)
+		{
+			XFree(children);
+		}
+		/* Verify that (a) the window exists and (b) has really been
+		 * reparented to the FW_W_PARENT(fw) window.
+		 *
+		 * Only after reparenting fvwm is guaranteed to receive a
+		 * possible DestroyNotify event on the window. */
+		if (!rc || parent != FW_W_PARENT(fw))
+		{
+			/* No, the window has disappeared.  In case (b) some
+			 * window exists with the same id that has not been
+			 * reparented (either the same window or a new one).
+			 */
+			if (Scr.bo.do_display_new_window_names)
+			{
+				fvwm_debug(
+					__func__, "new window disappeared"
+					" before it could be reparented");
+			}
+			destroy_window(fw);
+			fw = NULL;
+		}
 	}
 
 	if (is_tracking_shared && fw != NULL) {
@@ -3503,9 +3296,7 @@ void destroy_window(FvwmWindow *fw)
 
 	/****** destroy mini icon ******/
 
-#ifdef MINI_ICON
 	destroy_mini_icon(fw);
-#endif
 
 	/****** free strings ******/
 
@@ -3578,7 +3369,7 @@ void destroy_window(FvwmWindow *fw)
  *
  */
 void RestoreWithdrawnLocation(
-	FvwmWindow *fw, Bool is_restart_or_recapture, Window parent)
+	FvwmWindow *fw, Bool is_restart_or_exit, Window parent)
 {
 	int w2,h2;
 	unsigned int mask;
@@ -3630,7 +3421,7 @@ void RestoreWithdrawnLocation(
 	 * half off the screen. (RN)
 	 */
 
-	if (!is_restart_or_recapture)
+	if (!is_restart_or_exit)
 	{
 		/* Don't mess with it if its partially on the screen now */
 		if (unshaded_g.x < 0 || unshaded_g.y < 0 ||
@@ -3679,7 +3470,7 @@ void RestoreWithdrawnLocation(
 	}
 
 	XConfigureWindow(dpy, FW_W(fw), mask, &xwc);
-	if (!is_restart_or_recapture)
+	if (!is_restart_or_exit)
 	{
 		/* must be initial capture */
 		XFlush(dpy);
@@ -3728,13 +3519,12 @@ void Reborder(void)
 	return;
 }
 
-void CaptureAllWindows(const exec_context_t *exc, Bool is_recapture)
+void CaptureAllWindows(const exec_context_t *exc)
 {
 	int i,j;
 	unsigned int nchildren;
 	Window root, parent, *children;
 	initial_window_options_t win_opts;
-	FvwmWindow *fw;
 
 	MyXGrabServer(dpy);
 	if (!XQueryTree(dpy, Scr.Root, &root, &parent, &children, &nchildren))
@@ -3744,8 +3534,7 @@ void CaptureAllWindows(const exec_context_t *exc, Bool is_recapture)
 	}
 	memset(&win_opts, 0, sizeof(win_opts));
 	win_opts.flags.do_override_ppos = 1;
-	win_opts.flags.is_recapture = 1;
-	if (!(Scr.flags.are_windows_captured)) /* initial capture? */
+	assert(!(Scr.flags.are_windows_captured));
 	{
 		evh_args_t ea;
 		exec_context_changes_t ecc;
@@ -3801,50 +3590,8 @@ void CaptureAllWindows(const exec_context_t *exc, Bool is_recapture)
 				exc_destroy_context(ea.exc);
 			}
 		}
-		Scr.flags.are_windows_captured = 1;
-	}
-	else /* must be recapture */
-	{
-		Window keep_on_top_win;
-		Window parent_win;
-		Window focus_w;
-		FvwmWindow *t;
-		struct monitor *mon = monitor_get_current();
-
-		t = get_focus_window();
-		focus_w = (t) ? FW_W(t) : None;
-		hide_screen(True, &keep_on_top_win, &parent_win);
-		/* reborder all windows */
-		for (i=0;i<nchildren;i++)
-		{
-			if (XFindContext(
-				    dpy, children[i], FvwmContext,
-				    (caddr_t *)&fw) != XCNOENT)
-			{
-				if ((fw != NULL && fw->m != NULL)) {
-					if (fw->m != mon)
-						continue;
-				}
-				CaptureOneWindow(
-					exc, fw, children[i], keep_on_top_win,
-					parent_win, is_recapture);
-			}
-		}
-		hide_screen(False, NULL, NULL);
-		/* find the window that had the focus and focus it again */
-		if (focus_w)
-		{
-			for (t = Scr.FvwmRoot.next; t && FW_W(t) != focus_w;
-			     t = t->next)
-			{
-				;
-			}
-			if (t)
-			{
-				SetFocusWindow(t, True, FOCUS_SET_FORCE);
-			}
-		}
 	}
+	Scr.flags.are_windows_captured = 1;
 	if (nchildren > 0)
 	{
 		XFree((char *)children);
