@@ -10,16 +10,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
-
-/*
- * This module is all original code
- * by Rob Nation
- * Copyright 1993, Robert Nation
- *     You may use this code for any purpose, as long as the original
- *     copyright remains in the source code and all documentation
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 /* ---------------------------- included header files ---------------------- */
@@ -32,11 +23,12 @@
 #include "libs/fvwmlib.h"
 #include "libs/Parse.h"
 #include "libs/wild.h"
-#include "libs/FScreen.h"
 #include "libs/charmap.h"
+#include "libs/FEvent.h"
 #include "libs/wcontext.h"
 #include "fvwm.h"
 #include "externs.h"
+#include "cmdparser.h"
 #include "execcontext.h"
 #include "functions.h"
 #include "conditional.h"
@@ -50,6 +42,7 @@
 #include "commands.h"
 #include "decorations.h"
 #include "virtual.h"
+#include "infostore.h"
 
 /* ---------------------------- local definitions -------------------------- */
 
@@ -209,7 +202,7 @@ static void circulate_cmd(
 		ecc.w.wcontext = new_context;
 		exc2 = exc_clone_context(
 			exc, &ecc, ECC_FW | ECC_W | ECC_WCONTEXT);
-		execute_function(cond_rc, exc2, restofline, flags);
+		execute_function(cond_rc, exc2, restofline, pc, flags);
 		exc_destroy_context(exc2);
 	}
 
@@ -248,7 +241,7 @@ static void select_cmd(F_CMD_ARGS)
 			cond_rc->rc = COND_RC_OK;
 		}
 		execute_function_override_wcontext(
-			cond_rc, exc, restofline, 0, C_WINDOW);
+			cond_rc, exc, restofline, pc, 0, C_WINDOW);
 	}
 	else if (cond_rc != NULL)
 	{
@@ -348,9 +341,9 @@ char *CreateFlagString(char *string, char **restptr)
 		{
 			if (*c == 0)
 			{
-				fvwm_msg(ERR, "CreateFlagString",
-					 "Conditionals require closing "
-					 "parenthesis");
+				fvwm_debug(__func__,
+					   "Conditionals require closing "
+					   "parenthesis");
 				*restptr = NULL;
 				return NULL;
 			}
@@ -363,7 +356,8 @@ char *CreateFlagString(char *string, char **restptr)
 
 		/* We must allocate a new string because we null terminate the
 		 * string between the [ ] or ( ) characters. */
-		retval = safemalloc(length + 1);
+		/* TA:  FIXME!  xasprintf() */
+		retval = fxmalloc(length + 1);
 		strncpy(retval, start, length);
 		retval[length] = 0;
 
@@ -509,8 +503,8 @@ void CreateConditionMask(char *flags, WindowConditionMask *mask)
 				    mask->placed_by_button_set_mask &
 				    ~button_mask)
 				{
-				  	  fvwm_msg(WARN, "PlacedByButton",
-						   "Condition always False.");
+				  	  fvwm_debug(__func__,
+						       "Condition always False.");
 				}
 				mask->placed_by_button_mask |= button_mask;
 			}
@@ -527,8 +521,8 @@ void CreateConditionMask(char *flags, WindowConditionMask *mask)
 				if (mask->placed_by_button_mask &
 				    mask->placed_by_button_set_mask & ~(1<<2))
 				{
-				  	  fvwm_msg(WARN, "PlacedByButton3",
-						   "Condition always False.");
+				  	  fvwm_debug(__func__,
+						       "Condition always False.");
 				}
 				mask->placed_by_button_mask |= (1<<2);
 			}
@@ -600,6 +594,36 @@ void CreateConditionMask(char *flags, WindowConditionMask *mask)
 			SET_HAS_HANDLES(mask, on);
 			SETM_HAS_HANDLES(mask, 1);
 		}
+		else if (StrEquals(cond, "HasTitle"))
+		{
+			SET_HAS_TITLE(mask, on);
+			SETM_HAS_TITLE(mask, 1);
+		}
+		else if (StrEquals(cond, "TitleAtTop"))
+		{
+			SET_TITLE_DIR(mask, DIR_N);
+			SETM_TITLE_DIR(mask, 1);
+		}
+		else if (StrEquals(cond, "TitleAtBottom"))
+		{
+			SET_TITLE_DIR(mask, DIR_S);
+			SETM_TITLE_DIR(mask, 1);
+		}
+		else if (StrEquals(cond, "TitleAtLeft"))
+		{
+			SET_TITLE_DIR(mask, DIR_W);
+			SETM_TITLE_DIR(mask, 1);
+		}
+		else if (StrEquals(cond, "TitleAtRight"))
+		{
+			SET_TITLE_DIR(mask, DIR_E);
+			SETM_TITLE_DIR(mask, 1);
+		}
+		else if (StrEquals(cond, "HasBorders"))
+		{
+			SET_HAS_NO_BORDER(mask, !on);
+			SETM_HAS_NO_BORDER(mask, 1);
+		}
 		else if (StrEquals(cond,"Iconifiable"))
 		{
 			SET_IS_UNICONIFIABLE(mask, !on);
@@ -674,8 +698,7 @@ void CreateConditionMask(char *flags, WindowConditionMask *mask)
 		}
 		else if (StrEquals(cond,"State"))
 		{
-			if (sscanf(tmp, "%u", &state) &&
-			    state >= 0 && state <= 31)
+			if (sscanf(tmp, "%u", &state) && state <= 31)
 			{
 				state = (1 << state);
 				if (on)
@@ -708,22 +731,40 @@ void CreateConditionMask(char *flags, WindowConditionMask *mask)
 			}
 			mask->my_flags.needs_same_layer = on;
 		}
+		else if (StrEquals(cond, "Desk"))
+		{
+			if (sscanf(tmp, "%d", &mask->desk)) {
+				tmp = SkipNTokens(tmp, 1);
+			}
+			mask->my_flags.do_check_cond_desk = on;
+		}
+		else if (StrEquals(cond, "Screen"))
+		{
+			char	scr_name[1024];
+
+			if (sscanf(tmp, "%s", scr_name)) {
+				tmp = SkipNTokens(tmp, 1);
+				mask->screen = monitor_resolve_name(scr_name);
+			}
+			mask->my_flags.do_check_screen = 1;
+
+			if (!on)
+				mask->my_flags.do_not_check_screen = 1;
+		}
 		else
 		{
 			struct name_condition *pp;
 			struct namelist *p;
-			char *condp = safestrdup(cond);
+			char *condp = fxstrdup(cond);
 
-			pp = (struct name_condition *)
-				safemalloc(sizeof(struct name_condition));
+			pp = fxmalloc(sizeof *pp);
 			pp->invert = (!on ? True : False);
 			pp->namelist = NULL;
 			pp->next = mask->name_condition;
 			mask->name_condition = pp;
 			for (;;)
 			{
-				p = (struct namelist *)
-					safemalloc(sizeof(struct namelist));
+				p = fxmalloc(sizeof *p);
 				p->name=condp;
 				p->next=pp->namelist;
 				pp->namelist=p;
@@ -741,12 +782,10 @@ void CreateConditionMask(char *flags, WindowConditionMask *mask)
 
 		if (tmp && *tmp)
 		{
-			fvwm_msg(OLD, "CreateConditionMask",
-				 "Use comma instead of whitespace to "
-				 "separate conditions");
-		}
-		else
-		{
+			fvwm_debug( __func__,
+			    "Use comma instead of whitespace to separate "
+			    "conditions");
+		} else {
 			if (allocated_condition != NULL)
 			{
 				free(allocated_condition);
@@ -779,6 +818,7 @@ Bool MatchesConditionMask(FvwmWindow *fw, WindowConditionMask *mask)
 	FvwmWindow *sf = get_focus_window();
 	struct name_condition *pp;
 	struct namelist *p;
+	struct monitor	*m = fw->m;
 	char *name;
 
 	/* match FixedSize conditional */
@@ -893,29 +933,29 @@ Bool MatchesConditionMask(FvwmWindow *fw, WindowConditionMask *mask)
 	    mask->my_flags.do_check_desk_and_page ||
 	    mask->my_flags.do_check_desk_and_global_page)
 	{
-		is_on_desk = (fw->Desk == Scr.CurrentDesk);
+		is_on_desk = (fw->Desk == m->virtual_scr.CurrentDesk);
 	}
 	is_on_page = 1;
 	if (mask->my_flags.do_check_page ||
 	    mask->my_flags.do_check_desk_and_page)
 	{
-		if (FScreenIsEnabled() && !mask->my_flags.do_not_check_screen)
+		if (!mask->my_flags.do_not_check_screen)
 		{
 			is_on_page = !!FScreenIsRectangleOnScreen(
 				NULL, FSCREEN_CURRENT, &(fw->g.frame));
 		}
 		else
 		{
-			is_on_page = !!IsRectangleOnThisPage(
-				&(fw->g.frame), Scr.CurrentDesk);
+			is_on_page = !!IsRectangleOnThisPage(m,
+				&(fw->g.frame), m->virtual_scr.CurrentDesk);
 		}
 	}
 	is_on_global_page = 1;
 	if (mask->my_flags.do_check_global_page ||
 	    mask->my_flags.do_check_desk_and_global_page)
 	{
-		is_on_global_page = !!IsRectangleOnThisPage(
-			&(fw->g.frame), Scr.CurrentDesk);
+		is_on_global_page = !!IsRectangleOnThisPage(m,
+			&(fw->g.frame), m->virtual_scr.CurrentDesk);
 	}
 
 	if (mask->my_flags.do_check_desk_and_page)
@@ -1085,6 +1125,34 @@ Bool MatchesConditionMask(FvwmWindow *fw, WindowConditionMask *mask)
 			return False;
 		}
 	}
+	if (mask->my_flags.do_check_cond_desk)
+	{
+		if (fw->Desk == mask->desk)
+			return True;
+		else
+			return False;
+
+		if (fw->Desk != mask->desk)
+			return True;
+		else
+			return False;
+	}
+	if (mask->my_flags.do_check_screen)
+	{
+		rectangle	 g;
+		const char	*mon_name;
+		struct monitor	*scr;
+
+		get_unshaded_geometry(fw, &g);
+		mon_name = FScreenOfPointerXY(g.x, g.y);
+		scr = monitor_resolve_name(mon_name);
+
+		if (mask->my_flags.do_not_check_screen) {
+			/* Negation of (!screen n) specified. */
+			return (scr != mask->screen);
+		} else
+			return (scr == mask->screen);
+	}
 
 	return True;
 }
@@ -1134,8 +1202,8 @@ static void direction_cmd(F_CMD_ARGS, Bool is_scan)
 	dir = gravity_parse_dir_argument(tmp, NULL, -1);
 	if (dir == -1 || dir > DIR_ALL_MASK)
 	{
-		fvwm_msg(ERR, "Direction", "Invalid direction %s",
-			 (tmp) ? tmp : "");
+		fvwm_debug(__func__, "Invalid direction %s",
+			   (tmp) ? tmp : "");
 		if (cond_rc != NULL)
 		{
 			cond_rc->rc = COND_RC_ERROR;
@@ -1148,9 +1216,8 @@ static void direction_cmd(F_CMD_ARGS, Bool is_scan)
 		tmp = PeekToken(action, &action);
 		if ( ! tmp )
 		{
-			fvwm_msg(
-				ERR, "Direction", "Missing minor direction %s",
-				(tmp) ? tmp : "");
+			fvwm_debug(__func__, "Missing minor direction %s",
+				   (tmp) ? tmp : "");
 			if (cond_rc != NULL)
 			{
 				cond_rc->rc = COND_RC_ERROR;
@@ -1162,9 +1229,8 @@ static void direction_cmd(F_CMD_ARGS, Bool is_scan)
 		if (dir2 == -1 || dir2 > DIR_NW ||
 		    (dir < 4) != (dir2 < 4) || (abs(dir - dir2) & 1) != 1)
 		{
-			fvwm_msg(
-				ERR, "Direction", "Invalid minor direction %s",
-				(tmp) ? tmp : "");
+			fvwm_debug(__func__, "Invalid minor direction %s",
+				   (tmp) ? tmp : "");
 			if (cond_rc != NULL)
 			{
 				cond_rc->rc = COND_RC_ERROR;
@@ -1245,15 +1311,15 @@ static void direction_cmd(F_CMD_ARGS, Bool is_scan)
 
 		if (dir > DIR_MAJOR_MASK && dir <= DIR_MINOR_MASK)
 		{
-			int tx;
+			int tix;
 			/* Rotate the diagonals 45 degrees counterclockwise. To
 			 * do this, multiply the matrix /+h +h\ with the vector
 			 * (x y).                       \-h +h/
 			 * h = sqrt(0.5). We can set h := 1 since absolute
 			 * distance doesn't * matter here. */
-			tx = his_cx + his_cy;
+			tix = his_cx + his_cy;
 			his_cy = -his_cx + his_cy;
-			his_cx = tx;
+			his_cx = tix;
 		}
 		/* Arrange so that distance and offset are positive in desired
 		 * direction. */
@@ -1358,7 +1424,7 @@ static void direction_cmd(F_CMD_ARGS, Bool is_scan)
 			cond_rc->rc = COND_RC_OK;
 		}
 		execute_function_override_window(
-			cond_rc, exc, restofline, 0, fw_best);
+			cond_rc, exc, restofline, pc, 0, fw_best);
 	}
 	else if (cond_rc != NULL)
 	{
@@ -1411,11 +1477,7 @@ static int __rc_matches_rcstring_consume(
 		else
 		{
 			match_rc = COND_RC_NO_MATCH;
-			/* Does anyone check for other numerical returncode
-			 * values? If so, this might have to be changed. */
-			fprintf(
-				stderr, "Unrecognised condition \"%s\" in"
-				" TestRc command.\n", flags);
+			/* Unrecognised condition. */
 		}
 	}
 	if (orig_flags != NULL)
@@ -1571,7 +1633,9 @@ void CMD_All(F_CMD_ARGS)
 	{
 		num++;
 	}
-	g = (FvwmWindow **)safemalloc(num * sizeof(FvwmWindow *));
+	if (num == 0)
+		return;
+	g = fxmalloc(num * sizeof(FvwmWindow *));
 	num = 0;
 	if (!use_stack)
 	{
@@ -1601,7 +1665,7 @@ void CMD_All(F_CMD_ARGS)
 		for (i = num-1; i >= 0; i--)
 		{
 			execute_function_override_window(
-				cond_rc, exc, restofline, 0, g[i]);
+				cond_rc, exc, restofline, pc, 0, g[i]);
 		}
 	}
 	else
@@ -1609,7 +1673,7 @@ void CMD_All(F_CMD_ARGS)
 		for (i = 0; i < num; i++)
 		{
 			execute_function_override_window(
-				cond_rc, exc, restofline, 0, g[i]);
+				cond_rc, exc, restofline, pc, 0, g[i]);
 		}
 	}
 	if (cond_rc != NULL && cond_rc->rc != COND_RC_BREAK)
@@ -1729,7 +1793,7 @@ void CMD_WindowId(F_CMD_ARGS)
 					cond_rc->rc = COND_RC_OK;
 				}
 				execute_function_override_window(
-					cond_rc, exc, action, 0, t);
+					cond_rc, exc, action, pc, 0, t);
 			}
 			else if (cond_rc != NULL)
 			{
@@ -1773,7 +1837,7 @@ void CMD_WindowId(F_CMD_ARGS)
 					ECC_FW | ECC_W | ECC_WCONTEXT);
 				execute_function(
 					cond_rc, exc2, action,
-					FUNC_IS_UNMANAGED);
+pc, 					FUNC_IS_UNMANAGED);
 				exc_destroy_context(exc2);
 			}
 		}
@@ -1810,7 +1874,7 @@ void CMD_TestRc(F_CMD_ARGS)
 	{
 		/* execute the command in root window context; overwrite the
 		 * return code with the return code of the command */
-		execute_function(cond_rc, exc, rest, 0);
+		execute_function(cond_rc, exc, rest, pc, 0);
 	}
 
 	return;
@@ -1836,7 +1900,7 @@ void CMD_Break(F_CMD_ARGS)
 
 void CMD_NoWindow(F_CMD_ARGS)
 {
-	execute_function_override_window(cond_rc, exc, action, 0, NULL);
+	execute_function_override_window(cond_rc, exc, action, pc, 0, NULL);
 
 	return;
 }
@@ -1880,15 +1944,14 @@ static Bool match_version(char *version, char *operator)
 
 	if (fvwm_version < 0)
 	{
-		char *tmp = safestrdup(VERSION);
+		char *tmp = fxstrdup(VERSION);
 		fvwm_version = ver(tmp);
 		free(tmp);
 	}
 	if (v < 0)
 	{
-		fprintf(
-			stderr, "match_version: Invalid version: %s\n",
-			version);
+		fvwm_debug(__func__, "match_version: Invalid version: %s\n",
+			   version);
 		return False;
 	}
 	if (strcmp(operator, ">=") == 0)
@@ -1917,9 +1980,8 @@ static Bool match_version(char *version, char *operator)
 	}
 	else
 	{
-		fprintf(
-			stderr, "match_version: Invalid operator: %s\n",
-			operator);
+		fvwm_debug(__func__, "match_version: Invalid operator: %s\n",
+			   operator);
 	}
 
 	return False;
@@ -2067,9 +2129,15 @@ void CMD_Test(F_CMD_ARGS)
 			flags_ptr = GetNextSimpleOption(flags_ptr, &var_name);
 			if (var_name)
 			{
-				const char *value = getenv(var_name);
+				const char *value;
+
+				if (memcmp(var_name,"infostore.",10) == 0)
+					value = get_metainfo_value(var_name+10);
+				else
+					value = getenv(var_name);
 
 				match = (value != NULL) ? 1 : 0;
+				free(var_name);
 			}
 			else
 			{
@@ -2083,10 +2151,23 @@ void CMD_Test(F_CMD_ARGS)
 			flags_ptr = GetNextSimpleOption(flags_ptr, &var_name);
 			if (var_name)
 			{
-				const char *value = getenv(var_name);
+                		const char *value;
+				if ( (strlen(var_name) > 10) && (memcmp(var_name,"infostore.",10) == 0)  )
+				{
+					value = get_metainfo_value(var_name+10);
+				}
+				else
+				{
+					value = getenv(var_name);
+				}
 				char *pattern;
 				/* unfortunately, GetNextSimpleOption is
-				 * broken, does not accept quoted empty "" */
+				 * broken, does not accept quoted empty ""
+				 *
+				 * DV (2-Sep-2014):  It is *not* broken.  The
+				 * parsing functions never return empty tokens
+				 * by design.
+				 */
 				flags_ptr = GetNextSimpleOption(
 					flags_ptr, &pattern);
 				if (!value)
@@ -2099,11 +2180,13 @@ void CMD_Test(F_CMD_ARGS)
 						/* include empty string case */
 						(!pattern[0] && !value[0]) ||
 						matchWildcards(pattern, value);
+					free(pattern);
 				}
 				else
 				{
 					error = 1;
 				}
+				free(var_name);
 			}
 			else
 			{
@@ -2112,6 +2195,7 @@ void CMD_Test(F_CMD_ARGS)
 		}
 		else if (StrEquals(cond, "EdgeIsActive"))
 		{
+			struct monitor	*m = monitor_get_current();
 			direction_t dir= DIR_NONE;
 			char *dirname;
 			char *next;
@@ -2137,13 +2221,13 @@ void CMD_Test(F_CMD_ARGS)
 			if (!error)
 			{
 				if (((dir == DIR_W || dir == DIR_NONE) &&
-					Scr.PanFrameLeft.isMapped) ||
+					m->PanFrameLeft.isMapped) ||
 				    ((dir == DIR_N || dir == DIR_NONE) &&
-					Scr.PanFrameTop.isMapped) ||
+					m->PanFrameTop.isMapped) ||
 				    ((dir == DIR_S || dir == DIR_NONE) &&
-					Scr.PanFrameBottom.isMapped) ||
+					m->PanFrameBottom.isMapped) ||
 				    ((dir == DIR_E || dir == DIR_NONE) &&
- 				    	Scr.PanFrameRight.isMapped))
+ 				    	m->PanFrameRight.isMapped))
 				{
 				  	match = 1;
 				}
@@ -2156,6 +2240,7 @@ void CMD_Test(F_CMD_ARGS)
 		}
 		else if (StrEquals(cond, "EdgeHasPointer"))
 		{
+			struct monitor	*m = monitor_get_current();
 			int x,y;
 			Window win;
 			direction_t dir = DIR_NONE;
@@ -2194,13 +2279,13 @@ void CMD_Test(F_CMD_ARGS)
 				{
 					if (dir == DIR_NONE ||
 					    (dir == DIR_N &&
-					     win == Scr.PanFrameTop.win) ||
+					     win == m->PanFrameTop.win) ||
 					    (dir == DIR_S &&
-					     win == Scr.PanFrameBottom.win) ||
+					     win == m->PanFrameBottom.win) ||
 					    (dir == DIR_E &&
-					     win == Scr.PanFrameRight.win) ||
+					     win == m->PanFrameRight.win) ||
 					    (dir == DIR_W &&
-					     win == Scr.PanFrameLeft.win))
+					     win == m->PanFrameLeft.win))
 					{
 						match = 1;
 					}
@@ -2220,9 +2305,6 @@ void CMD_Test(F_CMD_ARGS)
 		{
 			/* unrecognized condition */
 			error = 1;
-			fprintf(
-				stderr, "Unrecognised condition \"%s\" in"
-				" Test command.\n", cond);
 		}
 
 		if (reverse)
@@ -2243,7 +2325,7 @@ void CMD_Test(F_CMD_ARGS)
 	}
 	if (!error && match)
 	{
-		execute_function(cond_rc, exc, restofline, 0);
+		execute_function(cond_rc, exc, restofline, pc, 0);
 	}
 	if (cond_rc != NULL)
 	{
