@@ -1,18 +1,17 @@
 /* -*-c-*- */
 /* Copyright (C) 2001  Olivier Chapuis */
 /* This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
+* it under the terms of the GNU General Public License as published by
+* the Free Software Foundation; either version 2 of the License, or
+* (at your option) any later version.
+*
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 /*
@@ -54,11 +53,13 @@
 #include "config.h"
 
 #include <stdio.h>
-#include <X11/Xatom.h>
-
+#include <stdarg.h>
+#include <sys/times.h>
+#include <time.h>
+#include "libs/fvwm_x11.h"
 #include "libs/fvwmlib.h"
-#include "libs/FScreen.h"
 #include "fvwm.h"
+#include "cmdparser.h"
 #include "execcontext.h"
 #include "functions.h"
 #include "commands.h"
@@ -74,6 +75,8 @@
 #include "geometry.h"
 #include "window_flags.h"
 
+#define EWMH_DEBUG 0
+
 typedef struct kst_item
 {
 	Window w;
@@ -84,18 +87,6 @@ typedef struct kst_item
 Atom XA_UTF8_STRING = None;
 KstItem *ewmh_KstWinList = NULL;
 
-/*
- * Default
- */
-ewmhInfo ewmhc =
-{
-	4,            /* NumberOfDesktops */
-	0,            /* MaxDesktops limit the number of desktops*/
-	0,            /* CurrentNumberOfDesktops */
-	False,        /* NeedsToCheckDesk */
-	{0, 0, 0, 0}, /* BaseStrut */
-};
-
 /*
  * The ewmh atoms lists
  */
@@ -268,6 +259,47 @@ ewmh_atom_list atom_list[] =
 	L_ENTRY(EWMH_ATOM_LIST_END,               NULL)
 };
 
+static void EWMH_DLOG(char *msg, ...)
+{
+	if (EWMH_DEBUG)
+	{
+		va_list args;
+		clock_t time_val, time_taken;
+		static clock_t start_time = 0;
+		static clock_t prev_time = 0;
+		struct tms not_used_tms;
+		char buffer[200]; /* oversized */
+		time_t mytime;
+		struct tm *t_ptr;
+
+		time(&mytime);
+		t_ptr = localtime(&mytime);
+		if (start_time == 0)
+		{
+			/* get clock ticks */
+			prev_time = start_time =
+				(unsigned int)times(&not_used_tms);
+		}
+		/* get clock ticks */
+		time_val = (unsigned int)times(&not_used_tms);
+		time_taken = time_val - prev_time;
+		prev_time = time_val;
+		sprintf(
+			buffer, "%.2d:%.2d:%.2d %6ld",
+			t_ptr->tm_hour, t_ptr->tm_min, t_ptr->tm_sec,
+			time_taken);
+
+		fvwm_debug(__func__, "EWMH DEBUG: ");
+		va_start(args,msg);
+		vfprintf(stderr, msg, args);
+		va_end(args);
+		fvwm_debug(__func__, "\n");
+		fvwm_debug(__func__, "            [time]: %s\n",buffer);
+	}
+
+	return;
+}
+
 /*
  * Atoms utilities
  */
@@ -308,29 +340,28 @@ ewmh_atom *get_ewmh_atom_by_name(
 
 ewmh_atom *ewmh_GetEwmhAtomByAtom(Atom atom, ewmh_atom_list_name list_name)
 {
-	Bool done = 0;
 	int i = 0;
 
-	while(!done && atom_list[i].name != EWMH_ATOM_LIST_END)
+	for (i = 0; atom_list[i].name != EWMH_ATOM_LIST_END; i++)
 	{
-		if (atom_list[i].name == list_name ||
-		    list_name == EWMH_ATOM_LIST_ALL)
+		ewmh_atom *item;
+
+		if (list_name != atom_list[i].name &&
+		    list_name != EWMH_ATOM_LIST_ALL)
 		{
-			ewmh_atom *list = atom_list[i].list;
-			while(list->name != NULL)
+			continue;
+		}
+		for (item = atom_list[i].list; item->name != NULL; item++)
+		{
+			if (atom == item->atom)
 			{
-				if (atom == list->atom)
-				{
-					return list;
-				}
-				list++;
-			}
-			if (atom_list[i].name == list_name)
-			{
-				return NULL;
+				return item;
 			}
 		}
-		i++;
+		if (list_name == atom_list[i].name)
+		{
+			return NULL;
+		}
 	}
 
 	return NULL;
@@ -364,13 +395,13 @@ void ewmh_ChangeProperty(
 		{
 			format = 8;
 		}
-	       
+
                 asize = atom_size(format);
                 if (
 		    format == 32 && asize * 8 != format &&
 		    strcmp(atom_name, "_NET_WM_ICON") == 0)
 		{
-			long *datacopy = (long*)safemalloc(asize * length);
+			long *datacopy = fxmalloc(asize * length);
 			int i;
 
 			for (i = 0; i < length; i++)
@@ -428,7 +459,7 @@ void *atom_get(Window win, Atom to_get, Atom type, int *size)
 		int asize;
 
 		asize = atom_size(format_ret);
-		data = safemalloc(num_ret * asize);
+		data = fxmalloc(num_ret * asize);
 		if (format_ret == 32 && asize * 8 != format_ret)
 		{
 			int i;
@@ -493,22 +524,24 @@ int check_desk(void)
 	return d;
 }
 
-void EWMH_SetCurrentDesktop(void)
+void EWMH_SetCurrentDesktop(struct monitor *m)
 {
 	long val;
 
-	val = Scr.CurrentDesk;
+	/* FIXME: this should broadcast to all monitors. */
 
-	if (val < 0 || (val >= ewmhc.MaxDesktops && ewmhc.MaxDesktops != 0))
+	val = m->virtual_scr.CurrentDesk;
+
+	if (val < 0 || (val >= m->ewmhc.MaxDesktops && m->ewmhc.MaxDesktops != 0))
 	{
 		return;
 	}
 
-	if (val >= ewmhc.CurrentNumberOfDesktops ||
-	    (ewmhc.NumberOfDesktops != ewmhc.CurrentNumberOfDesktops &&
-	     val < ewmhc.CurrentNumberOfDesktops))
+	if (val >= m->ewmhc.CurrentNumberOfDesktops ||
+	    (m->ewmhc.NumberOfDesktops != m->ewmhc.CurrentNumberOfDesktops &&
+	     val < m->ewmhc.CurrentNumberOfDesktops))
 	{
-		EWMH_SetNumberOfDesktops();
+		EWMH_SetNumberOfDesktops(m);
 	}
 
 	ewmh_ChangeProperty(Scr.Root,"_NET_CURRENT_DESKTOP",
@@ -518,51 +551,52 @@ void EWMH_SetCurrentDesktop(void)
 	return;
 }
 
-void EWMH_SetNumberOfDesktops(void)
+void EWMH_SetNumberOfDesktops(struct monitor *m)
 {
 	long val;
 
-	if (ewmhc.CurrentNumberOfDesktops < ewmhc.NumberOfDesktops)
+	if (m->ewmhc.CurrentNumberOfDesktops < m->ewmhc.NumberOfDesktops)
 	{
-		ewmhc.CurrentNumberOfDesktops = ewmhc.NumberOfDesktops;
+		m->ewmhc.CurrentNumberOfDesktops = m->ewmhc.NumberOfDesktops;
 	}
 
-	if (ewmhc.CurrentNumberOfDesktops > ewmhc.NumberOfDesktops ||
-	    ewmhc.NeedsToCheckDesk)
+	if (m->ewmhc.CurrentNumberOfDesktops > m->ewmhc.NumberOfDesktops ||
+	    m->ewmhc.NeedsToCheckDesk)
 	{
 		int d = check_desk();
 
-		ewmhc.NeedsToCheckDesk = False;
-		if (d >= ewmhc.MaxDesktops && ewmhc.MaxDesktops != 0)
+		m->ewmhc.NeedsToCheckDesk = False;
+		if (d >= m->ewmhc.MaxDesktops && m->ewmhc.MaxDesktops != 0)
 			d = 0;
-		ewmhc.CurrentNumberOfDesktops =
-			max(ewmhc.NumberOfDesktops, d+1);
+		m->ewmhc.CurrentNumberOfDesktops =
+			max(m->ewmhc.NumberOfDesktops, d+1);
 	}
 
-	if (Scr.CurrentDesk >= ewmhc.CurrentNumberOfDesktops &&
-	    (Scr.CurrentDesk < ewmhc.MaxDesktops || ewmhc.MaxDesktops == 0))
+	if (m->virtual_scr.CurrentDesk >= m->ewmhc.CurrentNumberOfDesktops &&
+	    (m->virtual_scr.CurrentDesk < m->ewmhc.MaxDesktops ||
+	    m->ewmhc.MaxDesktops == 0))
 	{
-		ewmhc.CurrentNumberOfDesktops = Scr.CurrentDesk + 1;
+		m->ewmhc.CurrentNumberOfDesktops = m->virtual_scr.CurrentDesk + 1;
 	}
 
-	val = (long)ewmhc.CurrentNumberOfDesktops;
+	val = (long)m->ewmhc.CurrentNumberOfDesktops;
 	ewmh_ChangeProperty(Scr.Root, "_NET_NUMBER_OF_DESKTOPS",
 			    EWMH_ATOM_LIST_CLIENT_ROOT,
 			    (unsigned char *)&val, 1);
-	ewmh_SetWorkArea();
+	ewmh_SetWorkArea(m);
 
 	return;
 }
 
-void EWMH_SetDesktopViewPort(void)
+void EWMH_SetDesktopViewPort(struct monitor *m)
 {
 	long val[256][2]; /* no more than 256 desktops */
 	int i = 0;
 
-	while(i < ewmhc.NumberOfDesktops && i < 256)
+	while(i < m->ewmhc.NumberOfDesktops && i < 256)
 	{
-		val[i][0] = Scr.Vx;
-		val[i][1] = Scr.Vy;
+		val[i][0] = m->virtual_scr.Vx;
+		val[i][1] = m->virtual_scr.Vy;
 		i++;
 	}
 	ewmh_ChangeProperty(
@@ -572,12 +606,14 @@ void EWMH_SetDesktopViewPort(void)
 	return;
 }
 
-void EWMH_SetDesktopGeometry(void)
+void EWMH_SetDesktopGeometry(struct monitor *m)
 {
 	long val[2];
 
-	val[0] = Scr.VxMax + Scr.MyDisplayWidth;
-	val[1] = Scr.VyMax + Scr.MyDisplayHeight;
+	/* FIXME: needs broadcast for global. */
+
+	val[0] = m->virtual_scr.VxMax + m->si->w; //m->virtual_scr.MyDisplayWidth;
+	val[1] = m->virtual_scr.VyMax + m->si->h; //m->virtual_scr.MyDisplayHeight;
 	ewmh_ChangeProperty(
 		Scr.Root,"_NET_DESKTOP_GEOMETRY", EWMH_ATOM_LIST_CLIENT_ROOT,
 		(unsigned char *)&val, 2);
@@ -599,16 +635,17 @@ void EWMH_SetActiveWindow(Window w)
 
 void EWMH_SetWMDesktop(FvwmWindow *fw)
 {
+	struct monitor *m = (fw && fw->m) ? fw->m : monitor_get_current();
 	long desk = fw->Desk;
 
 	if (IS_STICKY_ACROSS_DESKS(fw))
 	{
 		desk = (unsigned long)-1;
 	}
-	else if (desk >= ewmhc.CurrentNumberOfDesktops)
+	else if (desk >= m->ewmhc.CurrentNumberOfDesktops)
 	{
-		ewmhc.NeedsToCheckDesk = True;
-		EWMH_SetNumberOfDesktops();
+		m->ewmhc.NeedsToCheckDesk = True;
+		EWMH_SetNumberOfDesktops(m);
 	}
 	ewmh_ChangeProperty(
 		FW_W(fw), "_NET_WM_DESKTOP", EWMH_ATOM_LIST_CLIENT_WIN,
@@ -657,7 +694,7 @@ void EWMH_SetWMState(FvwmWindow *fw, Bool do_restore)
  */
 
 /*** kde system tray ***/
-/* #define DEBUG_KST */
+#define DEBUG_KST 0
 static
 void add_kst_item(Window w)
 {
@@ -671,7 +708,7 @@ void add_kst_item(Window w)
 		prev = &(t->next);
 		t = t->next;
 	}
-	*prev = (KstItem *)safemalloc(sizeof(KstItem));
+	*prev = fxmalloc(sizeof(KstItem));
 	(*prev)->w = w;
 	(*prev)->next = NULL;
 
@@ -719,24 +756,27 @@ void set_kde_sys_tray(void)
 
 	if (nbr > 0)
 	{
-		wins = (Window *)safemalloc(sizeof(Window) * nbr);
+		wins = fxmalloc(sizeof(Window) * nbr);
 	}
 
 	t = ewmh_KstWinList;
-#ifdef DEBUG_KST
-	fprintf(stderr,"ADD_TO_KST: ");
-#endif
+	if (DEBUG_KST)
+	{
+		fvwm_debug(__func__, "ADD_TO_KST: ");
+	}
 	while (t != NULL)
 	{
-#ifdef DEBUG_KST
-		fprintf(stderr,"0x%lx ",t->w);
-#endif
+		if (DEBUG_KST)
+		{
+			fvwm_debug(__func__, "0x%lx ",t->w);
+		}
 		wins[i++] = t->w;
 		t = t->next;
 	}
-#ifdef DEBUG_KST
-	fprintf(stderr,"\n");
-#endif
+	if (DEBUG_KST)
+	{
+		fvwm_debug(__func__, "\n");
+	}
 
 	ewmh_ChangeProperty(Scr.Root,"_KDE_NET_SYSTEM_TRAY_WINDOWS",
 			    EWMH_ATOM_LIST_FVWM_ROOT,
@@ -812,9 +852,10 @@ int EWMH_IsKdeSysTrayWindow(Window w)
 	{
 		return 0;
 	}
-#ifdef DEBUG_KST
-	fprintf(stderr,"IsKdeSysTrayWindow: 0x%lx\n", w);
-#endif
+	if (DEBUG_KST)
+	{
+		fvwm_debug(__func__, "IsKdeSysTrayWindow: 0x%lx\n", w);
+	}
 
 	return 1;
 }
@@ -835,32 +876,36 @@ void EWMH_ManageKdeSysTray(Window w, int type)
 	switch(type)
 	{
 	case UnmapNotify:
-#ifdef DEBUG_KST
-		fprintf(stderr,"KST_UNMAP: 0x%lx\n", w);
-#endif
+		if (DEBUG_KST)
+		{
+			fvwm_debug(__func__, "KST_UNMAP: 0x%lx\n", w);
+		}
 		XSelectInput(dpy, w, StructureNotifyMask);
 		XFlush(dpy);
 		break;
 	case DestroyNotify:
-#ifdef DEBUG_KST
-		fprintf(stderr,"KST_DESTROY: 0x%lx\n", w);
-#endif
+		if (DEBUG_KST)
+		{
+			fvwm_debug(__func__, "KST_DESTROY: 0x%lx\n", w);
+		}
 		XSelectInput(dpy, t->w, NoEventMask);
 		XFlush(dpy);
 		delete_kst_item(w);
 		set_kde_sys_tray();
 		break;
 	case ReparentNotify:
-#ifdef DEBUG_KST
-		fprintf(stderr,"KST_Reparent: 0x%lx\n", w);
-#endif
+		if (DEBUG_KST)
+		{
+			fvwm_debug(__func__, "KST_Reparent: 0x%lx\n", w);
+		}
 		XSelectInput(dpy, w, StructureNotifyMask);
 		XFlush(dpy);
 		break;
 	default:
-#ifdef DEBUG_KST
-		fprintf(stderr,"KST_NO: 0x%lx\n", w);
-#endif
+		if (DEBUG_KST)
+		{
+			fvwm_debug(__func__, "KST_NO: 0x%lx\n", w);
+		}
 		break;
 	}
 
@@ -869,7 +914,7 @@ void EWMH_ManageKdeSysTray(Window w, int type)
 
 /**** Client lists ****/
 
-void EWMH_SetClientList(void)
+void EWMH_SetClientList(struct monitor *m)
 {
 	Window *wl = NULL;
 	FvwmWindow *fw;
@@ -882,7 +927,7 @@ void EWMH_SetClientList(void)
 	}
 	if (nbr != 0)
 	{
-		wl = (Window *)safemalloc(sizeof(Window) * nbr);
+		wl = fxmalloc(sizeof(Window) * nbr);
 		for (fw = Scr.FvwmRoot.next; fw != NULL; fw = fw->next)
 		{
 			wl[i++] = FW_W(fw);
@@ -899,7 +944,7 @@ void EWMH_SetClientList(void)
 	return;
 }
 
-void EWMH_SetClientListStacking(void)
+void EWMH_SetClientListStacking(struct monitor *m)
 {
 	Window *wl = NULL;
 	FvwmWindow *fw;
@@ -915,7 +960,7 @@ void EWMH_SetClientListStacking(void)
 	i = nbr-1;
 	if (nbr != 0)
 	{
-		wl = (Window *)safemalloc(sizeof(Window) * nbr);
+		wl = fxmalloc(sizeof(Window) * nbr);
 		for (
 			fw = Scr.FvwmRoot.stack_next; fw != &Scr.FvwmRoot;
 			fw = fw->stack_next)
@@ -937,17 +982,20 @@ void EWMH_SetClientListStacking(void)
 /**** Working Area stuff ****/
 /**** At present time we support only sticky windows with strut ****/
 
-void ewmh_SetWorkArea(void)
+void ewmh_SetWorkArea(struct monitor *m)
 {
 	long val[256][4]; /* no more than 256 desktops */
 	int i = 0;
 
-	while(i < ewmhc.NumberOfDesktops && i < 256)
+	if (m->Desktops == NULL)
+		return;
+
+	while(i < m->ewmhc.NumberOfDesktops && i < 256)
 	{
-		val[i][0] = Scr.Desktops->ewmh_working_area.x;
-		val[i][1] = Scr.Desktops->ewmh_working_area.y;
-		val[i][2] = Scr.Desktops->ewmh_working_area.width;
-		val[i][3] = Scr.Desktops->ewmh_working_area.height;
+		val[i][0] = m->Desktops->ewmh_working_area.x;
+		val[i][1] = m->Desktops->ewmh_working_area.y;
+		val[i][2] = m->Desktops->ewmh_working_area.width;
+		val[i][3] = m->Desktops->ewmh_working_area.height;
 		i++;
 	}
 	ewmh_ChangeProperty(
@@ -957,17 +1005,21 @@ void ewmh_SetWorkArea(void)
 	return;
 }
 
-void ewmh_ComputeAndSetWorkArea(void)
+void ewmh_ComputeAndSetWorkArea(struct monitor *m)
 {
-	int left = ewmhc.BaseStrut.left;
-	int right = ewmhc.BaseStrut.right;
-	int top = ewmhc.BaseStrut.top;
-	int bottom = ewmhc.BaseStrut.bottom;
+	int left = m->ewmhc.BaseStrut.left;
+	int right = m->ewmhc.BaseStrut.right;
+	int top = m->ewmhc.BaseStrut.top;
+	int bottom = m->ewmhc.BaseStrut.bottom;
 	int x,y,width,height;
 	FvwmWindow *fw;
 
+	/* FIXME: needs broadcast if global monitor in use. */
+
 	for (fw = Scr.FvwmRoot.next; fw != NULL; fw = fw->next)
 	{
+		if (fw->m != m)
+			continue;
 		if (
 			DO_EWMH_IGNORE_STRUT_HINTS(fw) ||
 			!IS_STICKY_ACROSS_PAGES(fw))
@@ -982,36 +1034,55 @@ void ewmh_ComputeAndSetWorkArea(void)
 
 	x = left;
 	y = top;
-	width = Scr.MyDisplayWidth - (left + right);
-	height = Scr.MyDisplayHeight - (top + bottom);
+
+	width =  m->si->w - (left + right);
+	height = m->si->h - (top + bottom);
+
+	if (Scr.bo.do_debug_randr)
+	{
+		fvwm_debug(
+			__func__, "monitor '%s': {l: %d, r: %d, t: %d, b: %d} "
+			"{x: %d, y: %d, w: %d, h: %d}\n", m->si->name,
+			left, right, top, bottom, x, y, width, height);
+	}
 
 	if (
-		Scr.Desktops->ewmh_working_area.x != x ||
-		Scr.Desktops->ewmh_working_area.y != y ||
-		Scr.Desktops->ewmh_working_area.width != width ||
-		Scr.Desktops->ewmh_working_area.height != height)
+		m->Desktops->ewmh_working_area.x != x ||
+		m->Desktops->ewmh_working_area.y != y ||
+		m->Desktops->ewmh_working_area.width != width ||
+		m->Desktops->ewmh_working_area.height != height)
 	{
-		Scr.Desktops->ewmh_working_area.x = x;
-		Scr.Desktops->ewmh_working_area.y = y;
-		Scr.Desktops->ewmh_working_area.width = width;
-		Scr.Desktops->ewmh_working_area.height = height;
-		ewmh_SetWorkArea();
+		m->Desktops->ewmh_working_area.x = x;
+		m->Desktops->ewmh_working_area.y = y;
+		m->Desktops->ewmh_working_area.width = width;
+		m->Desktops->ewmh_working_area.height = height;
+
+		if (Scr.bo.do_debug_randr)
+		{
+			fvwm_debug(__func__, "differ, so setting work area\n");
+		}
+
+		ewmh_SetWorkArea(m);
 	}
 
 	return;
 }
 
-void ewmh_HandleDynamicWorkArea(void)
+void ewmh_HandleDynamicWorkArea(struct monitor *m)
 {
-	int dyn_left = ewmhc.BaseStrut.left;
-	int dyn_right = ewmhc.BaseStrut.right;
-	int dyn_top = ewmhc.BaseStrut.top;
-	int dyn_bottom = ewmhc.BaseStrut.bottom;
+	int dyn_left = m->ewmhc.BaseStrut.left;
+	int dyn_right = m->ewmhc.BaseStrut.right;
+	int dyn_top = m->ewmhc.BaseStrut.top;
+	int dyn_bottom = m->ewmhc.BaseStrut.bottom;
 	int x,y,width,height;
 	FvwmWindow *fw;
 
+	/* FIXME: needs broadcast if global monitor in use. */
+
 	for (fw = Scr.FvwmRoot.next; fw != NULL; fw = fw->next)
 	{
+		if (fw->m != m)
+			continue;
 		if (
 			DO_EWMH_IGNORE_STRUT_HINTS(fw) ||
 			!IS_STICKY_ACROSS_PAGES(fw))
@@ -1026,43 +1097,58 @@ void ewmh_HandleDynamicWorkArea(void)
 
 	x = dyn_left;
 	y = dyn_top;
-	width = Scr.MyDisplayWidth - (dyn_left + dyn_right);
-	height = Scr.MyDisplayHeight - (dyn_top + dyn_bottom);
+
+	width  = m->si->w - (dyn_left + dyn_right);
+	height = m->si->h - (dyn_top + dyn_bottom);
 
 	if (
-		Scr.Desktops->ewmh_dyn_working_area.x != x ||
-		Scr.Desktops->ewmh_dyn_working_area.y != y ||
-		Scr.Desktops->ewmh_dyn_working_area.width != width ||
-		Scr.Desktops->ewmh_dyn_working_area.height != height)
+		m->Desktops->ewmh_dyn_working_area.x != x ||
+		m->Desktops->ewmh_dyn_working_area.y != y ||
+		m->Desktops->ewmh_dyn_working_area.width != width ||
+		m->Desktops->ewmh_dyn_working_area.height != height)
 	{
-		Scr.Desktops->ewmh_dyn_working_area.x = x;
-		Scr.Desktops->ewmh_dyn_working_area.y = y;
-		Scr.Desktops->ewmh_dyn_working_area.width = width;
-		Scr.Desktops->ewmh_dyn_working_area.height = height;
+		m->Desktops->ewmh_dyn_working_area.x = x;
+		m->Desktops->ewmh_dyn_working_area.y = y;
+		m->Desktops->ewmh_dyn_working_area.width = width;
+		m->Desktops->ewmh_dyn_working_area.height = height;
 		/* here we may update the maximized window ...etc */
 	}
 
 	return;
 }
 
-void EWMH_UpdateWorkArea(void)
+void EWMH_UpdateWorkArea(struct monitor *m)
 {
-	ewmh_ComputeAndSetWorkArea();
-	ewmh_HandleDynamicWorkArea();
-
-	return;
+	ewmh_ComputeAndSetWorkArea(m);
+	ewmh_HandleDynamicWorkArea(m);
 }
 
 void EWMH_GetWorkAreaIntersection(
 	FvwmWindow *fw, int *x, int *y, int *w, int *h, int type)
 {
+	struct monitor	*m = (fw && fw->m) ? fw->m : monitor_get_current();
+
+	EWMH_UpdateWorkArea(m);
+
 	int nx,ny,nw,nh;
-	int area_x = Scr.Desktops->ewmh_working_area.x;
-	int area_y = Scr.Desktops->ewmh_working_area.y;
-	int area_w = Scr.Desktops->ewmh_working_area.width;
-	int area_h = Scr.Desktops->ewmh_working_area.height;
+	int area_x = m->Desktops->ewmh_working_area.x;
+	int area_y = m->Desktops->ewmh_working_area.y;
+	int area_w = m->Desktops->ewmh_working_area.width;
+	int area_h = m->Desktops->ewmh_working_area.height;
 	Bool is_dynamic = False;
 
+	if (Scr.bo.do_debug_randr)
+	{
+		fvwm_debug(
+			__func__, "mon: %s {ax: %d, ay: %d, aw: %d, ah: %d\n",
+			m->si->name, area_x, area_y, area_w, area_h);
+		fvwm_debug(
+			__func__, "mon: %s {x: %d, y: %d, w: %d, h: %d\n",
+			m->si->name, *x, *y, *w, *h);
+	}
+
+	/* FIXME: needs broadcast if global monitor in use. */
+
 	switch(type)
 	{
 	case EWMH_IGNORE_WORKING_AREA:
@@ -1077,20 +1163,29 @@ void EWMH_GetWorkAreaIntersection(
 	}
 	if (is_dynamic)
 	{
-		area_x = Scr.Desktops->ewmh_dyn_working_area.x;
-		area_y = Scr.Desktops->ewmh_dyn_working_area.y;
-		area_w = Scr.Desktops->ewmh_dyn_working_area.width;
-		area_h = Scr.Desktops->ewmh_dyn_working_area.height;
+		area_x = m->Desktops->ewmh_dyn_working_area.x;
+		area_y = m->Desktops->ewmh_dyn_working_area.y;
+		area_w = m->Desktops->ewmh_dyn_working_area.width;
+		area_h = m->Desktops->ewmh_dyn_working_area.height;
 	}
-	nx = max(*x, area_x);
-	ny = max(*y, area_y);
-	nw = min(*x + *w, area_x + area_w) - nx;
-	nh = min(*y + *h, area_y + area_h) - ny;
+	nx = max(*x, area_x + m->si->x);
+	ny = max(*y, area_y + m->si->y);
+	nw = min(*x + *w, area_x + area_w) - nx + m->si->x;
+	nh = min(*y + *h, area_y + area_h) - ny + m->si->y;
+
 	*x = nx;
 	*y = ny;
 	*w = nw;
 	*h = nh;
 
+	if (Scr.bo.do_debug_randr)
+	{
+		fvwm_debug(
+			__func__,
+			"mon: %s finalising: {x: %d, y: %d, w: %d, h: %d}\n",
+			m->si->name, *x, *y, *w, *h);
+	}
+
 	return;
 }
 
@@ -1121,7 +1216,7 @@ float get_intersection(
 }
 
 static
-float ewmh_GetStrutIntersection(
+float ewmh_GetStrutIntersection(struct monitor *m,
 	int x11, int y11, int x12, int y12,
 	int left, int right, int top, int bottom,
 	Bool use_percent)
@@ -1129,62 +1224,66 @@ float ewmh_GetStrutIntersection(
 	float ret = 0;
 	int x21, y21, x22, y22;
 
+	/* FIXME: possibly need to consider using m->si->x/y/w/h */
+
 	/* left */
 	x21 = 0;
 	y21 = 0;
 	x22 = left;
-	y22 = Scr.MyDisplayHeight;
+	y22 = monitor_get_all_heights();
 	ret += get_intersection(
 		x11, y11, x12, y12, x21, y21, x22, y22, use_percent);
 	/* right */
-	x21 = Scr.MyDisplayWidth - right;
+	x21 = monitor_get_all_widths() - right;
 	y21 = 0;
-	x22 = Scr.MyDisplayWidth;
-	y22 = Scr.MyDisplayHeight;
+	x22 = monitor_get_all_widths();
+	y22 = monitor_get_all_heights();
 	ret += get_intersection(
 		x11, y11, x12, y12, x21, y21, x22, y22, use_percent);
 	/* top */
 	x21 = 0;
 	y21 = 0;
-	x22 = Scr.MyDisplayWidth;
+	x22 = monitor_get_all_widths();
 	y22 = top;
 	ret += get_intersection(
 		x11, y11, x12, y12, x21, y21, x22, y22, use_percent);
 	/* bottom */
 	x21 = 0;
-	y21 = Scr.MyDisplayHeight - bottom;
-	x22 = Scr.MyDisplayWidth;
-	y22 = Scr.MyDisplayHeight;
+	y21 = monitor_get_all_heights() - bottom;
+	x22 = monitor_get_all_widths();
+	y22 = monitor_get_all_heights();
 	ret += get_intersection(
 		x11, y11, x12, y12, x21, y21, x22, y22, use_percent);
 
 	return ret;
 }
 
-float EWMH_GetBaseStrutIntersection(
+float EWMH_GetBaseStrutIntersection(struct monitor *m,
 	int x11, int y11, int x12, int y12, Bool use_percent)
 {
-	return ewmh_GetStrutIntersection(
-		x11, y11, x12, y12, ewmhc.BaseStrut.left,
-		ewmhc.BaseStrut.right, ewmhc.BaseStrut.top,
-		ewmhc.BaseStrut.bottom, use_percent);
+	return ewmh_GetStrutIntersection(m,
+		x11, y11, x12, y12, m->ewmhc.BaseStrut.left,
+		m->ewmhc.BaseStrut.right, m->ewmhc.BaseStrut.top,
+		m->ewmhc.BaseStrut.bottom, use_percent);
 }
 
-float EWMH_GetStrutIntersection(
+float EWMH_GetStrutIntersection(struct monitor *m,
 	int x11, int y11, int x12, int y12, Bool use_percent)
 {
 	int left, right, top, bottom;
 
-	left = Scr.Desktops->ewmh_working_area.x;
-	right = Scr.MyDisplayWidth -
-		(Scr.Desktops->ewmh_working_area.x
-		 + Scr.Desktops->ewmh_working_area.width);
-	top = Scr.Desktops->ewmh_working_area.y;
-	bottom = Scr.MyDisplayHeight -
-		(Scr.Desktops->ewmh_working_area.y
-		 + Scr.Desktops->ewmh_working_area.height);
+	/* FIXME: needs broadcast if global monitor in use. */
 
-	return ewmh_GetStrutIntersection(
+	left = m->Desktops->ewmh_working_area.x;
+	right = monitor_get_all_widths() -
+		(m->Desktops->ewmh_working_area.x
+		 + m->Desktops->ewmh_working_area.width);
+	top = m->Desktops->ewmh_working_area.y;
+	bottom = monitor_get_all_heights() -
+		(m->Desktops->ewmh_working_area.y
+		 + m->Desktops->ewmh_working_area.height);
+
+	return ewmh_GetStrutIntersection(m,
 		x11, y11, x12, y12, left, right, top, bottom, use_percent);
 }
 
@@ -1225,18 +1324,21 @@ void EWMH_SetFrameStrut(FvwmWindow *fw)
 /*
  * allowed actions
  */
-Bool ewmh_AllowsYes(EWMH_CMD_ARGS)
+Bool ewmh_AllowsYes(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	return True;
 }
 
-Bool ewmh_AllowsClose(EWMH_CMD_ARGS)
+Bool ewmh_AllowsClose(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	return is_function_allowed(
 		F_CLOSE, NULL, fw, RQORIG_PROGRAM_US, False);
 }
 
-Bool ewmh_AllowsFullScreen(EWMH_CMD_ARGS)
+Bool ewmh_AllowsFullScreen(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	if (
 		!is_function_allowed(
@@ -1252,22 +1354,26 @@ Bool ewmh_AllowsFullScreen(EWMH_CMD_ARGS)
 	return True;
 }
 
-Bool ewmh_AllowsMinimize(EWMH_CMD_ARGS)
+Bool ewmh_AllowsMinimize(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	return is_function_allowed(F_ICONIFY, NULL, fw, RQORIG_PROGRAM_US, False);
 }
 
-Bool ewmh_AllowsMaximize(EWMH_CMD_ARGS)
+Bool ewmh_AllowsMaximize(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	return is_function_allowed(F_MAXIMIZE, NULL, fw, RQORIG_PROGRAM_US, False);
 }
 
-Bool ewmh_AllowsMove(EWMH_CMD_ARGS)
+Bool ewmh_AllowsMove(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	return is_function_allowed(F_MOVE, NULL, fw, RQORIG_PROGRAM_US, False);
 }
 
-Bool ewmh_AllowsResize(EWMH_CMD_ARGS)
+Bool ewmh_AllowsResize(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	return is_function_allowed(F_RESIZE, NULL, fw, RQORIG_PROGRAM_US, False);
 }
@@ -1306,14 +1412,14 @@ void EWMH_SetAllowedActions(FvwmWindow *fw)
  * Window types
  */
 
-int ewmh_HandleDesktop(EWMH_CMD_ARGS)
+int ewmh_HandleDesktop(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	if (Scr.EwmhDesktop != NULL && FW_W(Scr.EwmhDesktop) != FW_W(fw))
 	{
-		fvwm_msg(
-			WARN,"ewmh_HandleDesktop",
-			"A Desktop application (0x%lx) already runs! This"
-			" can cause problems\n", FW_W(Scr.EwmhDesktop));
+		fvwm_debug(__func__,
+			   "A Desktop application (0x%lx) already runs! This"
+			   " can cause problems\n", FW_W(Scr.EwmhDesktop));
 		/* what to do ? */
 	}
 
@@ -1401,14 +1507,16 @@ int ewmh_HandleDesktop(EWMH_CMD_ARGS)
 	return 1;
 }
 
-int ewmh_HandleDialog(EWMH_CMD_ARGS)
+int ewmh_HandleDialog(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	fw->ewmh_window_type = EWMH_WINDOW_TYPE_DIALOG_ID;
 
 	return 0;
 }
 
-int ewmh_HandleDock(EWMH_CMD_ARGS)
+int ewmh_HandleDock(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	fw->ewmh_window_type = EWMH_WINDOW_TYPE_DOCK_ID;
 
@@ -1458,7 +1566,8 @@ int ewmh_HandleDock(EWMH_CMD_ARGS)
 	return 1;
 }
 
-int ewmh_HandleMenu(EWMH_CMD_ARGS)
+int ewmh_HandleMenu(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	fw->ewmh_window_type = EWMH_WINDOW_TYPE_MENU_ID;
 
@@ -1500,14 +1609,16 @@ int ewmh_HandleMenu(EWMH_CMD_ARGS)
 	return 1;
 }
 
-int ewmh_HandleNormal(EWMH_CMD_ARGS)
+int ewmh_HandleNormal(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	fw->ewmh_window_type = EWMH_WINDOW_TYPE_NORMAL_ID;
 
 	return 0;
 }
 
-int ewmh_HandleToolBar(EWMH_CMD_ARGS)
+int ewmh_HandleToolBar(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	fw->ewmh_window_type = EWMH_WINDOW_TYPE_TOOLBAR_ID;
 
@@ -1533,7 +1644,8 @@ int ewmh_HandleToolBar(EWMH_CMD_ARGS)
 	return 1;
 }
 
-int ewmh_HandleNotification(EWMH_CMD_ARGS)
+int ewmh_HandleNotification(
+	FvwmWindow *fw, XEvent *ev, window_style *style, unsigned long any)
 {
 	/* fw->ewmh_window_type is generally used by FvwmIdent, but for this
 	 * EWMH type it is not used.  Reporting on unmanaged windows with
@@ -1671,7 +1783,7 @@ static void ewmh_check_wm_pid(FvwmWindow *fw)
 /* see also EWMH_WMName and EWMH_WMIconName in add_window */
 void EWMH_WindowInit(FvwmWindow *fw)
 {
-	/*EWMH_DLOG("Init window 0x%lx",FW_W(fw));*/
+	EWMH_DLOG("Init window 0x%lx", FW_W(fw));
 	EWMH_SetWMState(fw, False);
 	EWMH_SetWMDesktop(fw);
 	EWMH_SetAllowedActions(fw);
@@ -1689,7 +1801,7 @@ void EWMH_WindowInit(FvwmWindow *fw)
 	}
 	ewmh_WMIcon(fw, NULL, NULL, 0);
 	ewmh_check_wm_pid(fw);
-	/*EWMH_DLOG("window 0x%lx initialised",FW_W(fw));*/
+	EWMH_DLOG("window 0x%lx initialised", FW_W(fw));
 
 	return;
 }
@@ -1727,9 +1839,9 @@ void EWMH_DestroyWindow(FvwmWindow *fw)
 	{
 		Scr.EwmhDesktop = NULL;
 	}
-	if (fw->Desk >= ewmhc.NumberOfDesktops)
+	if (fw->Desk >= fw->m->ewmhc.NumberOfDesktops)
 	{
-		ewmhc.NeedsToCheckDesk = True;
+		fw->m->ewmhc.NeedsToCheckDesk = True;
 	}
 
 	return;
@@ -1738,14 +1850,19 @@ void EWMH_DestroyWindow(FvwmWindow *fw)
 /* a window has been destroyed (unmap/reparent/destroy) */
 void EWMH_WindowDestroyed(void)
 {
-	EWMH_SetClientList();
-	EWMH_SetClientListStacking();
-	if (ewmhc.NeedsToCheckDesk)
+	struct monitor	*m = monitor_get_current();
+
+	if (m == NULL)
+		return;
+
+	EWMH_SetClientList(m);
+	EWMH_SetClientListStacking(m);
+	if (m->ewmhc.NeedsToCheckDesk)
 	{
-		EWMH_SetNumberOfDesktops();
+		EWMH_SetNumberOfDesktops(m);
 	}
-	ewmh_ComputeAndSetWorkArea();
-	ewmh_HandleDynamicWorkArea();
+	ewmh_ComputeAndSetWorkArea(m);
+	ewmh_HandleDynamicWorkArea(m);
 
 	return;
 }
@@ -1778,7 +1895,7 @@ void set_net_supported(int l)
 	Atom *supported;
 	int i, k = 0;
 
-	supported = (Atom *)safemalloc(l*sizeof(Atom));
+	supported = fxmalloc(l*sizeof(Atom));
 	for(i=0; i < NUMBER_OF_ATOM_LISTS; i++)
 	{
 		ewmh_atom *list = atom_list[i].list;
@@ -1807,13 +1924,13 @@ void clean_up(void)
 	return;
 }
 
-void EWMH_Init(void)
+void EWMH_Init(struct monitor *m)
 {
 	int i;
 	int supported_count = 0;
 	long val;
 	XTextProperty text;
-	unsigned char utf_name[4];
+	unsigned char utf_name[5];
 	char *names[1];
 	XClassHint classhints;
 
@@ -1836,9 +1953,9 @@ void EWMH_Init(void)
 		Scr.NoFocusWin, "_NET_SUPPORTING_WM_CHECK",
 		EWMH_ATOM_LIST_FVWM_ROOT, (unsigned char *)&val, 1);
 
-	names[0] = "fvwm";
-	classhints.res_name= "fvwm";
-	classhints.res_class= "FVWM";
+	names[0] = "fvwm3";
+	classhints.res_name= "fvwm3";
+	classhints.res_class= "FVWM3";
 
 	XSetClassHint(dpy, Scr.NoFocusWin, &classhints);
 	if (XStringListToTextProperty(names, 1, &text))
@@ -1852,6 +1969,7 @@ void EWMH_Init(void)
 	utf_name[1] = 0x56;
 	utf_name[2] = 0x57;
 	utf_name[3] = 0x4D;
+	utf_name[4] = 0x3;
 
 	ewmh_ChangeProperty(
 		Scr.NoFocusWin, "_NET_WM_NAME", EWMH_ATOM_LIST_PROPERTY_NOTIFY,
@@ -1859,14 +1977,14 @@ void EWMH_Init(void)
 
 	clean_up();
 
-	EWMH_SetDesktopNames();
-	EWMH_SetCurrentDesktop();
-	EWMH_SetNumberOfDesktops();
-	EWMH_SetDesktopViewPort();
-	EWMH_SetDesktopGeometry();
-	EWMH_SetClientList();
-	EWMH_SetClientListStacking();
-	ewmh_ComputeAndSetWorkArea();
+	EWMH_SetDesktopNames(m);
+	EWMH_SetCurrentDesktop(m);
+	EWMH_SetNumberOfDesktops(m);
+	EWMH_SetDesktopViewPort(m);
+	EWMH_SetDesktopGeometry(m);
+	EWMH_SetClientList(m);
+	EWMH_SetClientListStacking(m);
+	ewmh_ComputeAndSetWorkArea(m);
 
 	return;
 }
@@ -1886,44 +2004,6 @@ void EWMH_ExitStuff(void)
 	return;
 }
 
-#ifdef EWMH_DEBUG
-void EWMH_DLOG(char *msg, ...)
-{
-	va_list args;
-	clock_t time_val, time_taken;
-	static clock_t start_time = 0;
-	static clock_t prev_time = 0;
-	struct tms not_used_tms;
-	char buffer[200]; /* oversized */
-	time_t mytime;
-	struct tm *t_ptr;
-
-	time(&mytime);
-	t_ptr = localtime(&mytime);
-	if (start_time == 0)
-	{
-		/* get clock ticks */
-		prev_time = start_time = (unsigned int)times(&not_used_tms);
-	}
-	/* get clock ticks */
-	time_val = (unsigned int)times(&not_used_tms);
-	time_taken = time_val - prev_time;
-	prev_time = time_val;
-	sprintf(
-		buffer, "%.2d:%.2d:%.2d %6ld",
-		t_ptr->tm_hour, t_ptr->tm_min, t_ptr->tm_sec, time_taken);
-
-	fprintf(stderr, "EWMH DEBUG: ");
-	va_start(args,msg);
-	vfprintf(stderr, msg, args);
-	va_end(args);
-	fprintf(stderr, "\n");
-	fprintf(stderr, "            [time]: %s\n",buffer);
-
-	return;
-}
-#endif
-
 void EWMH_fullscreen(FvwmWindow *fw)
 {
 	fscreen_scr_arg fscr;
@@ -1958,7 +2038,7 @@ void EWMH_fullscreen(FvwmWindow *fw)
 	{
 		fw->fullscreen.is_iconified = 1;
 		execute_function_override_window(
-			NULL, NULL, "Iconify off", 0, fw);
+			NULL, NULL, "Iconify off", NULL, 0, fw);
 	}
 	if (IS_SHADED(fw))
 	{
@@ -1967,7 +2047,7 @@ void EWMH_fullscreen(FvwmWindow *fw)
 		fw->fullscreen.is_shaded = 1;
 		fw->shade_anim_steps = 0;
 		execute_function_override_window(
-			NULL, NULL, "WindowShade off", 0, fw);
+			NULL, NULL, "WindowShade off", NULL, 0, fw);
 		fw->shade_anim_steps = sas;
 	}
 	SET_EWMH_FULLSCREEN(fw,True);
@@ -2001,7 +2081,7 @@ void EWMH_fullscreen(FvwmWindow *fw)
 	if (cmd[0] != 0)
 	{
 		SET_DISABLE_CONSTRAIN_SIZE_FULLSCREEN(fw, 1);
-		execute_function_override_window(NULL, NULL, cmd, 0, fw);
+		execute_function_override_window(NULL, NULL, cmd, NULL, 0, fw);
 		SET_DISABLE_CONSTRAIN_SIZE_FULLSCREEN(fw, 0);
 	}
 
