@@ -28,7 +28,9 @@
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
+#include "cmdparser.h"
 #include "functions.h"
+#include "expand.h"
 #include "misc.h"
 #include "move_resize.h"
 #include "screen.h"
@@ -505,27 +507,38 @@ static signed int expand_vars_extended(
 		}
 
 		if (strcmp(rest, "primary") == 0) {
-			struct monitor *m = monitor_by_primary();
+			struct monitor *m2 = monitor_by_primary();
 
-			if (m != NULL)
-				string = m->si->name;
+			if (m2 != NULL)
+				string = m2->si->name;
 			should_quote = False;
 			goto GOT_STRING;
 		}
 
 		if (strcmp(rest, "current") == 0) {
-			struct monitor *m = monitor_get_current();
+			struct monitor *m2 = monitor_get_current();
 
 			should_quote = False;
-			string = m->si->name;
+			string = m2->si->name;
+
+			goto GOT_STRING;
+		}
+
+		if (strcmp(rest, "prev") == 0) {
+			struct monitor *m2 = monitor_get_prev();
+
+			should_quote = False;
+			string = (m2 != NULL) ? m2->si->name : "";
 
 			goto GOT_STRING;
 		}
 
 		/* We could be left with "<NAME>.?" */
 		char		*m_name = NULL;
-		struct monitor  *mon;
+		struct monitor  *mon2;
 		char		*rest_s;
+		char *p_free;
+		int got_string;
 
 		/* The first word is the monitor name:
 		 *
@@ -534,12 +547,15 @@ static signed int expand_vars_extended(
 		 * so scan for the first full-stop.
 		 */
 		rest_s = fxstrdup(rest);
+		p_free = rest_s;
+		got_string = 0;
 		while ((m_name = strsep(&rest_s, ".")) != NULL) {
-			mon = monitor_resolve_name(m_name);
-			if (m_name == NULL)
-				return -1;
-			if (strcmp(mon->si->name, m_name) == 1)
+			mon2 = monitor_resolve_name(m_name);
+			if (mon2 == NULL)
+			{
+				free(p_free);
 				return -1;
+			}
 
 			/* Skip over the monitor name. */
 			rest += strlen(m_name) + 1;
@@ -547,74 +563,90 @@ static signed int expand_vars_extended(
 			/* Match remainder to valid fields. */
 			if (strcmp(rest, "x") == 0) {
 				is_numeric = True;
-				val = mon->si->x;
-				goto GOT_STRING;
+				val = mon2->si->x;
+				got_string = 1;
+				break;
 			}
 
 			if (strcmp(rest, "y") == 0) {
 				is_numeric = True;
-				val = mon->si->y;
-				goto GOT_STRING;
+				val = mon2->si->y;
+				got_string = 1;
+				break;
 			}
 
 			if (strcmp(rest, "width") == 0) {
 				is_numeric = True;
-				val = mon->si->w;
-				goto GOT_STRING;
+				val = mon2->si->w;
+				got_string = 1;
+				break;
 			}
 
 			if (strcmp(rest, "height") == 0) {
 				is_numeric = True;
-				val = mon->si->h;
-				goto GOT_STRING;
+				val = mon2->si->h;
+				got_string = 1;
+				break;
 			}
 
 			if (strcmp(rest, "output") == 0) {
 				is_numeric = True;
-				val = (int)mon->si->rr_output;
-				goto GOT_STRING;
+				val = (int)mon2->si->rr_output;
+				got_string = 1;
+				break;
 			}
 
 			if (strcmp(rest, "desk") == 0) {
 				is_numeric = True;
-				val = mon->virtual_scr.CurrentDesk;
-				goto GOT_STRING;
+				val = mon2->virtual_scr.CurrentDesk;
+				got_string = 1;
+				break;
 			}
 
 			if (strcmp(rest, "pagex") == 0) {
 				is_numeric = True;
-				val = (int)(mon->virtual_scr.Vx /
+				val = (int)(mon2->virtual_scr.Vx /
 					monitor_get_all_widths());
-				goto GOT_STRING;
+				got_string = 1;
+				break;
 			}
 
 			if (strcmp(rest, "pagey") == 0) {
 				is_numeric = True;
-				val = (int)(mon->virtual_scr.Vy /
+				val = (int)(mon2->virtual_scr.Vy /
 					monitor_get_all_heights());
-				goto GOT_STRING;
+				got_string = 1;
+				break;
 			}
 
 			if (strcmp(rest, "prev_desk") == 0) {
 				is_numeric = True;
 				val = m->virtual_scr.prev_desk_and_page_desk;
-				goto GOT_STRING;
+				got_string = 1;
+				break;
 			}
 
 			if (strcmp(rest, "prev_pagex") == 0) {
 				is_numeric = True;
 				val = (int)(m->virtual_scr.prev_page_x /
 					monitor_get_all_widths());
-				goto GOT_STRING;
+				got_string = 1;
+				break;
 			}
 
 			if (strcmp(rest, "prev_pagey") == 0) {
 				is_numeric = True;
 				val = (int)(m->virtual_scr.prev_page_y /
 					monitor_get_all_heights());
-				goto GOT_STRING;
+				got_string = 1;
+				break;
 			}
 		}
+		free(p_free);
+		if (got_string)
+		{
+			goto GOT_STRING;
+		}
 		break;
 	}
 	default:
@@ -1140,7 +1172,7 @@ static signed int expand_vars_extended(
 		break;
 	case VAR_DEBUG_LOG_STATE:
 		is_numeric = True;
-		val = log_get_level();
+		val = lib_log_level;
 		break;
 	default:
 		/* unknown variable - try to find it in the environment */
@@ -1206,7 +1238,7 @@ GOT_STRING:
 /* ---------------------------- interface functions ------------------------ */
 
 char *expand_vars(
-	char *input, char *arguments[], Bool addto, Bool ismod,
+	char *input, cmdparser_context_t *pc, Bool addto, Bool ismod,
 	cond_rc_t *cond_rc, const exec_context_t *exc)
 {
 	int l, i, l2, n, k, j, m;
@@ -1218,6 +1250,7 @@ char *expand_vars(
 	Bool is_string = False;
 	FvwmWindow *fw = exc->w.fw;
 	struct monitor	*mon = fw ? fw->m : monitor_get_current();
+	char *args_string = (pc) ? pc->all_pos_args_string : NULL;
 
 	l = strlen(input);
 	l2 = l;
@@ -1280,12 +1313,11 @@ char *expand_vars(
 					if (name_has_dollar)
 					{
 						var = expand_vars(
-							var, arguments, addto,
-							ismod, cond_rc, exc);
+							var, pc, addto, ismod,
+							cond_rc, exc);
 					}
 					xlen = expand_args_extended(
-						var, arguments ? arguments[0] :
-						NULL, NULL);
+						var, args_string, NULL);
 					if (xlen < 0)
 					{
 						xlen = expand_vars_extended(
@@ -1315,20 +1347,26 @@ char *expand_vars(
 			case '8':
 			case '9':
 			case '*':
+			{
+				char *s;
+
 				if (input[i + 1] == '*')
 				{
-					n = 0;
+					s = args_string;
 				}
 				else
 				{
-					n = input[i + 1] - '0' + 1;
+					n = input[i + 1] - '0';
+					s = (pc) ? pc->pos_arg_tokens[n] : 0;
 				}
-				if (arguments[n] != NULL)
+				n = input[i + 1] - '0';
+				if (s != NULL)
 				{
-					l2 += strlen(arguments[n]) - 2;
+					l2 += strlen(s) - 2;
 					i++;
 				}
 				break;
+			}
 			case '.':
 				string = get_current_read_dir();
 				break;
@@ -1459,13 +1497,11 @@ char *expand_vars(
 					if (name_has_dollar)
 					{
 						var = expand_vars(
-							var, arguments,	addto,
-							ismod, cond_rc,	exc);
+							var, pc, addto, ismod,
+							cond_rc, exc);
 					}
 					xlen = expand_args_extended(
-						var, arguments ?
-						arguments[0] : NULL,
-						&out[j]);
+						var, args_string, &out[j]);
 					if (xlen < 0)
 					{
 						xlen = expand_vars_extended(
@@ -1509,19 +1545,23 @@ char *expand_vars(
 			case '8':
 			case '9':
 			case '*':
+			{
+				char *s;
+
 				if (input[i + 1] == '*')
 				{
-					n = 0;
+					s = args_string;
 				}
 				else
 				{
-					n = input[i + 1] - '0' + 1;
+					n = input[i + 1] - '0';
+					s = (pc) ? pc->pos_arg_tokens[n] : NULL;
 				}
-				if (arguments[n] != NULL)
+				if (s != NULL)
 				{
-					for (k = 0; arguments[n][k]; k++)
+					for (k = 0; s[k]; k++)
 					{
-						out[j++] = arguments[n][k];
+						out[j++] = s[k];
 					}
 					i++;
 				}
@@ -1534,6 +1574,7 @@ char *expand_vars(
 					i++;
 				}
 				break;
+			}
 			case '.':
 				string = get_current_read_dir();
 				is_string = True;
