@@ -10,8 +10,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 /* ---------------------------- included header files ---------------------- */
@@ -72,11 +71,9 @@ const func_t func_table[] =
 		FUNC_DECOR, 0),
 	/* - Add to a title style (see TitleStyle) */
 
-#ifdef USEDECOR
 	CMD_ENT("addtodecor", CMD_AddToDecor, F_ADD_DECOR, 0, 0),
 	/* - Add a decor definition (will be obsolete) */
 
-#endif /* USEDECOR */
 	CMD_ENT("addtofunc", CMD_AddToFunc, F_ADDFUNC, FUNC_ADD_TO, 0),
 	/* - Add a function definition */
 
@@ -87,7 +84,7 @@ const func_t func_table[] =
 	/* - Operate on all windows matching the given condition */
 
 	CMD_ENT("animatedmove", CMD_AnimatedMove, F_ANIMATED_MOVE,
-		FUNC_NEEDS_WINDOW, CRS_MOVE),
+		FUNC_NEEDS_WINDOW | FUNC_IS_MOVE_TYPE, CRS_MOVE),
 	/* - Like Move, but uses animation to move windows */
 
 	CMD_ENT("any", CMD_Any, F_ANY, 0, 0),
@@ -116,12 +113,10 @@ const func_t func_table[] =
 		FUNC_DECOR, 0),
 	/* - Define a window button look (will be reworked) */
 
-#ifdef USEDECOR
 	CMD_ENT("changedecor", CMD_ChangeDecor, F_CHANGE_DECOR,
 		FUNC_NEEDS_WINDOW, CRS_SELECT),
 	/* - Attach decor to a window (will be obsolete) */
 
-#endif /* USEDECOR */
 	CMD_ENT("changemenustyle", CMD_ChangeMenuStyle, F_CHANGE_MENUSTYLE,
 		0, 0),
 	/* - Attach menu style to a menu (see MenuStyle) */
@@ -183,6 +178,10 @@ const func_t func_table[] =
 	CMD_ENT("desk", CMD_Desk, F_GOTO_DESK, 0, 0),
 	/* - (obsolete, use GotoDesk instead) */
 
+	CMD_ENT("desktopconfiguration", CMD_DesktopConfiguration,
+		F_DESKTOP_CONFIGURATION, 0, 0),
+	/* - Defines how desktops are managed. */
+
 	CMD_ENT("desktopname", CMD_DesktopName, F_DESKTOP_NAME, 0, 0),
 	/* - Define the desktop names used in WindowList, modules */
 
@@ -193,11 +192,9 @@ const func_t func_table[] =
 		FUNC_NEEDS_WINDOW, CRS_DESTROY),
 	/* - Kill a window without any warning to an application */
 
-#ifdef USEDECOR
 	CMD_ENT("destroydecor", CMD_DestroyDecor, F_DESTROY_DECOR, 0, 0),
 	/* - Delete decor defined by AddToDecor (will be obsolete) */
 
-#endif /* USEDECOR */
 	CMD_ENT("destroyfunc", CMD_DestroyFunc, F_DESTROY_FUNCTION, 0, 0),
 	/* - Delete function defined using AddToFunc */
 
@@ -285,14 +282,8 @@ const func_t func_table[] =
 	CMD_ENT("function", CMD_Function, F_FUNCTION, 0, 0),
 	/* Function - Execute a user defined function, see AddToFunc */
 
-	CMD_ENT("globalopts", CMD_GlobalOpts, F_GLOBAL_OPTS, 0, 0),
-	/* - (obsolete, use corresponding Style * instead) */
-
-	CMD_ENT("gnomebutton", CMD_GnomeButton, F_MOUSE, 0, 0),
-	/* - Pass mouse button presses on root to GNOME program */
-
-	CMD_ENT("gnomeshowdesks", CMD_GnomeShowDesks, F_GOTO_DESK, 0, 0),
-	/* - Limit GNOME pager to the number of desks */
+	CMD_ENT("geometrywindow", CMD_GeometryWindow, F_GEOMWINDOW, 0, 0),
+	/* - Configuration settings for position/size window */
 
 	CMD_ENT("gotodesk", CMD_GotoDesk, F_GOTO_DESK, 0, 0),
 	/* - Switch viewport to another desk same page */
@@ -305,7 +296,7 @@ const func_t func_table[] =
 
 	CMD_ENT("hidegeometrywindow", CMD_HideGeometryWindow,
 		F_HIDEGEOMWINDOW, 0, 0),
-	/* - Hide/show the position/size window */
+	/* - (obsolete, use GeometryWindow Hide instead) */
 
 	CMD_ENT("hilightcolor", CMD_HilightColor, F_HICOLOR, 0, 0),
 	/* - (obsolete, use Style * HighlightFore/Back) */
@@ -331,10 +322,13 @@ const func_t func_table[] =
 	/* - Directories to search for images */
 
 	CMD_ENT("infostoreadd", CMD_InfoStoreAdd, F_INFOSTOREADD, 0, 0),
-	/* Adds an entry (key/value pairs) to the infostore. */
+	/* - Adds an entry (key/value pairs) to the infostore */
+
+	CMD_ENT("infostoreclear", CMD_InfoStoreClear, F_INFOSTORECLEAR, 0, 0),
+	/* - Clears all entries from the infostore */
 
 	CMD_ENT("infostoreremove", CMD_InfoStoreRemove, F_INFOSTOREREMOVE, 0, 0),
-	/* Removes an entry from the infostore. */
+	/* - Removes an entry from the infostore */
 
 	CMD_ENT(PRE_KEEPRC, CMD_KeepRc, F_KEEPRC, 0, 0),
 	/* KeepRc - Do not modify the previous command return code */
@@ -387,7 +381,7 @@ const func_t func_table[] =
 	/* - Bind or unbind a mouse button press to an fvwm action */
 
 	CMD_ENT("move", CMD_Move, F_MOVE,
-		FUNC_NEEDS_WINDOW, CRS_MOVE),
+		FUNC_NEEDS_WINDOW | FUNC_IS_MOVE_TYPE, CRS_MOVE),
 	/* - Move a window */
 
 	CMD_ENT("movethreshold", CMD_MoveThreshold, F_MOVE_THRESHOLD, 0, 0),
@@ -403,7 +397,7 @@ const func_t func_table[] =
 
 	CMD_ENT("movetoscreen", CMD_MoveToScreen, F_MOVE_TO_SCREEN,
 		FUNC_NEEDS_WINDOW, CRS_SELECT),
-	/* - Move a window to another Xinerama screen */
+	/* - Move a window to another screen */
 
 	CMD_ENT("next", CMD_Next, F_NEXT, 0, 0),
 	/* - Operate on the next window matching conditions */
@@ -411,7 +405,7 @@ const func_t func_table[] =
 	CMD_ENT("none", CMD_None, F_NONE, 0, 0),
 	/* - Perform command if no window matches conditions */
 
-	CMD_ENT("nop", CMD_Nop, F_NOP, FUNC_DONT_REPEAT, 0),
+	CMD_ENT("nop", CMD_Nop, F_NOP, 0, 0),
 	/* - Do nothing (used internally) */
 
 	CMD_ENT("nowindow", CMD_NoWindow, F_NOP, 0, 0),
@@ -474,13 +468,6 @@ const func_t func_table[] =
 	CMD_ENT("readwritecolors", CMD_ReadWriteColors, F_NOP, 0, 0),
 	/* - Used for colorset speed hacks (will be removed?) */
 
-	CMD_ENT("recapture", CMD_Recapture, F_RECAPTURE, 0, 0),
-	/* - Reapply styles to all windows (will be obsolete) */
-
-	CMD_ENT("recapturewindow", CMD_RecaptureWindow, F_RECAPTURE_WINDOW,
-		FUNC_NEEDS_WINDOW, CRS_SELECT),
-	/* - Reapply styles to one window (will be obsolete) */
-
 	CMD_ENT("refresh", CMD_Refresh, F_REFRESH, 0, 0),
 	/* - Cause all windows to redraw themselves */
 
@@ -488,23 +475,21 @@ const func_t func_table[] =
 		FUNC_NEEDS_WINDOW, CRS_SELECT),
 	/* - Cause one window to redraw itself */
 
-	CMD_ENT(PRE_REPEAT, CMD_Repeat, F_REPEAT, FUNC_DONT_REPEAT, 0),
-	/* - Repeat (very unreliably) the last command, don't use */
-
 	CMD_ENT("resize", CMD_Resize, F_RESIZE,
-		FUNC_NEEDS_WINDOW, CRS_RESIZE),
+		FUNC_NEEDS_WINDOW | FUNC_IS_MOVE_TYPE, CRS_RESIZE),
 	/* - Cause a window to be resized */
 
 	CMD_ENT("resizemaximize", CMD_ResizeMaximize, F_RESIZE_MAXIMIZE,
-		FUNC_NEEDS_WINDOW, CRS_RESIZE),
+		FUNC_NEEDS_WINDOW | FUNC_IS_MOVE_TYPE, CRS_RESIZE),
 	/* - Resize a window and mark window as maximized */
 
 	CMD_ENT("resizemove", CMD_ResizeMove, F_RESIZEMOVE,
-		FUNC_NEEDS_WINDOW, CRS_RESIZE),
+		FUNC_NEEDS_WINDOW | FUNC_IS_MOVE_TYPE, CRS_RESIZE),
 	/* - Resize and move in one operation */
 
 	CMD_ENT("resizemovemaximize", CMD_ResizeMoveMaximize,
-		F_RESIZEMOVE_MAXIMIZE, FUNC_NEEDS_WINDOW, CRS_RESIZE),
+		F_RESIZEMOVE_MAXIMIZE, FUNC_NEEDS_WINDOW | FUNC_IS_MOVE_TYPE,
+		CRS_RESIZE),
 	/* - Resize and move in one operation and mark maximized */
 
 	CMD_ENT("restacktransients", CMD_RestackTransients, F_RESTACKTRANSIENTS,
@@ -530,31 +515,27 @@ const func_t func_table[] =
 	CMD_ENT("scroll", CMD_Scroll, F_SCROLL, 0, 0),
 	/* - Scroll the desktop viewport */
 
-	CMD_ENT("send_configinfo", CMD_Send_ConfigInfo, F_CONFIG_LIST,
-		FUNC_DONT_REPEAT, 0),
+	CMD_ENT("send_configinfo", CMD_Send_ConfigInfo, F_CONFIG_LIST, 0, 0),
 	/* - Internal, used for module communication */
 
-	CMD_ENT("send_reply", CMD_Send_Reply, F_SEND_REPLY,
-		FUNC_DONT_REPEAT, 0),
+	CMD_ENT("send_reply", CMD_Send_Reply, F_SEND_REPLY, 0, 0),
 	/* - Internal, used for module communication */
 
 	CMD_ENT("send_windowlist", CMD_Send_WindowList, F_SEND_WINDOW_LIST,
-		FUNC_DONT_REPEAT, 0),
+		0, 0),
 	/* - Internal, used for module communication */
 
-	CMD_ENT("sendtomodule", CMD_SendToModule, F_SEND_STRING,
-		FUNC_DONT_REPEAT, 0),
+	CMD_ENT("sendtomodule", CMD_SendToModule, F_SEND_STRING, 0, 0),
 	/* - Send a string (action) to a module */
 
-	CMD_ENT("set_mask", CMD_set_mask, F_SET_MASK, FUNC_DONT_REPEAT, 0),
+	CMD_ENT("set_mask", CMD_set_mask, F_SET_MASK, 0, 0),
 	/* - Internal, used for module communication */
 
 	CMD_ENT("set_nograb_mask", CMD_set_nograb_mask, F_SET_NOGRAB_MASK,
-		FUNC_DONT_REPEAT, 0),
+		0, 0),
 	/* - Internal, used for module communication */
 
-	CMD_ENT("set_sync_mask", CMD_set_sync_mask, F_SET_SYNC_MASK,
-		FUNC_DONT_REPEAT, 0),
+	CMD_ENT("set_sync_mask", CMD_set_sync_mask, F_SET_SYNC_MASK, 0, 0),
 	/* - Internal, used for module communication */
 
 	CMD_ENT("setanimation", CMD_SetAnimation, F_SET_ANIMATION, 0, 0),
@@ -576,6 +557,9 @@ const func_t func_table[] =
 		FUNC_NEEDS_WINDOW, CRS_SELECT),
 	/* - Control user defined window states */
 
+	CMD_ENT("status", CMD_Status, F_STATUS, 0, 0),
+	/* - Sets if we send status. */
+
 	CMD_ENT("stick", CMD_Stick, F_STICK,
 		FUNC_NEEDS_WINDOW, CRS_SELECT),
 	/* - Change window stickyness */
@@ -588,14 +572,6 @@ const func_t func_table[] =
 		FUNC_NEEDS_WINDOW, CRS_SELECT),
 	/* - Change window stickyness on a page basis */
 
-#ifdef HAVE_STROKE
-	CMD_ENT("stroke", CMD_Stroke, F_STROKE, 0, 0),
-	/* - Bind a stroke to an fvwm action */
-
-	CMD_ENT("strokefunc", CMD_StrokeFunc, F_STROKE_FUNC, 0, 0),
-	/* - Record stroke and execute corresponding stroke action */
-
-#endif /* HAVE_STROKE */
 	CMD_ENT("style", CMD_Style, F_STYLE, 0, 0),
 	/* - Set attributes of windows that match a pattern */
 
@@ -655,24 +631,6 @@ const func_t func_table[] =
 		FUNC_NEEDS_WINDOW, CRS_SELECT),
 	/* - Set styles on the selected window */
 
-	CMD_ENT("xinerama", CMD_Xinerama, F_XINERAMA, 0, 0),
-	/* - Control Xinerama support */
-
-	CMD_ENT("xineramaprimaryscreen", CMD_XineramaPrimaryScreen,
-		F_XINERAMAPRIMARYSCREEN, 0, 0),
-	/* - Identify Xinerama primary screen */
-
-	CMD_ENT("xineramasls", CMD_XineramaSls, F_XINERAMASLS, 0, 0),
-	/* - Control single logical screen support */
-
-	CMD_ENT("xineramaslsscreens", CMD_XineramaSlsScreens,
-		F_XINERAMASLSSCREENS, 0, 0),
-	/* - Configure layout for single logical screen */
-
-	CMD_ENT("xineramaslssize", CMD_XineramaSlsSize, F_XINERAMASLSSIZE,
-		0, 0),
-	/* - Configure layout for single logical screen sizes */
-
 	CMD_ENT("xorpixmap", CMD_XorPixmap, F_XOR, 0, 0),
 	/* - Use a pixmap for move/resize rubber-band */
 
@@ -687,3 +645,56 @@ const func_t func_table[] =
 
 	{ "", 0, 0, 0, 0 }
 };
+
+/* ---------------------------- local functions ---------------------------- */
+
+/*
+ * do binary search on func list
+ */
+static int func_comp(const void *a, const void *b)
+{
+	return (strcmp((char *)a, ((func_t *)b)->keyword));
+}
+
+/* ---------------------------- interface functions ------------------------ */
+
+const func_t *find_builtin_function(const char *func)
+{
+	static int nfuncs = 0;
+	func_t *ret_func;
+	char *temp;
+	char *s;
+
+	if (!func || func[0] == 0)
+	{
+		return NULL;
+	}
+
+	/* since a lot of lines in a typical rc are probably menu/func
+	 * continues: */
+	if (func[0]=='+' || (func[0] == ' ' && func[1] == '+'))
+	{
+		return &(func_table[0]);
+	}
+
+	temp = fxstrdup(func);
+	for (s = temp; *s != 0; s++)
+	{
+		if (isupper(*s))
+		{
+			*s = tolower(*s);
+		}
+	}
+	if (nfuncs == 0)
+	{
+		for ( ; (func_table[nfuncs]).action != NULL; nfuncs++)
+		{
+			/* nothing to do here */
+		}
+	}
+	ret_func = (func_t *)bsearch(
+		temp, func_table, nfuncs, sizeof(func_t), func_comp);
+	free(temp);
+
+	return ret_func;
+}
