@@ -149,7 +149,6 @@ Window JunkRoot, JunkChild;             /* junk window */
 int JunkWidth, JunkHeight, JunkBW, JunkDepth;
 unsigned int JunkMask;
 
-Bool debugging = False;
 Bool debugging_stack_ring = False;
 
 Window bad_window = None;
@@ -497,14 +496,14 @@ static int parse_command_args(
  *
  */
 static
-char *get_display_name(char *display_name, int screen_num)
+char *get_display_name(char *disp_name, int screen_num)
 {
 	char *msg;
 	char *new_dn;
 	char *cp;
 	char string_screen_num[32];
 
-	CopyString(&msg, display_name);
+	CopyString(&msg, disp_name);
 	cp = strchr(msg, ':');
 	if (cp != NULL)
 	{
@@ -551,12 +550,12 @@ void Done(int restart, char *command)
 
 		char *action;
 
-		xasprintf(&action, "Function ", exit_func_name);
+		xasprintf(&action, "Function %s", exit_func_name);
 
 		ecc.type = restart ? EXCT_TORESTART : EXCT_QUIT;
 		ecc.w.wcontext = C_ROOT;
 		exc = exc_create_context(&ecc, ECC_TYPE | ECC_WCONTEXT);
-		execute_function(NULL, exc, action, 0);
+		execute_function(NULL, exc, action, NULL, 0);
 		exc_destroy_context(exc);
 		free(action);
 	}
@@ -598,7 +597,7 @@ void Done(int restart, char *command)
 			do_preserve_state);
 
 		/* RBW - 06/08/1999 - without this, windows will wander to
-		 * other pages on a Restart/Recapture because Restart gets the
+		 * other pages on a Restart because Restart gets the
 		 * window position information out of sync. There may be a
 		 * better way to do this (i.e., adjust the Restart code), but
 		 * this works for now. */
@@ -662,30 +661,6 @@ void Done(int restart, char *command)
 			}
 			else
 			{
-				char *str = NULL;
-
-				/* Warn against an old 'Restart fvwm2' usage */
-				if (n == 1 && strcmp(my_argv[0], "fvwm2") == 0)
-				{
-					str = "fvwm2";
-				}
-				/* If we are at it, warn against a 'Restart
-				 * fvwm' usage as well */
-				else if (n == 1 &&
-					 strcmp(my_argv[0], "fvwm") == 0)
-				{
-					str = "fvwm";
-				}
-				if (str)
-				{
-					fvwm_debug(__func__,
-						   "`Restart %s' might not do"
-						   " what you want, see the man"
-						   " page.\n\tUse Restart without"
-						   " parameters if you mean to"
-						   " restart the same WM.",
-						   str);
-				}
 				execvp(my_argv[0], my_argv);
 				fvwm_debug(__func__, "Call of '%s' failed!"
 					   " (restarting '%s' instead)",
@@ -1242,10 +1217,12 @@ static void usage(int is_verbose)
 		   " -I vis-id:    use visual <vis-id>\n"
 		   " -l colors:    try to use no more than <colors> colors\n"
 		   " -L:           strict color limit\n"
+                   " -o logfile:   output file or '-' for stderr\n"
 		   " -P:           visual palette\n"
 		   " -r:           replace running window manager\n"
 		   " -s [screen]:  manage a single screen\n"
 		   " -S:           static palette\n"
+		   " -v:           verbose log output\n"
 		   " -V:           print version information\n"
 		);
 	fprintf(stderr, "Try 'man %s' for more information.\n",
@@ -1401,7 +1378,7 @@ static void SetRCDefaults(void)
 		exc = exc_create_context(&ecc, ECC_TYPE | ECC_WCONTEXT);
 		xasprintf(&cmd, "%s%s%s", defaults[i][0], defaults[i][1],
 			defaults[i][2]);
-		execute_function(NULL, exc, cmd, 0);
+		execute_function(NULL, exc, cmd, NULL, 0);
 		free(cmd);
 		exc_destroy_context(exc);
 	}
@@ -1410,7 +1387,7 @@ static void SetRCDefaults(void)
 	return;
 }
 
-static int CatchRedirectError(Display *dpy, XErrorEvent *event)
+static int CatchRedirectError(Display *disp, XErrorEvent *event)
 {
 	fvwm_debug(__func__, "another WM is running");
 	exit(1);
@@ -1420,7 +1397,7 @@ static int CatchRedirectError(Display *dpy, XErrorEvent *event)
 }
 
 /* CatchFatal - Shuts down if the server connection is lost */
-static int CatchFatal(Display *dpy)
+static int CatchFatal(Display *disp)
 {
 	/* No action is taken because usually this action is caused by someone
 	   using "xlogout" to be able to switch between multiple window managers
@@ -1433,7 +1410,7 @@ static int CatchFatal(Display *dpy)
 }
 
 /* FvwmErrorHandler - displays info on internal errors */
-static int FvwmErrorHandler(Display *dpy, XErrorEvent *event)
+static int FvwmErrorHandler(Display *disp, XErrorEvent *event)
 {
 	char errtext[512];
 
@@ -1458,7 +1435,7 @@ static int FvwmErrorHandler(Display *dpy, XErrorEvent *event)
 		return 0;
 	}
 
-	XGetErrorText(dpy, event->error_code, errtext, 512);
+	XGetErrorText(disp, event->error_code, errtext, 512);
 
 	fvwm_debug(__func__, "*** internal error ***");
 	fvwm_debug(__func__, "Request %d, Error %d, Text %s, "
@@ -1484,7 +1461,7 @@ void StartupStuff(void)
 	ecc.type = Restarting ? EXCT_RESTART : EXCT_INIT;
 	ecc.w.wcontext = C_ROOT;
 	exc = exc_create_context(&ecc, ECC_TYPE | ECC_WCONTEXT);
-	CaptureAllWindows(exc, False);
+	CaptureAllWindows(exc);
 	/* Turn off the SM stuff after the initial capture so that new windows
 	 * will not be matched by accident. */
 	if (Restarting)
@@ -1525,7 +1502,7 @@ void StartupStuff(void)
 	{
 		char *action = "Function " start_func_name;
 
-		execute_function(NULL, exc, action, 0);
+		execute_function(NULL, exc, action, NULL, 0);
 	}
 
 	/* migo (03-Jul-1999): execute [Session]{Init|Restart}Function */
@@ -1535,7 +1512,7 @@ void StartupStuff(void)
 		char *action;
 
 		xasprintf(&action, "Function %s", init_func_name);
-		execute_function(NULL, exc, action, 0);
+		execute_function(NULL, exc, action, NULL, 0);
 		free(action);
 	}
 	/* see comment above */
@@ -1738,6 +1715,7 @@ int main(int argc, char **argv)
 	exec_context_changes_t ecc;
 	struct monitor	*m = NULL;
 
+	functions_init();
 	fvwmlib_init_max_fd();
 	/* Tell the FEvent module an event type that is not used by fvwm. */
 	fev_init_invalid_event_type(KeymapNotify);
@@ -1802,6 +1780,7 @@ int main(int argc, char **argv)
 			   fvwm_userdir);
 	}
 
+	set_log_file(NULL);
 	for (i = 1; i < argc; i++)
 	{
 		if (strcmp(argv[i], "-debug_stack_ring") == 0 ||
@@ -1809,12 +1788,6 @@ int main(int argc, char **argv)
 		{
 			debugging_stack_ring = True;
 		}
-		else if (strcmp(argv[i], "-D") == 0 ||
-			 strcmp(argv[i], "-debug") == 0 ||
-			 strcmp(argv[i], "--debug") == 0)
-		{
-			debugging = True;
-		}
 		else if (strcmp(argv[i], "-i") == 0 ||
 			 strcmp(argv[i], "-clientid") == 0 ||
 			 strcmp(argv[i], "--clientid") == 0 ||
@@ -1931,13 +1904,6 @@ int main(int argc, char **argv)
 			usage(1);
 			exit(0);
 		}
-		else if (strcmp(argv[i], "-blackout") == 0)
-		{
-			/* obsolete option */
-			fvwm_debug(__func__,
-				   "The -blackout option is obsolete, it will be "
-				   "removed in 3.0.");
-		}
 		else if (strcmp(argv[i], "-r") == 0 ||
 			 strcmp(argv[i], "-replace") == 0 ||
 			 strcmp(argv[i], "--replace") == 0)
@@ -2050,10 +2016,21 @@ int main(int argc, char **argv)
 			free(Fvwm_SupportInfo);
 			exit(0);
 		}
-		else if (strcmp(argv[i], "-v") == 0)
+		else if (
+			strcmp(argv[i], "-v") == 0 ||
+			strcmp(argv[i], "--verbose") == 0)
 		{
-			log_set_level(1);
-			log_open(fvwm_userdir);
+			lib_log_level = 1;
+		}
+		else if (strcmp(argv[i], "-o") == 0 ||
+			 strcmp(argv[i], "--output-file") == 0)
+		{
+			if (++i >= argc)
+			{
+				usage(0);
+				exit(1);
+			}
+			set_log_file(argv[i]);
 		}
 		else
 		{
@@ -2063,6 +2040,7 @@ int main(int argc, char **argv)
 			exit(1);
 		}
 	}
+	log_open(fvwm_userdir);
 
 	InstallSignals();
 
@@ -2226,7 +2204,7 @@ int main(int argc, char **argv)
 
 	{
 		XVisualInfo template, *vinfo = NULL;
-		int total, i;
+		int total, j;
 
 		Pdepth = 0;
 		Pdefault = False;
@@ -2261,12 +2239,12 @@ int main(int argc, char **argv)
 
 		/* visualID's are unique so there will only be one.
 		   Select the visualClass with the biggest depth */
-		for (i = 0; i < total; i++)
+		for (j = 0; j < total; j++)
 		{
-			if (vinfo[i].depth > Pdepth)
+			if (vinfo[j].depth > Pdepth)
 			{
-				Pvisual = vinfo[i].visual;
-				Pdepth = vinfo[i].depth;
+				Pvisual = vinfo[j].visual;
+				Pdepth = vinfo[j].depth;
 			}
 		}
 		if (vinfo)
@@ -2284,13 +2262,13 @@ int main(int argc, char **argv)
 				dpy, VisualScreenMask|VisualClassMask,
 				&template, &total);
 
-			for(i = 0; i<total; i++)
+			for(j = 0; j<total; j++)
 			{
-				if (Pdepth < vinfo[i].depth &&
-				    vinfo[i].depth > 8)
+				if (Pdepth < vinfo[j].depth &&
+				    vinfo[j].depth > 8)
 				{
-					Pvisual = vinfo[i].visual;
-					Pdepth = vinfo[i].depth;
+					Pvisual = vinfo[j].visual;
+					Pdepth = vinfo[j].depth;
 				}
 			}
 			if (vinfo)
@@ -2334,9 +2312,7 @@ int main(int argc, char **argv)
 	Scr.ColorLimit = PictureInitColors(
 		PICTURE_CALLED_BY_FVWM, True, &colorLimitop, True, True);
 
-#ifdef Frsvg_init
 	Frsvg_init();
-#endif
 	FShapeInit(dpy);
 	FRenderInit(dpy);
 
@@ -2372,10 +2348,6 @@ int main(int argc, char **argv)
 
 	frame_init();
 	XFlush(dpy);
-	if (debugging)
-	{
-		sync_server(1);
-	}
 
 	SetupICCCM2(replace_wm);
 	XSetIOErrorHandler(CatchFatal);
@@ -2458,12 +2430,13 @@ int main(int argc, char **argv)
 	exc = exc_create_context(&ecc, ECC_TYPE | ECC_WCONTEXT);
 	if (num_config_commands > 0)
 	{
-		int i;
-		for (i = 0; i < num_config_commands; i++)
+		int j;
+		for (j = 0; j < num_config_commands; j++)
 		{
 			DoingCommandLine = True;
-			execute_function(NULL, exc, config_commands[i], 0);
-			free(config_commands[i]);
+			execute_function(
+				NULL, exc, config_commands[j], NULL, 0);
+			free(config_commands[j]);
 		}
 		DoingCommandLine = False;
 	}
@@ -2494,7 +2467,7 @@ int main(int argc, char **argv)
 		xasprintf(&cfg_loc[++nl], "%s/default-config/config", FVWM_DATADIR);
 
 		for (nl = 0; nl < upper; nl++) {
-			if (!run_command_file(cfg_loc[nl], exc)) {
+			if (!run_command_file(cfg_loc[nl], exc, NULL)) {
 				fvwm_debug(__func__,
 				    "couldn't find/load [%d]: %s\n", nl, cfg_loc[nl]);
 				tries++;
