@@ -461,6 +461,8 @@ void menuitem_paint(
 	}
 	else
 	{
+		if (!MI_IS_SEPARATOR(mi))
+			MI_IS_SELECTABLE(mi) = True;
 		gcs = ST_MENU_INACTIVE_GCS(ms);
 		off_gcs = ST_MENU_INACTIVE_GCS(ms);
 	}
@@ -480,7 +482,7 @@ void menuitem_paint(
 	/*
 	 * Hilight the item.
 	 */
-	if (FftSupport && ST_PSTDFONT(ms)->fftf.fftfont != NULL)
+	if (FftSupport)
 	{
 		xft_clear = True;
 	}
