@@ -10,8 +10,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 /* ---------------------------- included header files ---------------------- */
@@ -20,8 +19,7 @@
 
 #include <stdio.h>
 #include <assert.h>
-#include <X11/keysym.h>
-
+#include "libs/fvwm_x11.h"
 #include "libs/fvwmlib.h"
 #include "libs/Parse.h"
 #include "libs/ColorUtils.h"
@@ -68,8 +66,7 @@ static void menustyle_free_face(MenuFace *mf)
 			Pixel *p;
 			int i;
 
-			p = (Pixel *)safemalloc(
-				mf->u.grad.npixels * sizeof(Pixel));
+			p = fxmalloc(mf->u.grad.npixels * sizeof(Pixel));
 			for(i=0; i < mf->u.grad.npixels; i++)
 			{
 				p[i] = mf->u.grad.xcs[i].pixel;
@@ -114,8 +111,7 @@ static void menustyle_copy_face(MenuFace *destmf, MenuFace *origmf)
 		destmf->type = SolidMenu;
 		break;
 	case GradientMenu:
-		destmf->u.grad.xcs =
-			(XColor *)safemalloc(sizeof(XColor) *
+		destmf->u.grad.xcs = fxmalloc(sizeof(XColor) *
 					     origmf->u.grad.npixels);
 		memcpy(destmf->u.grad.xcs,
 		       origmf->u.grad.xcs,
@@ -133,15 +129,15 @@ static void menustyle_copy_face(MenuFace *destmf, MenuFace *origmf)
 	case PixmapMenu:
 	case TiledPixmapMenu:
 		fpa.mask = (Pdepth <= 8)?  FPAM_DITHER:0;
-	
-		if (destmf->u.p) 
+
+		if (destmf->u.p)
 			destmf->u.p = NULL;
 
 		if (origmf->u.p)
 			destmf->u.p = PCacheFvwmPicture(
 				dpy, Scr.NoFocusWin, NULL, origmf->u.p->name,
 				fpa);
-	
+
 		destmf->type = origmf->type;
 		break;
 	default:
@@ -154,7 +150,7 @@ static void menustyle_copy_face(MenuFace *destmf, MenuFace *origmf)
  * Reads a menu face line into a structure (veliaa@rpi.edu)
  *
  */
-static Boolean menustyle_parse_face(char *s, MenuFace *mf, int verbose)
+static Bool menustyle_parse_face(char *s, MenuFace *mf, int verbose)
 {
 	char *style;
 	char *token;
@@ -189,9 +185,9 @@ static Boolean menustyle_parse_face(char *s, MenuFace *mf, int verbose)
 		{
 			if (verbose)
 			{
-				fvwm_msg(ERR, "menustyle_parse_face",
-					 "no color given for Solid face type:"
-					 " %s", action);
+				fvwm_debug(__func__,
+					   "no color given for Solid face type:"
+					   " %s", action);
 			}
 			free(style);
 			return False;
@@ -247,9 +243,9 @@ static Boolean menustyle_parse_face(char *s, MenuFace *mf, int verbose)
 			{
 				if (verbose)
 				{
-					fvwm_msg(ERR, "menustyle_parse_face",
-						 "couldn't load pixmap %s",
-						 token);
+					fvwm_debug(__func__,
+						   "couldn't load pixmap %s",
+						   token);
 				}
 				free(token);
 				free(style);
@@ -263,9 +259,9 @@ static Boolean menustyle_parse_face(char *s, MenuFace *mf, int verbose)
 		{
 			if (verbose)
 			{
-				fvwm_msg(ERR, "menustyle_parse_face",
-					 "missing pixmap name for style %s",
-					 style);
+				fvwm_debug(__func__,
+					   "missing pixmap name for style %s",
+					   style);
 			}
 			free(style);
 			return False;
@@ -275,10 +271,8 @@ static Boolean menustyle_parse_face(char *s, MenuFace *mf, int verbose)
 	{
 		if (verbose)
 		{
-			fvwm_msg(
-				ERR,
-				"menustyle_parse_face", "unknown style %s: %s",
-				style, action);
+			fvwm_debug(__func__, "unknown style %s: %s",
+				   style, action);
 		}
 		free(style);
 		return False;
@@ -345,8 +339,8 @@ static MenuStyle *menustyle_parse_old_style(F_CMD_ARGS)
 
 	if (!fore || !back || !stipple || !font || !style)
 	{
-		fvwm_msg(ERR, "menustyle_parse_old_style",
-			 "error in %s style specification", action);
+		fvwm_debug(__func__,
+			   "error in %s style specification", action);
 	}
 	else
 	{
@@ -357,10 +351,10 @@ static MenuStyle *menustyle_parse_old_style(F_CMD_ARGS)
 			style, fore, back, stipple, font,
 			(animated && StrEquals(animated, "anim")) ?
 			"Animation" : "AnimationOff");
-		fvwm_msg(OLD, "menustyle_parse_old_style",
-			 "The old MenuStyle snytax has been deprecated.  "
-			 "Use 'MenuStyle %s' instead of 'MenuStyle %s'\n",
-			 buffer, action);
+		fvwm_debug(__func__,
+			   "The old MenuStyle syntax has been deprecated.  "
+			   "Use 'MenuStyle %s' instead of 'MenuStyle %s'\n",
+			   buffer, action);
 		action = buffer;
 		ms = menustyle_parse_style(F_PASS_ARGS);
 	}
@@ -600,8 +594,8 @@ void menustyle_update(MenuStyle *ms)
 
 	if (ST_USAGE_COUNT(ms) != 0)
 	{
-		fvwm_msg(ERR,"menustyle_update", "menu style %s is in use",
-			 ST_NAME(ms));
+		fvwm_debug(__func__, "menu style %s is in use",
+			   ST_NAME(ms));
 		return;
 	}
 	ST_IS_UPDATED(ms) = 1;
@@ -699,7 +693,7 @@ void menustyle_update(MenuStyle *ms)
 		c_title.hilight = c_inactive.hilight;
 		c_title.shadow = c_inactive.shadow;
 	}
-	/* override hilighting colours if necessary */
+	/* override highlighting colours if necessary */
 	if (!ST_DO_HILIGHT_FORE(ms))
 	{
 		c_active.fore = c_inactive.fore;
@@ -850,18 +844,18 @@ MenuStyle *menustyle_parse_style(F_CMD_ARGS)
 	action = GetNextToken(action, &name);
 	if (!name)
 	{
-		fvwm_msg(ERR, "NewMenuStyle",
-			 "error in %s style specification",action);
+		fvwm_debug(__func__,
+			   "error in %s style specification",action);
 		return NULL;
 	}
 
 	ms = menustyle_find(name);
 	if (ms && ST_USAGE_COUNT(ms) != 0)
 	{
-		fvwm_msg(ERR,"NewMenuStyle", "menu style %s is in use", name);
+		fvwm_debug(__func__, "menu style %s is in use", name);
 		return ms;
 	}
-	tmpms = (MenuStyle *)safemalloc(sizeof(MenuStyle));
+	tmpms = fxmalloc(sizeof(MenuStyle));
 	if (ms)
 	{
 		/* copy the structure over our temporary menu face. */
@@ -1164,8 +1158,8 @@ MenuStyle *menustyle_parse_style(F_CMD_ARGS)
 			if (arg1 != NULL &&
 			    !(new_font = FlocaleLoadFont(dpy, arg1, "fvwm")))
 			{
-				fvwm_msg(ERR, "NewMenuStyle",
-					 "Couldn't load font '%s'\n", arg1);
+				fvwm_debug(__func__,
+					   "Couldn't load font '%s'\n", arg1);
 				break;
 			}
 			if (ST_PSTDFONT(tmpms) && !ST_USING_DEFAULT_FONT(tmpms))
@@ -1210,9 +1204,9 @@ MenuStyle *menustyle_parse_style(F_CMD_ARGS)
 		case 18: /* PopupOffset */
 			if ((n = GetIntegerArguments(args, NULL, val, 2)) == 0)
 			{
-				fvwm_msg(ERR,"NewMenuStyle",
-					 "PopupOffset requires one or two"
-					 " arguments");
+				fvwm_debug(__func__,
+					   "PopupOffset requires one or two"
+					   " arguments");
 			}
 			else
 			{
@@ -1298,9 +1292,9 @@ MenuStyle *menustyle_parse_style(F_CMD_ARGS)
 					dpy, Scr.NoFocusWin, NULL, arg1, fpa);
 				if (!ST_SIDEPIC(tmpms))
 				{
-					fvwm_msg(WARN, "NewMenuStyle",
-						 "Couldn't find pixmap %s",
-						 arg1);
+					fvwm_debug(__func__,
+						   "Couldn't find pixmap %s",
+						   arg1);
 				}
 			}
 			break;
@@ -1382,7 +1376,7 @@ MenuStyle *menustyle_parse_style(F_CMD_ARGS)
 			}
 			if (arg1)
 			{
-				ST_ITEM_FORMAT(tmpms) = safestrdup(arg1);
+				ST_ITEM_FORMAT(tmpms) = fxstrdup(arg1);
 			}
 			break;
 
@@ -1530,11 +1524,10 @@ MenuStyle *menustyle_parse_style(F_CMD_ARGS)
 				}
 				else
 				{
-					fvwm_msg(
-						ERR, "NewMenuStyle",
-						"unknown argument to"
-						" MouseWheel '%s'",
-						 arg1);
+					fvwm_debug(__func__,
+						   "unknown argument to"
+						   " MouseWheel '%s'",
+						   arg1);
 					ST_MOUSE_WHEEL(tmpms) = MMW_POINTER;
 				}
 		        }
@@ -1574,8 +1567,8 @@ MenuStyle *menustyle_parse_style(F_CMD_ARGS)
 			if (arg1 != NULL &&
 			    !(new_font = FlocaleLoadFont(dpy, arg1, "fvwm")))
 			{
-				fvwm_msg(ERR, "NewMenuStyle",
-					 "Couldn't load font '%s'\n", arg1);
+				fvwm_debug(__func__,
+					   "Couldn't load font '%s'\n", arg1);
 				break;
 			}
 			if (
@@ -1614,8 +1607,8 @@ MenuStyle *menustyle_parse_style(F_CMD_ARGS)
 #endif
 
 		default:
-			fvwm_msg(ERR, "NewMenuStyle", "unknown option '%s'",
-				 poption);
+			fvwm_debug(__func__, "unknown option '%s'",
+				   poption);
 			break;
 		} /* switch */
 
@@ -1669,7 +1662,7 @@ void menustyle_copy(MenuStyle *origms, MenuStyle *destms)
 {
 	FvwmPictureAttributes fpa;
 	/* Copy origms to destms, be aware of all pointers in the MenuStyle
-	   strcture. Use  the same order as in menustyle_parse_style */
+	   structure. Use  the same order as in menustyle_parse_style */
 
 	/* menu colors */
 	fvwmlib_copy_color(
@@ -1720,9 +1713,9 @@ void menustyle_copy(MenuStyle *origms, MenuStyle *destms)
 		{
 			ST_PSTDFONT(destms) = Scr.DefaultFont;
 			ST_USING_DEFAULT_FONT(destms) = True;
-			fvwm_msg(ERR, "CopyMenuStyle",
-				 "Couldn't load font '%s' use Default Font\n",
-				 ST_PSTDFONT(origms)->name);
+			fvwm_debug(__func__,
+				   "Couldn't load font '%s' use Default Font\n",
+				   ST_PSTDFONT(origms)->name);
 		}
 		else
 		{
@@ -1747,9 +1740,9 @@ void menustyle_copy(MenuStyle *origms, MenuStyle *destms)
 		{
 			ST_PTITLEFONT(destms) = Scr.DefaultFont;
 			ST_USING_DEFAULT_TITLEFONT(destms) = True;
-			fvwm_msg(ERR, "CopyMenuStyle",
-				 "Couldn't load font '%s' use Default Font\n",
-				 ST_PTITLEFONT(origms)->name);
+			fvwm_debug(__func__,
+				   "Couldn't load font '%s' use Default Font\n",
+				   ST_PTITLEFONT(origms)->name);
 		}
 		else
 		{
@@ -1826,7 +1819,7 @@ void menustyle_copy(MenuStyle *origms, MenuStyle *destms)
 	}
 	if (ST_ITEM_FORMAT(origms))
 	{
-		ST_ITEM_FORMAT(destms) = safestrdup(ST_ITEM_FORMAT(origms));
+		ST_ITEM_FORMAT(destms) = fxstrdup(ST_ITEM_FORMAT(origms));
 	}
 
 	/* AutomaticHotkeys */
@@ -1883,28 +1876,28 @@ void CMD_CopyMenuStyle(F_CMD_ARGS)
 	origname = PeekToken(action, &action);
 	if (origname == NULL)
 	{
-		fvwm_msg(ERR,"CopyMenuStyle", "need two arguments");
+		fvwm_debug(__func__, "need two arguments");
 		return;
 	}
 
 	origms = menustyle_find(origname);
 	if (!origms)
 	{
-		fvwm_msg(ERR, "CopyMenuStyle", "%s: no such menu style",
-			 origname);
+		fvwm_debug(__func__, "%s: no such menu style",
+			   origname);
 		return;
 	}
 
 	destname = PeekToken(action, &action);
 	if (destname == NULL)
 	{
-		fvwm_msg(ERR,"CopyMenuStyle", "need two arguments");
+		fvwm_debug(__func__, "need two arguments");
 		return;
 	}
 
 	if (action && *action)
 	{
-		fvwm_msg(ERR,"CopyMenuStyle", "too many arguments");
+		fvwm_debug(__func__, "too many arguments");
 		return;
 	}
 
@@ -1912,7 +1905,8 @@ void CMD_CopyMenuStyle(F_CMD_ARGS)
 	if (!destms)
 	{
 		/* create destms menu style */
-		buffer = (char *)safemalloc(strlen(destname) + 3);
+		/* TA:  FIXME!  xasprintf() */
+		buffer = fxmalloc(strlen(destname) + 3);
 		sprintf(buffer,"\"%s\"",destname);
 		action = buffer;
 		destms = menustyle_parse_style(F_PASS_ARGS);
@@ -1920,31 +1914,31 @@ void CMD_CopyMenuStyle(F_CMD_ARGS)
 		if (!destms)
 		{
 			/* this must never happen */
-			fvwm_msg(ERR, "CopyMenuStyle",
-				 "impossible to create %s menu style",
-				 destname);
+			fvwm_debug(__func__,
+				   "impossible to create %s menu style",
+				   destname);
 			return;
 		}
 	}
 
 	if (strcasecmp("*",destname) == 0)
 	{
-		fvwm_msg(ERR, "CopyMenuStyle",
-			 "You cannot copy on the default menu style");
+		fvwm_debug(__func__,
+			   "You cannot copy on the default menu style");
 		return;
 	}
 	if (strcasecmp(ST_NAME(origms),destname) == 0)
 	{
-		fvwm_msg(ERR, "CopyMenuStyle",
-			 "%s and %s identify the same menu style",
-			 ST_NAME(origms),destname);
+		fvwm_debug(__func__,
+			   "%s and %s identify the same menu style",
+			   ST_NAME(origms),destname);
 		return;
 	}
 
 	if (ST_USAGE_COUNT(destms) != 0)
 	{
-		fvwm_msg(ERR, "CopyMenuStyle", "menu style %s is in use",
-			 destname);
+		fvwm_debug(__func__, "menu style %s is in use",
+			   destname);
 		return;
 	}
 
