@@ -33,6 +33,7 @@
 #include "libs/FEvent.h"
 #include "fvwm.h"
 #include "externs.h"
+#include "cmdparser.h"
 #include "functions.h"
 #include "bindings.h"
 #include "misc.h"
@@ -96,6 +97,7 @@ make_new_vpacket(unsigned char *body, unsigned long event_type,
 	unsigned long *bp = (unsigned long *)body;
 	unsigned long *bp1 = bp;
 	unsigned long plen = 0;
+	unsigned long dlen;
 
 	*(bp++) = START_FLAG;
 	*(bp++) = event_type;
@@ -169,10 +171,16 @@ make_new_vpacket(unsigned char *body, unsigned long event_type,
 
 	/*
 	  Round up to a long word boundary. Most of the module interface
-	  still thinks in terms of an array of longss, so let's humor it.
+	  still thinks in terms of an array of longs, so let's humor it.
 	*/
-	plen = (unsigned long) ((char *)bp - (char *)bp1);
-	plen = ((plen + (sizeof(long) - 1)) / sizeof(long)) * sizeof(long);
+
+	dlen = (unsigned long) ((unsigned char *)bp - body);
+	plen = ((dlen + (sizeof(long) - 1)) / sizeof(long)) * sizeof(long);
+	if (plen > dlen)
+	{
+		/* initialise padding */
+		memset(body + dlen, 0, plen - dlen);
+	}
 	*(((unsigned long*)bp1)+2) = (plen / (sizeof(unsigned long)));
 
 	return plen;
@@ -727,7 +735,7 @@ void module_input_execute(struct fmodule_input *input)
 	exc = exc_create_context(
 		&ecc, ECC_TYPE | ECC_ETRIGGER | ECC_FW | ECC_W | ECC_WCONTEXT |
 		ECC_MODULE);
-	execute_function(NULL, exc, input->command, flags);
+	execute_function(NULL, exc, input->command, NULL, flags);
 	exc_destroy_context(exc);
 	module_input_discard(input);
 
@@ -1002,7 +1010,7 @@ void CMD_Send_WindowList(F_CMD_ARGS)
 		}
 
 		/* If we're tracking just the global monitor, then stop here
-		 * -- essentialy, we only need to send this informatio once.
+		 * -- essentially, we only need to send this information once.
 		 */
 		if (monitor_mode == MONITOR_TRACKING_G)
 			break;
