@@ -10,8 +10,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 #include "config.h"
@@ -49,6 +48,7 @@
 #include "libs/Strings.h"
 #include "libs/wild.h"
 #include "libs/fvwmsignal.h"
+#include "libs/FEvent.h"
 #include "events.h"
 #include "bindings.h"
 
@@ -64,6 +64,13 @@
 #  define O_NONBLOCK  O_NDELAY
 #endif
 
+#ifndef FORK_CREATES_CHILD
+#define FORK_CREATES_CHILD 0
+#endif
+#ifndef REMOVE_EXECUTABLE_EXTENSION
+#define REMOVE_EXECUTABLE_EXTENSION 0
+#endif
+
 #define MOD_NOGRABMASK(m) ((m)->xNoGrabMask)
 #define MOD_SYNCMASK(m) ((m)->xSyncMask)
 
@@ -80,6 +87,8 @@ static fmodule_list module_list = NULL;
 /* keep to-be-deleted modules in a deathrow until they are deleted safely. */
 static fmodule_list death_row = NULL;
 
+extern int moduleTimeout;
+
 /*
  * static functions
  */
@@ -92,7 +101,6 @@ static void module_free(fmodule *module);
 static inline void module_list_insert(fmodule *module, fmodule_list *list);
 static inline fmodule *module_list_remove(fmodule *module, fmodule_list *list);
 static inline void module_list_destroy(fmodule_list *list);
-static inline int module_list_len(fmodule_list *list);
 
 static void KillModuleByName(char *name, char *alias);
 static char *get_pipe_name(fmodule *module);
@@ -114,7 +122,7 @@ static fmodule *module_alloc(void)
 {
 	fmodule *module;
 
-	module = (fmodule *)safemalloc(sizeof(fmodule));
+	module = fxmalloc(sizeof(fmodule));
 	MOD_SET_CMDLINE(module, 0);
 	MOD_READFD(module) = -1;
 	MOD_WRITEFD(module) = -1;
@@ -158,12 +166,11 @@ static void module_free(fmodule *module)
 static inline void module_list_insert(fmodule *module, fmodule_list *list)
 {
 	fmodule_store *new_store;
-	DBUG("module_list_insert", "inserting module");
 	if (module == NULL)
 	{
 		return;
 	}
-	new_store = (fmodule_store*)safemalloc(sizeof(fmodule_store));
+	new_store = fxmalloc(sizeof(fmodule_store));
 	new_store->module = module;
 	new_store->next = *list;
 	*list = new_store;
@@ -173,34 +180,28 @@ static inline void module_list_insert(fmodule *module, fmodule_list *list)
 
 static inline fmodule *module_list_remove(fmodule *module, fmodule_list *list)
 {
-	fmodule_store **position;
+	fmodule_store **pos;
 
 	if (module == NULL)
 	{
 		return NULL;
 	}
 
-	for (
-		position = list; *position != NULL;
-		position = &((*position)->next))
+	for (pos = list; *pos != NULL; pos = &((*pos)->next))
 	{
-		if ((*position)->module == module)
+		if ((*pos)->module == module)
 		{
 			/* found it */
 			fmodule_store *current;
 
-			DBUG("module_list_remove", "Removing from module list");
-			current = *position;
-			*position = (*position)->next;
+			current = *pos;
+			*pos = (*pos)->next;
 			free(current);
 
 			return module;
 		}
 	}
 
-	/* module not found */
-	DBUG("module_list_remove", "Tried to remove a not listed module!");
-
 	return NULL;
 }
 
@@ -242,10 +243,7 @@ static fmodule *do_execute_module(
 	fvwm_to_app[1] = -1;
 	app_to_fvwm[1] = -1;
 	app_to_fvwm[0] = -1;
-	args = (char **)safemalloc(7 * sizeof(char *));
-	/* Olivier: Why ? */
-	/*   if (eventp->type != KeyPress) */
-	/*     UngrabEm(); */
+	args = fxmalloc(7 * sizeof(char *));
 	if (action == NULL)
 	{
 		goto err_exit;
@@ -272,24 +270,23 @@ static fmodule *do_execute_module(
 		 * likely a typo in a command, not a module name. */
 		if (!desperate)
 		{
-			fvwm_msg(
-				ERR, "executeModule",
-				"No such module '%s' in ModulePath '%s'",
-				cptr, ModulePath);
+			fvwm_debug(__func__,
+				   "No such module '%s' in ModulePath '%s'",
+				   cptr, ModulePath);
 		}
 		goto err_exit;
 	}
-#ifdef REMOVE_EXECUTABLE_EXTENSION
+	if (REMOVE_EXECUTABLE_EXTENSION && EXECUTABLE_EXTENSION != NULL)
 	{
 		char *p;
+		char *ext = EXECUTABLE_EXTENSION;
 
-		p = arg1 + strlen(arg1) - strlen(EXECUTABLE_EXTENSION);
-		if (strcmp(p, EXECUTABLE_EXTENSION) == 0)
+		p = arg1 + strlen(arg1) - strlen(ext);
+		if (strcmp(p, ext) == 0)
 		{
 			*p = 0;
 		}
 	}
-#endif
 
 	/* I want one-ended pipes, so I open two two-ended pipes,
 	 * and close one end of each. I need one ended pipes so that
@@ -301,12 +298,12 @@ static fmodule *do_execute_module(
 	}
 	else if (pipe(fvwm_to_app) != 0)
 	{
-		fvwm_msg(ERR, "executeModule", "Failed to open pipe");
+		fvwm_debug(__func__, "Failed to open pipe");
 		goto err_exit;
 	}
 	if (pipe(app_to_fvwm) != 0)
 	{
-		fvwm_msg(ERR, "executeModule", "Failed to open pipe2");
+		fvwm_debug(__func__, "Failed to open pipe2");
 		goto err_exit;
 	}
 	if (
@@ -315,7 +312,7 @@ static fmodule *do_execute_module(
 		app_to_fvwm[0] >= fvwmlib_max_fd ||
 		app_to_fvwm[1] >= fvwmlib_max_fd)
 	{
-		fvwm_msg(ERR, "executeModule", "too many open fds");
+		fvwm_debug(__func__, "too many open fds");
 		goto err_exit;
 	}
 
@@ -340,8 +337,7 @@ static fmodule *do_execute_module(
 	args[5] = arg6;
 	for (nargs = 6; action = GetNextToken(action, &token), token; nargs++)
 	{
-		args = (char **)saferealloc(
-			(void *)args, (nargs + 2) * sizeof(char *));
+		args = fxrealloc((void *)args, (nargs + 2), sizeof(char *));
 		args[nargs] = token;
 		if (MOD_ALIAS(module) == NULL)
 		{
@@ -374,11 +370,9 @@ static fmodule *do_execute_module(
 		MOD_READFD(module) = app_to_fvwm[0];
 		msg_mask_set(
 			&MOD_PIPEMASK(module), DEFAULT_MASK, DEFAULT_MASK);
-		free(arg1);
 		if (DoingCommandLine)
 		{
 			/* add to the list of command line modules */
-			DBUG("executeModule", "starting commandline module\n");
 			MOD_SET_CMDLINE(module, 1);
 		}
 
@@ -392,38 +386,30 @@ static fmodule *do_execute_module(
 		 * won`t inherit them */
 		if (fcntl(MOD_READFD(module), F_SETFD, 1) == -1)
 		{
-			fvwm_msg(
-				ERR, "executeModule",
-				"module close-on-exec failed");
+			fvwm_debug(__func__,
+				   "module close-on-exec failed");
 		}
 		if (
 			MOD_WRITEFD(module) >= 0 &&
 			fcntl(MOD_WRITEFD(module), F_SETFD, 1) == -1)
 		{
-			fvwm_msg(
-				ERR, "executeModule",
-				"module close-on-exec failed");
+			fvwm_debug(__func__,
+				   "module close-on-exec failed");
 		}
 		/* module struct is completed, insert into the list */
 		module_list_insert(module, &module_list);
 
-		for (i = 6; i < nargs; i++)
-		{
-			if (args[i] != 0)
-			{
-				free(args[i]);
-			}
-		}
 	}
-	else if (val ==0)
+	else if (val == 0)
 	{
 		/* this is the child */
 		/* this fork execs the module */
-#ifdef FORK_CREATES_CHILD
-		/* dont't care that this may be -1 */
-		close(fvwm_to_app[1]);
-		close(app_to_fvwm[0]);
-#endif
+		if (FORK_CREATES_CHILD)
+		{
+			/* dont't care that this may be -1 */
+			close(fvwm_to_app[1]);
+			close(app_to_fvwm[0]);
+		}
 		fvmm_deinstall_signals();
 		if (!Pdefault)
 		{
@@ -445,33 +431,32 @@ static fmodule *do_execute_module(
 
 		/* Why is this execvp??  We've already searched the module
 		 * path! */
-		execvp(arg1,args);
-		fvwm_msg(
-			ERR, "executeModule", "Execution of module failed: %s",
-			arg1);
+		execvp(arg1, args);
+		fvwm_debug(__func__, "Execution of module failed: %s", arg1);
 		perror("");
 		close(app_to_fvwm[1]);
-		/* dont't care that this may be -1 */
-		close(fvwm_to_app[0]);
-#ifdef FORK_CREATES_CHILD
-		exit(1);
-#endif
+		if (fvwm_to_app[0] >= 0)
+		{
+			close(fvwm_to_app[0]);
+		}
+		if (FORK_CREATES_CHILD)
+		{
+			exit(1);
+		}
 	}
 	else
 	{
-		fvwm_msg(ERR, "executeModule", "Fork failed");
-		free(arg1);
-		for (i = 6; i < nargs; i++)
-		{
-			if (args[i] != 0)
-			{
-				free(args[i]);
-			}
-		}
-		free(args);
+		fvwm_debug(__func__, "Fork failed");
 		module_free(module);
-
-		return NULL;
+		module = NULL;
+	}
+	free(arg1);
+	for (i = 6; i < nargs; i++)
+	{
+		if (args[i] != 0)
+		{
+			free(args[i]);
+		}
 	}
 	free(args);
 
@@ -490,11 +475,22 @@ static fmodule *do_execute_module(
 	{
 		free(args);
 	}
-	/* dont't care that these may be -1 */
-	close(fvwm_to_app[0]);
-	close(fvwm_to_app[1]);
-	close(app_to_fvwm[0]);
-	close(app_to_fvwm[1]);
+	if (fvwm_to_app[0] >= 0)
+	{
+		close(fvwm_to_app[0]);
+	}
+	if (fvwm_to_app[1] >= 0)
+	{
+		close(fvwm_to_app[1]);
+	}
+	if (app_to_fvwm[0] >= 0)
+	{
+		close(app_to_fvwm[0]);
+	}
+	if (app_to_fvwm[1] >= 0)
+	{
+		close(app_to_fvwm[1]);
+	}
 
 	return NULL;
 }
@@ -532,7 +528,6 @@ extern int myxgrabcount;                /* defined in libs/Grab.c */
 extern char *ModuleUnlock;              /* defined in libs/Module.c */
 void PositiveWrite(fmodule *module, unsigned long *ptr, int size)
 {
-	extern int moduleTimeout;
 	msg_masks_t mask;
 
 	if (ptr == NULL)
@@ -568,7 +563,7 @@ void PositiveWrite(fmodule *module, unsigned long *ptr, int size)
 			sizeof(mqueue_object_type) + size);
 		if (c == NULL)
 		{
-			fvwm_msg(ERR, "PositiveWrite", "malloc failed\n");
+			fvwm_debug(__func__, "malloc failed\n");
 			exit(1);
 		}
 		c->size = size;
@@ -658,15 +653,15 @@ void PositiveWrite(fmodule *module, unsigned long *ptr, int size)
 				name = get_pipe_name(module);
 				/* Doh! Something has gone wrong - get rid of
 				 * the offender! */
-				fvwm_msg(ERR, "PositiveWrite",
-					 "Failed to read descriptor from"
-					 " '%s':\n"
-					 "- data available=%c\n"
-					 "- terminate signal=%c\n",
-					 name,
-					 (FD_ISSET(channel, &readSet) ?
-					  'Y' : 'N'),
-					 isTerminated ? 'Y' : 'N');
+				fvwm_debug(__func__,
+					   "Failed to read descriptor from"
+					   " '%s':\n"
+					   "- data available=%c\n"
+					   "- terminate signal=%c\n",
+					   name,
+					   (FD_ISSET(channel, &readSet) ?
+					    'Y' : 'N'),
+					   isTerminated ? 'Y' : 'N');
 				module_kill(module);
 				break;
 			}
@@ -712,18 +707,17 @@ fmodule_input *module_receive(fmodule *module)
 	n = read(MOD_READFD(module), &size, sizeof(size));
 	if (n < sizeof(size))
 	{
-		fvwm_msg(
-			ERR, "module_receive",
-			"Fail to read command size (Module: %p, read: %i, "
-			"size: %i)", module, n, (int)sizeof(size));
+		fvwm_debug(__func__,
+			   "Fail to read command size (Module: %p, read: %i, "
+			   "size: %i)", module, n, (int)sizeof(size));
 		goto err;
 	}
 
 	if (size > MAX_MODULE_INPUT_TEXT_LEN)
 	{
-		fvwm_msg(ERR, "module_receive",
-			 "Module(%p) command is too big (%ld), limit is %d",
-			 module, size, MAX_MODULE_INPUT_TEXT_LEN);
+		fvwm_debug(__func__,
+			   "Module(%p) command is too big (%ld), limit is %d",
+			   module, size, MAX_MODULE_INPUT_TEXT_LEN);
 		/* The rest of the output from this module is going to be
 		 * scrambled so let's kill it rather than risk interpreting
 		 * garbage */
@@ -732,8 +726,7 @@ fmodule_input *module_receive(fmodule *module)
 
 	/* allocate all storage at once */
 	/* also save space for the '\0' termination character */
-	input = (fmodule_input *)safemalloc(
-		sizeof(fmodule_input) + sizeof(char)*(size + 1));
+	input = fxmalloc(sizeof(fmodule_input) + sizeof(char)*(size + 1));
 
 	input->module = module;
 	input->window = win;
@@ -742,19 +735,18 @@ fmodule_input *module_receive(fmodule *module)
 	n = read(MOD_READFD(module), input->command, size);
 	if (n < size)
 	{
-		fvwm_msg(
-			ERR, "module_receive",
-			"Fail to read command (Module: %p, read: %i, size:"
-			" %ld)", module, n, size);
+		fvwm_debug(__func__,
+			   "Fail to read command (Module: %p, read: %i, size:"
+			   " %ld)", module, n, size);
 		goto err;
 	}
 	input->command[n] = '\0';
 	n = read(MOD_READFD(module), &cont, sizeof(cont));
 	if (n < sizeof(cont))
 	{
-		fvwm_msg(ERR, "module_receive",
-			 "Module %p, Size Problems (read: %d, size: %d)",
-			 module, n, (int)sizeof(cont));
+		fvwm_debug(__func__,
+			   "Module %p, Size Problems (read: %d, size: %d)",
+			   module, n, (int)sizeof(cont));
 		goto err;
 	}
 	if (cont == 0)
@@ -819,19 +811,6 @@ fmodule *module_list_itr_next(fmodule_list_itr *itr)
 	return module;
 }
 
-int module_list_len(fmodule_list *list)
-{
-	int count=0;
-	fmodule_store *current=*list;
-	while (current != NULL)
-	{
-		current = current->next;
-		count++;
-	}
-
-	return count;
-}
-
 static void KillModuleByName(char *name, char *alias)
 {
 	fmodule_list_itr moditr;
@@ -860,23 +839,23 @@ static void KillModuleByName(char *name, char *alias)
 
 static char *get_pipe_name(fmodule *module)
 {
-	char *name="";
+	char *name;
 
 	if (MOD_NAME(module) != NULL)
 	{
 		if (MOD_ALIAS(module) != NULL)
 		{
-			name = CatString3(
-				MOD_NAME(module), " ", MOD_ALIAS(module));
+			xasprintf(&name, "%s %s", MOD_NAME(module),
+			    MOD_ALIAS(module));
 		}
 		else
 		{
-			name = MOD_NAME(module);
+			xasprintf(&name, "%s", MOD_NAME(module));
 		}
 	}
 	else
 	{
-		name = CatString3("(null)", "", "");
+		name = fxstrdup("(null)");
 	}
 
 	return name;
@@ -964,7 +943,6 @@ static void DeleteMessageQueueBuff(fmodule *module)
 
 void FlushMessageQueue(fmodule *module)
 {
-	extern int moduleTimeout;
 	mqueue_object_type *obj;
 	char *dptr;
 	int a;
@@ -1025,15 +1003,15 @@ void FlushMessageQueue(fmodule *module)
 					name = get_pipe_name(module);
 					/* Doh! Something has gone wrong - get
 					 * rid of the offender! */
-					fvwm_msg(
-						ERR, "FlushMessageQueue",
-						"Failed to write descriptor to"
-						" '%s':\n"
-						"- select rc=%d\n"
-						"- terminate signal=%c\n",
-						name, rc, isTerminated ?
-						'Y' : 'N');
+					fvwm_debug(__func__,
+						   "Failed to write descriptor to"
+						   " '%s':\n"
+						   "- select rc=%d\n"
+						   "- terminate signal=%c\n",
+						   name, rc, isTerminated ?
+						   'Y' : 'N');
 					module_kill(module);
+					free(name);
 
 					return;
 				}
@@ -1151,7 +1129,7 @@ void CMD_ModuleSynchronous(F_CMD_ARGS)
 		}
 		else
 		{
-			fvwm_msg(ERR, "executeModuleSync", "illegal timeout");
+			fvwm_debug(__func__, "illegal timeout");
 			return;
 		}
 	}
@@ -1270,7 +1248,7 @@ void CMD_ModuleSynchronous(F_CMD_ARGS)
 				name = NULL;
 			}
 			escape = CheckBinding(
-				Scr.AllBindings, STROKE_ARG(0)
+				Scr.AllBindings,
 				tmpevent.xkey.keycode, tmpevent.xkey.state,
 				GetUnusedModifiers(), context, BIND_KEYPRESS,
 				class, name);
