@@ -51,8 +51,8 @@
 
 /* ---------------------------- local definitions -------------------------- */
 
-/*#define FVWM_SM_DEBUG_PROTO*/
-/*#define FVWM_SM_DEBUG_WINMATCH*/
+#define FVWM_SM_DEBUG_PROTO 0
+#define FVWM_SM_DEBUG_WINMATCH 0
 
 /* ---------------------------- local macros ------------------------------- */
 
@@ -61,7 +61,6 @@
 /* ---------------------------- imports ------------------------------------ */
 
 extern Bool Restarting;
-extern int master_pid;
 extern char **g_argv;
 extern int g_argc;
 
@@ -104,7 +103,7 @@ static Bool sent_save_done = 0;
 static char *real_state_filename = NULL;
 static Bool going_to_restart = False;
 static FIceIOErrorHandler prev_handler;
-static FSmcConn sm_conn = NULL;
+static FSmcConn ssm_conn = NULL;
 
 static int num_match = 0;
 static Match *matches = NULL;
@@ -663,45 +662,46 @@ static Bool matchWin(FvwmWindow *w, Match *m)
 		} /* else no window roles */
 	} /* if client_id's agree */
 
-#ifdef FVWM_SM_DEBUG_WINMATCH
-	fvwm_debug(__func__,
-		   "\twin(%s, %s, %s, %s, %s,",
-		   w->class.res_name, w->class.res_class, w->name.name,
-		   (client_id)? client_id:"(null)",
-		   (window_role)? window_role:"(null)");
-	if (wm_command)
+	if (FVWM_SM_DEBUG_WINMATCH)
 	{
-		for (i = 0; i < wm_command_count; i++)
+		fvwm_debug(__func__,
+			   "\twin(%s, %s, %s, %s, %s,",
+			   w->class.res_name, w->class.res_class, w->name.name,
+			   (client_id)? client_id:"(null)",
+			   (window_role)? window_role:"(null)");
+		if (wm_command)
 		{
-			fvwm_debug(__func__, " %s", wm_command[i]);
+			for (i = 0; i < wm_command_count; i++)
+			{
+				fvwm_debug(__func__, " %s", wm_command[i]);
+			}
+			fvwm_debug(__func__, ",");
 		}
-		fvwm_debug(__func__, ",");
-	}
-	else
-	{
-		fvwm_debug(__func__, " no_wmc,");
-	}
-	fvwm_debug(__func__, " %d)", IS_NAME_CHANGED(w));
-	fvwm_debug(__func__, "\n[%d]", found);
-	fvwm_debug(__func__,
-		   "\tmat(%s, %s, %s, %s, %s,",
-		   m->res_name, m->res_class, m->wm_name,
-		   (m->client_id)?m->client_id:"(null)",
-		   (m->window_role)?m->window_role:"(null)");
-	if (m->wm_command)
-	{
-		for (i = 0; i < m->wm_command_count; i++)
+		else
 		{
-			fvwm_debug(__func__, " %s", m->wm_command[i]);
+			fvwm_debug(__func__, " no_wmc,");
 		}
-		fvwm_debug(__func__, ",");
+		fvwm_debug(__func__, " %d)", IS_NAME_CHANGED(w));
+		fvwm_debug(__func__, "\n[%d]", found);
+		fvwm_debug(__func__,
+			   "\tmat(%s, %s, %s, %s, %s,",
+			   m->res_name, m->res_class, m->wm_name,
+			   (m->client_id)?m->client_id:"(null)",
+			   (m->window_role)?m->window_role:"(null)");
+		if (m->wm_command)
+		{
+			for (i = 0; i < m->wm_command_count; i++)
+			{
+				fvwm_debug(__func__, " %s", m->wm_command[i]);
+			}
+			fvwm_debug(__func__, ",");
+		}
+		else
+		{
+			fvwm_debug(__func__, " no_wmc,");
+		}
+		fvwm_debug(__func__, " %d)\n\n", IS_NAME_CHANGED(m));
 	}
-	else
-	{
-		fvwm_debug(__func__, " no_wmc,");
-	}
-	fvwm_debug(__func__, " %d)\n\n", IS_NAME_CHANGED(m));
-#endif
 
 	if (client_id)
 	{
@@ -770,12 +770,14 @@ set_sm_properties(FSmcConn sm_conn, char *filename, char hint)
 		return;
 	}
 
-#ifdef FVWM_SM_DEBUG_PROTO
-	fvwm_debug(__func__,
-		   "[FVWM_SMDEBUG][set_sm_properties] state filename: %s%s\n",
-		   filename ? filename : "(null)",
-		   sm_conn ? "" : " - not connected");
-#endif
+	if (FVWM_SM_DEBUG_PROTO)
+	{
+		fvwm_debug(
+			__func__,
+			"[FVWM_SMDEBUG][set_sm_properties] state filename:"
+			" %s%s\n", filename ? filename : "(null)",
+			sm_conn ? "" : " - not connected");
+	}
 
 	if (!sm_conn)
 	{
@@ -967,9 +969,11 @@ callback_save_yourself2(FSmcConn sm_conn, FSmPointer client_data)
 	}
 
 	filename = get_unique_state_filename();
-#ifdef FVWM_SM_DEBUG_PROTO
-	fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_save_yourself2]\n");
-#endif
+	if (FVWM_SM_DEBUG_PROTO)
+	{
+		fvwm_debug(
+			__func__, "[FVWM_SMDEBUG][callback_save_yourself2]\n");
+	}
 
 	success = save_state_file(filename);
 	if (success)
@@ -994,42 +998,51 @@ callback_save_yourself(FSmcConn sm_conn, FSmPointer client_data,
 		return;
 	}
 
-#ifdef FVWM_SM_DEBUG_PROTO
-	fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_save_yourself] "
-		   "(save=%d, shut=%d, intr=%d, fast=%d)\n",
-		   save_style, shutdown, interact_style, fast);
-#endif
+	if (FVWM_SM_DEBUG_PROTO)
+	{
+		fvwm_debug(
+			__func__, "[FVWM_SMDEBUG][callback_save_yourself] "
+			"(save=%d, shut=%d, intr=%d, fast=%d)\n",
+			save_style, shutdown, interact_style, fast);
+	}
 
 	if (save_style == FSmSaveGlobal)
 	{
 		/* nothing to do */
-#ifdef FVWM_SM_DEBUG_PROTO
-		fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_save_yourself] "
-			   "Global Save type ... do nothing\n");
-#endif
+		if (FVWM_SM_DEBUG_PROTO)
+		{
+			fvwm_debug(
+				__func__,
+				"[FVWM_SMDEBUG][callback_save_yourself] "
+				"Global Save type ... do nothing\n");
+		}
 		FSmcSaveYourselfDone (sm_conn, True);
 		sent_save_done = 1;
 		return;
 
 	}
-#ifdef FVWM_SM_DEBUG_PROTO
-	fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_save_yourself] "
-		   "Both or Local save type, going to phase 2 ...");
-#endif
+	if (FVWM_SM_DEBUG_PROTO)
+	{
+		fvwm_debug(
+			__func__, "[FVWM_SMDEBUG][callback_save_yourself] "
+			"Both or Local save type, going to phase 2 ...");
+	}
 	if (!FSmcRequestSaveYourselfPhase2(
 		    sm_conn, callback_save_yourself2, NULL))
 	{
 		FSmcSaveYourselfDone (sm_conn, False);
 		sent_save_done = 1;
-#ifdef FVWM_SM_DEBUG_PROTO
-		fvwm_debug(__func__, " failed!\n");
-#endif
+		if (FVWM_SM_DEBUG_PROTO)
+		{
+			fvwm_debug(__func__, " failed!\n");
+		}
 	}
 	else
 	{
-#ifdef FVWM_SM_DEBUG_PROTO
-		fvwm_debug(__func__, " OK\n");
-#endif
+		if (FVWM_SM_DEBUG_PROTO)
+		{
+			fvwm_debug(__func__, " OK\n");
+		}
 		sent_save_done = 0;
 	}
 
@@ -1045,9 +1058,10 @@ callback_die(FSmcConn sm_conn, FSmPointer client_data)
 		return;
 	}
 
-#ifdef FVWM_SM_DEBUG_PROTO
-	fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_die]\n");
-#endif
+	if (FVWM_SM_DEBUG_PROTO)
+	{
+		fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_die]\n");
+	}
 
 	if (FSmcCloseConnection(sm_conn, 0, NULL) != FSmcClosedNow)
 	{
@@ -1069,9 +1083,11 @@ callback_save_complete(FSmcConn sm_conn, FSmPointer client_data)
 	{
 		return;
 	}
-#ifdef FVWM_SM_DEBUG_PROTO
-	fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_save_complete]\n");
-#endif
+	if (FVWM_SM_DEBUG_PROTO)
+	{
+		fvwm_debug(
+			__func__, "[FVWM_SMDEBUG][callback_save_complete]\n");
+	}
 
 	return;
 }
@@ -1084,9 +1100,12 @@ callback_shutdown_cancelled(FSmcConn sm_conn, FSmPointer client_data)
 		return;
 	}
 
-#ifdef FVWM_SM_DEBUG_PROTO
-	fvwm_debug(__func__, "[FVWM_SMDEBUG][callback_shutdown_cancelled]\n");
-#endif
+	if (FVWM_SM_DEBUG_PROTO)
+	{
+		fvwm_debug(
+			__func__,
+			"[FVWM_SMDEBUG][callback_shutdown_cancelled]\n");
+	}
 
 	if (!sent_save_done)
 	{
@@ -1325,7 +1344,7 @@ LoadWindowStates(char *filename)
 		    !strcmp(s1, "[REAL_STATE_FILENAME]"))
 		{
 			sscanf(s, "%*s %s", s1);
-			set_sm_properties(sm_conn, s1, FSmRestartIfRunning);
+			set_sm_properties(ssm_conn, s1, FSmRestartIfRunning);
 			set_real_state_filename(s1);
 		}
 		else if (!strcmp(s1, "[CLIENT]"))
@@ -1429,10 +1448,10 @@ LoadWindowStates(char *filename)
 
 			for (i = 0; i < sizeof(window_flags); i++)
 			{
-				unsigned int f;
-				sscanf(ts, "%02x ", &f);
+				unsigned int fl;
+				sscanf(ts, "%02x ", &fl);
 				((unsigned char *)&
-				 (matches[num_match-1].flags))[i] = f;
+				 (matches[num_match-1].flags))[i] = fl;
 				ts += 3;
 			}
 		}
@@ -1663,12 +1682,12 @@ RestartInSession (char *filename, Bool is_native, Bool _do_preserve_state)
 {
 	do_preserve_state = _do_preserve_state;
 
-	if (SessionSupport && sm_conn && is_native)
+	if (SessionSupport && ssm_conn && is_native)
 	{
 		going_to_restart = True;
 
 		save_state_file(filename);
-		set_sm_properties(sm_conn, filename, FSmRestartImmediately);
+		set_sm_properties(ssm_conn, filename, FSmRestartImmediately);
 
 		MoveViewport(monitor_get_current(), 0, 0, False);
 		Reborder();
@@ -1676,15 +1695,17 @@ RestartInSession (char *filename, Bool is_native, Bool _do_preserve_state)
 		CloseICCCM2();
 		XCloseDisplay(dpy);
 
-		if ((!FSmcCloseConnection(sm_conn, 0, NULL)) != FSmcClosedNow)
+		if ((!FSmcCloseConnection(ssm_conn, 0, NULL)) != FSmcClosedNow)
 		{
 			/* go a head any way ? */
 		}
 
-#ifdef  FVWM_SM_DEBUG_PROTO
-		fvwm_debug(__func__, "[FVWM_SMDEBUG]: Exiting, now SM must "
-			   "restart us.\n");
-#endif
+		if (FVWM_SM_DEBUG_PROTO)
+		{
+			fvwm_debug(
+				__func__, "[FVWM_SMDEBUG]: Exiting, now SM"
+				" must restart us.\n");
+		}
 		/* Close all my pipes */
 		module_kill_all();
 
@@ -1734,13 +1755,13 @@ SessionInit(void)
 		callbacks.save_complete.client_data =
 		callbacks.shutdown_cancelled.client_data = (FSmPointer) NULL;
 	SUPPRESS_UNUSED_VAR_WARNING(context);
-	sm_conn = FSmcOpenConnection(
+	ssm_conn = FSmcOpenConnection(
 		NULL, &context, FSmProtoMajor, FSmProtoMinor,
 		FSmcSaveYourselfProcMask | FSmcDieProcMask |
 		FSmcSaveCompleteProcMask | FSmcShutdownCancelledProcMask,
 		&callbacks, previous_sm_client_id, &sm_client_id, 4096,
 		error_string_ret);
-	if (!sm_conn)
+	if (!ssm_conn)
 	{
 		/*
 		  Don't annoy users which don't use a session manager
@@ -1752,22 +1773,27 @@ SessionInit(void)
 				   error_string_ret);
 		}
 		sm_fd = -1;
-#ifdef FVWM_SM_DEBUG_PROTO
-		fvwm_debug(__func__, "[FVWM_SMDEBUG] No SM connection\n");
-#endif
+		if (FVWM_SM_DEBUG_PROTO)
+		{
+			fvwm_debug(
+				__func__, "[FVWM_SMDEBUG] No SM connection\n");
+		}
 	}
 	else
 	{
-		sm_fd = FIceConnectionNumber(FSmcGetIceConnection(sm_conn));
-#ifdef FVWM_SM_DEBUG_PROTO
-		fvwm_debug(__func__, "[FVWM_SMDEBUG] Connectecd to a SM\n");
-#endif
+		sm_fd = FIceConnectionNumber(FSmcGetIceConnection(ssm_conn));
+		if (FVWM_SM_DEBUG_PROTO)
+		{
+			fvwm_debug(
+				__func__,
+				"[FVWM_SMDEBUG] Connectecd to a SM\n");
+		}
 		set_init_function_name(0, "SessionInitFunction");
 		set_init_function_name(1, "SessionRestartFunction");
 		set_init_function_name(2, "SessionExitFunction");
 		/* basically to restet our restart style hint after a
 		 * restart */
-		set_sm_properties(sm_conn, NULL, FSmRestartIfRunning);
+		set_sm_properties(ssm_conn, NULL, FSmRestartIfRunning);
 	}
 
 	return;
@@ -1783,20 +1809,23 @@ ProcessICEMsgs(void)
 		return;
 	}
 
-#ifdef FVWM_SM_DEBUG_PROTO
-	fvwm_debug(__func__, "[FVWM_SMDEBUG][ProcessICEMsgs] %i\n",
-		   (int)sm_fd);
-#endif
+	if (FVWM_SM_DEBUG_PROTO)
+	{
+		fvwm_debug(
+			__func__, "[FVWM_SMDEBUG][ProcessICEMsgs] %i\n",
+			(int)sm_fd);
+	}
 	if (sm_fd < 0)
 	{
 		return;
 	}
-	status = FIceProcessMessages(FSmcGetIceConnection(sm_conn), NULL, NULL);
+	status = FIceProcessMessages(
+		FSmcGetIceConnection(ssm_conn), NULL, NULL);
 	if (status == FIceProcessMessagesIOError)
 	{
 		fvwm_debug(__func__,
 			   "Connection to session manager lost\n");
-		sm_conn = NULL;
+		ssm_conn = NULL;
 		sm_fd = -1;
 	}
 
@@ -1851,14 +1880,14 @@ Bool quit_session(void)
 	{
 		return False;
 	}
-	if (!sm_conn)
+	if (!ssm_conn)
 	{
 		return False;
 	}
 
 	FSmcRequestSaveYourself(
-		sm_conn, FSmSaveLocal, True /* shutdown */, FSmInteractStyleAny,
-		False /* fast */, True /* global */);
+		ssm_conn, FSmSaveLocal, True /* shutdown */,
+		FSmInteractStyleAny, False /* fast */, True /* global */);
 	return True;
 
 	/* migo: xsm does not support RequestSaveYourself, but supports
@@ -1879,14 +1908,14 @@ Bool save_session(void)
 	{
 		return False;
 	}
-	if (!sm_conn)
+	if (!ssm_conn)
 	{
 		return False;
 	}
 
 	FSmcRequestSaveYourself(
-		sm_conn, FSmSaveBoth, False /* shutdown */, FSmInteractStyleAny,
-		False /* fast */, True /* global */);
+		ssm_conn, FSmSaveBoth, False /* shutdown */,
+		FSmInteractStyleAny, False /* fast */, True /* global */);
 	return True;
 
 	/* migo: xsm does not support RequestSaveYourself, but supports
@@ -1908,13 +1937,13 @@ Bool save_quit_session(void)
 		return False;
 	}
 
-	if (!sm_conn)
+	if (!ssm_conn)
 	{
 		return False;
 	}
 
 	FSmcRequestSaveYourself(
-		sm_conn, FSmSaveBoth, True /* shutdown */, FSmInteractStyleAny,
+		ssm_conn, FSmSaveBoth, True /* shutdown */, FSmInteractStyleAny,
 		False /* fast */, True /* global */);
 	return True;
 
