@@ -28,6 +28,7 @@
 #include "libs/FScreen.h"
 #include "fvwm.h"
 #include "externs.h"
+#include "cmdparser.h"
 #include "execcontext.h"
 #include "expand.h"
 #include "cursor.h"
@@ -64,17 +65,20 @@
 	do {								    \
 		char	*cmd;						    \
 		xasprintf(&cmd, "GotoDesk %s 0 %d", (m)->si->name, (d));    \
-		execute_function_override_window(NULL, NULL, cmd, 0, NULL); \
+		execute_function_override_window(			    \
+			NULL, NULL, cmd, NULL, 0, NULL);		    \
 		free(cmd);						    \
 	} while (0)
 
 #define CMD_MOVE_SCREEN_DESK(m, d)					    \
 	do {								    \
 		char	*cmd;						    \
-		xasprintf(&cmd, "All (!Screen %s, Desk %d, !CirculateHit) " \
+		xasprintf(&cmd,						    \
+		    "All (!Screen %s, Desk %d, CurrentPage, !CirculateHit) "\
 		    "MoveToPage %s $[w.pagex] $[w.pagey]",		    \
 		    (m)->si->name, (d), (m)->si->name);			    \
-		execute_function_override_window(NULL, NULL, cmd, 0, NULL); \
+		execute_function_override_window(			    \
+			NULL, NULL, cmd, NULL, 0, NULL);		    \
 		free(cmd);                                                  \
 	} while (0)
 
@@ -438,10 +442,11 @@ static void unmap_window(FvwmWindow *t)
 	{
 		XUnmapWindow(dpy,FW_W_FRAME(t));
 		border_undraw_decorations(t);
-#ifdef ICCCM2_UNMAP_WINDOW_PATCH
-		/* this is required by the ICCCM2 */
-		XUnmapWindow(dpy, FW_W(t));
-#endif
+		if (ICCCM2_UNMAP_WINDOW_PATCH)
+		{
+			/* this is required by the ICCCM2 */
+			XUnmapWindow(dpy, FW_W(t));
+		}
 		if (!Scr.bo.do_enable_ewmh_iconic_state_workaround)
 		{
 			SetMapStateProp(t, IconicState);
@@ -501,10 +506,11 @@ static void map_window(FvwmWindow *t)
 		XMapWindow(dpy, FW_W_FRAME(t));
 		XMapWindow(dpy, FW_W_PARENT(t));
 		XMapSubwindows(dpy, FW_W_FRAME(t));
-#ifdef ICCCM2_UNMAP_WINDOW_PATCH
-		/* this is required by the ICCCM2 */
-		XMapWindow(dpy, FW_W(t));
-#endif
+		if (ICCCM2_UNMAP_WINDOW_PATCH)
+		{
+			/* this is required by the ICCCM2 */
+			XMapWindow(dpy, FW_W(t));
+		}
 		if (!Scr.bo.do_enable_ewmh_iconic_state_workaround)
 		{
 			SetMapStateProp(t, NormalState);
@@ -626,8 +632,8 @@ static void MapDesk(struct monitor *m, int desk, Bool grab)
 					/* execute_function_override_window()
 					 * will expand cmd's variables.
 					 */
-					execute_function_override_window(NULL,
-					    NULL, cmd, 0, t);
+					execute_function_override_window(
+						NULL, NULL, cmd, NULL, 0, t);
 					free(cmd);
 
 					/* No need to map the window as it's
@@ -711,9 +717,8 @@ static void MapDesk(struct monitor *m, int desk, Bool grab)
  * -1: no need to call the function again before a new event arrives
  */
 int HandlePaging(
-	XEvent *pev, int HorWarpSize, int VertWarpSize, int *xl, int *yt,
-	int *delta_x, int *delta_y, Bool Grab, Bool fLoop,
-	Bool do_continue_previous, int delay)
+	XEvent *pev, position warp_size, position *p, position *delta,
+	Bool Grab, Bool fLoop, Bool do_continue_previous, int delay)
 {
 	static int add_time = 0;
 	int x,y;
@@ -731,8 +736,8 @@ int HandlePaging(
 	mheight = monitor_get_all_heights();
 
 
-	*delta_x = 0;
-	*delta_y = 0;
+	delta->x = 0;
+	delta->y = 0;
 	if (!is_timestamp_valid && do_continue_previous)
 	{
 		/* don't call me again until something has happened */
@@ -752,7 +757,7 @@ int HandlePaging(
 			return -1;
 		}
 	}
-	if (delay < 0 || (HorWarpSize == 0 && VertWarpSize==0))
+	if (delay < 0 || (warp_size.x == 0 && warp_size.y==0))
 	{
 		is_timestamp_valid = False;
 		add_time = 0;
@@ -862,104 +867,104 @@ int HandlePaging(
 
 	if (x <= (m->si->x + edge_thickness))
 	{
-		*delta_x = -HorWarpSize;
+		delta->x = -warp_size.x;
 	}
 	else if (x >= (m->si->x + m->si->w) - edge_thickness)
 	{
-		*delta_x = HorWarpSize;
+		delta->x = warp_size.x;
 	}
 	else
 	{
-		*delta_x = 0;
+		delta->x = 0;
 	}
 	if (m->virtual_scr.VxMax == 0)
 	{
-		*delta_x = 0;
+		delta->x = 0;
 	}
 	if (y <= (m->si->y + edge_thickness))
 	{
-		*delta_y = -VertWarpSize;
+		delta->y = -warp_size.y;
 	}
 	else if (y >= (m->si->y + m->si->h) - edge_thickness)
 	{
-		*delta_y = VertWarpSize;
+		delta->y = warp_size.y;
 	}
 	else
 	{
-		*delta_y = 0;
+		delta->y = 0;
 	}
 	if (m->virtual_scr.VyMax == 0)
 	{
-		*delta_y = 0;
+		delta->y = 0;
 	}
 
 	/* Ouch! lots of bounds checking */
-	if (m->virtual_scr.Vx + *delta_x < 0)
+	if (m->virtual_scr.Vx + delta->x < 0)
 	{
 		if (!(Scr.flags.do_edge_wrap_x))
 		{
-			*delta_x = -m->virtual_scr.Vx;
-			*xl = x - *delta_x;
+			delta->x = -m->virtual_scr.Vx;
+			p->x = x - delta->x;
 		}
 		else
 		{
-			*delta_x += m->virtual_scr.VxMax + mwidth;
-			*xl = x + *delta_x % mwidth + HorWarpSize;
+			delta->x += m->virtual_scr.VxMax + mwidth;
+			p->x = x + delta->x % mwidth + warp_size.x;
 		}
 	}
-	else if (m->virtual_scr.Vx + *delta_x > m->virtual_scr.VxMax)
+	else if (m->virtual_scr.Vx + delta->x > m->virtual_scr.VxMax)
 	{
 		if (!(Scr.flags.do_edge_wrap_x))
 		{
-			*delta_x = m->virtual_scr.VxMax - m->virtual_scr.Vx;
-			*xl = x - *delta_x;
+			delta->x = m->virtual_scr.VxMax - m->virtual_scr.Vx;
+			p->x = x - delta->x;
 		}
 		else
 		{
-			*delta_x -= m->virtual_scr.VxMax + mwidth;
-			*xl = x + *delta_x % mwidth - HorWarpSize;
+			delta->x -= m->virtual_scr.VxMax + mwidth;
+			p->x = x + delta->x % mwidth - warp_size.x;
 		}
 	}
 	else
 	{
-		*xl = x - *delta_x;
+		p->x = x - delta->x;
 	}
 
-	if (m->virtual_scr.Vy + *delta_y < 0)
+	if (m->virtual_scr.Vy + delta->y < 0)
 	{
 		if (!(Scr.flags.do_edge_wrap_y))
 		{
-			*delta_y = -m->virtual_scr.Vy;
-			*yt = y - *delta_y;
+			delta->y = -m->virtual_scr.Vy;
+			p->y = y - delta->y;
 		}
 		else
 		{
-			*delta_y += m->virtual_scr.VyMax + mheight;
-			*yt = y + *delta_y % mheight + VertWarpSize;
+			delta->y += m->virtual_scr.VyMax + mheight;
+			p->y = y + delta->y % mheight + warp_size.y;
 		}
 	}
-	else if (m->virtual_scr.Vy + *delta_y > m->virtual_scr.VyMax)
+	else if (m->virtual_scr.Vy + delta->y > m->virtual_scr.VyMax)
 	{
 		if (!(Scr.flags.do_edge_wrap_y))
 		{
-			*delta_y = m->virtual_scr.VyMax - m->virtual_scr.Vy;
-			*yt = y - *delta_y;
+			delta->y = m->virtual_scr.VyMax - m->virtual_scr.Vy;
+			p->y = y - delta->y;
 		}
 		else
 		{
-			*delta_y -= m->virtual_scr.VyMax + mheight;
-			*yt = y + *delta_y % mheight - VertWarpSize;
+			delta->y -= m->virtual_scr.VyMax + mheight;
+			p->y = y + delta->y % mheight - warp_size.y;
 		}
 	}
 	else
 	{
-		*yt = y - *delta_y;
+		p->y = y - delta->y;
 	}
 
 	/* Check for paging -- and don't warp the pointer. */
 	is_timestamp_valid = False;
 	add_time = 0;
-	if (*delta_x == 0 && *delta_y == 0)
+	if (delta->x == 0 && delta->y == 0)
 	{
 		return 0;
 	}
@@ -968,24 +973,24 @@ int HandlePaging(
 	/* make sure the pointer isn't warped into the panframes */
 	/* Handle global/per-monitor separately.*/
 	if (monitor_mode == MONITOR_TRACKING_G && !is_tracking_shared) {
-		if (*xl < edge_thickness)
-			*xl = edge_thickness;
-		if (*yt < edge_thickness)
-			*yt = edge_thickness;
-		if (*xl >= mwidth - edge_thickness)
-			*xl = mwidth - edge_thickness -1;
-		if (*yt >= mheight - edge_thickness)
-			*yt = mheight - edge_thickness -1;
+		if (p->x < edge_thickness)
+			p->x = edge_thickness;
+		if (p->y < edge_thickness)
+			p->y = edge_thickness;
+		if (p->x >= mwidth - edge_thickness)
+			p->x = mwidth - edge_thickness -1;
+		if (p->y >= mheight - edge_thickness)
+			p->y = mheight - edge_thickness -1;
 	} else {
 		/* Per-monitor warping of the cursor. */
-		if (*xl <= (m->si->x + edge_thickness))
-			*xl = m->si->x + edge_thickness;
-		if (*yt <= (m->si->y + edge_thickness))
-			*yt = m->si->y + edge_thickness;
-		if (*xl >= (m->si->x + m->si->w) - edge_thickness)
-			*xl = (m->si->x + m->si->w) - edge_thickness -1;
-		if (*yt >= (m->si->y + m->si->h) - edge_thickness)
-			*yt = (m->si->y + m->si->h) - edge_thickness -1;
+		if (p->x <= (m->si->x + edge_thickness))
+			p->x = m->si->x + edge_thickness;
+		if (p->y <= (m->si->y + edge_thickness))
+			p->y = m->si->y + edge_thickness;
+		if (p->x >= (m->si->x + m->si->w) - edge_thickness)
+			p->x = (m->si->x + m->si->w) - edge_thickness -1;
+		if (p->y >= (m->si->y + m->si->h) - edge_thickness)
+			p->y = (m->si->y + m->si->h) - edge_thickness -1;
 	}
 	if (Grab)
 	{
@@ -993,16 +998,16 @@ int HandlePaging(
 	}
 	/* Turn off the rubberband if its on */
 	switch_move_resize_grid(False);
-	FWarpPointer(dpy,None,Scr.Root,0,0,0,0,*xl,*yt);
-	MoveViewport(m, m->virtual_scr.Vx + *delta_x,
-			m->virtual_scr.Vy + *delta_y,False);
+	FWarpPointer(dpy,None,Scr.Root,0,0,0,0,p->x,p->y);
+	MoveViewport(m, m->virtual_scr.Vx + delta->x,
+			m->virtual_scr.Vy + delta->y,False);
 	if (FQueryPointer(
-		    dpy, Scr.Root, &JunkRoot, &JunkChild, xl, yt, &JunkX,
-		    &JunkY, &JunkMask) == False)
+		    dpy, Scr.Root, &JunkRoot, &JunkChild, &(p->x), &(p->y),
+		    &JunkX, &JunkY, &JunkMask) == False)
 	{
 		/* pointer is on a different screen */
-		*xl = 0;
-		*yt = 0;
+		p->x = 0;
+		p->y = 0;
 	}
 	if (Grab)
 	{
@@ -1388,14 +1393,187 @@ Bool is_pan_frame(Window w)
  *  Moves the viewport within the virtual desktop
  *
  */
+static void move_viewport_delta(
+	struct monitor *m, position delta, position page_tl, position page_br,
+	int do_broadcast)
+{
+	struct monitor	*mloop;
+	FvwmWindow *t, *t1;
+	int txl, txr, tyt, tyb;
+
+	if (do_broadcast)
+	{
+		TAILQ_FOREACH(mloop, &monitor_q, entry)
+		{
+			if (is_tracking_shared) {
+			    if ((!m->virtual_scr.is_swapping ||
+			        !mloop->virtual_scr.is_swapping) && mloop != m)
+					continue;
+			}
+			BroadcastPacket(
+				M_NEW_PAGE, 8,
+				(long)mloop->virtual_scr.Vx,
+				(long)mloop->virtual_scr.Vy,
+				(long)mloop->virtual_scr.CurrentDesk,
+				(long) monitor_get_all_widths(),
+				(long) monitor_get_all_heights(),
+				(long)((mloop->virtual_scr.VxMax /
+					monitor_get_all_widths()) + 1),
+				(long)((mloop->virtual_scr.VyMax /
+					monitor_get_all_heights()) + 1),
+				(long)mloop->si->rr_output);
+		}
+	}
+	/*
+	 * RBW - 11/13/1998      - new:  chase the chain
+	 * bidirectionally, all at once! The idea is to move the
+	 * windows that are moving out of the viewport from the bottom
+	 * of the stacking order up, to minimize the expose-redraw
+	 * overhead. Windows that will be moving into view will be
+	 * moved top down, for the same reason. Use the new
+	 * stacking-order chain, rather than the old last-focused
+	 * chain.
+	 *
+	 * domivogt (29-Nov-1999): It's faster to first map windows
+	 * top to bottom and then unmap windows bottom up.
+	 */
+	/* TA: 2020-01-21:  This change of skipping monitors will
+	 * break using 'Scroll' and __drag_viewport().  We need to
+	 * ensure we handle this case properly.
+	 */
+	t = get_next_window_in_stack_ring(&Scr.FvwmRoot);
+	while (t != &Scr.FvwmRoot)
+	{
+		if (NOT_GLOBALLY_ACTIVE(t->m, m)) {
+			/*  Bump to next win...  */
+			t = get_next_window_in_stack_ring(t);
+			continue;
+		}
+		/*
+		 * If the window is moving into the viewport...
+		 */
+		txl = t->g.frame.x;
+		tyt = t->g.frame.y;
+		txr = t->g.frame.x + t->g.frame.width - 1;
+		tyb = t->g.frame.y + t->g.frame.height - 1;
+		if (is_window_sticky_across_pages(t) &&
+		    !IS_VIEWPORT_MOVED(t))
+		{
+			/* the absolute position has changed */
+			t->g.normal.x -= delta.x;
+			t->g.normal.y -= delta.y;
+			t->g.max.x -= delta.x;
+			t->g.max.y -= delta.y;
+			/*  Block double move.  */
+			SET_VIEWPORT_MOVED(t, 1);
+		}
+		if ((txr >= page_tl.x && txl <= page_br.x
+		     && tyb >= page_tl.y && tyt <= page_br.y)
+		    && !IS_VIEWPORT_MOVED(t)
+		    && !IS_WINDOW_BEING_MOVED_OPAQUE(t))
+		{
+			/*  Block double move.  */
+			SET_VIEWPORT_MOVED(t, 1);
+			/* If the window is iconified, and sticky Icons is set,
+			 * then the window should essentially be sticky */
+			if (!is_window_sticky_across_pages(t))
+			{
+				if (IS_ICONIFIED(t))
+				{
+					modify_icon_position(
+						t, delta.x, delta.y);
+					move_icon_to_position(t);
+					if (do_broadcast)
+					{
+						broadcast_icon_geometry(
+							t, False);
+					}
+				}
+				frame_setup_window(
+					t, t->g.frame.x + delta.x,
+					t->g.frame.y + delta.y,
+					t->g.frame.width,
+					t->g.frame.height, False);
+			}
+		}
+		/*  Bump to next win...  */
+		t = get_next_window_in_stack_ring(t);
+	}
+	t1 = get_prev_window_in_stack_ring(&Scr.FvwmRoot);
+	while (t1 != &Scr.FvwmRoot)
+	{
+		if (NOT_GLOBALLY_ACTIVE(t1->m, m)) {
+			/*  Bump to next win...  */
+			t1 = get_prev_window_in_stack_ring(t1);
+			continue;
+		}
+		/*
+		 *If the window is not moving into the viewport...
+		 */
+		SET_VIEWPORT_MOVED(t, 1);
+		txl = t1->g.frame.x;
+		tyt = t1->g.frame.y;
+		txr = t1->g.frame.x + t1->g.frame.width - 1;
+		tyb = t1->g.frame.y + t1->g.frame.height - 1;
+		if (! (txr >= page_tl.x && txl <= page_br.x
+		       && tyb >= page_tl.y && tyt <= page_br.y)
+		    && !IS_VIEWPORT_MOVED(t1)
+		    && !IS_WINDOW_BEING_MOVED_OPAQUE(t1))
+		{
+			/* If the window is iconified, and sticky Icons is set,
+			 * then the window should essentially be sticky */
+			if (!is_window_sticky_across_pages(t1))
+			{
+				if (IS_ICONIFIED(t1))
+				{
+					modify_icon_position(
+						t1, delta.x, delta.y);
+					move_icon_to_position(t1);
+					if (do_broadcast)
+					{
+						broadcast_icon_geometry(
+							t1, False);
+					}
+				}
+				frame_setup_window(
+					t1, t1->g.frame.x + delta.x,
+					t1->g.frame.y + delta.y,
+					t1->g.frame.width,
+					t1->g.frame.height, False);
+			}
+		}
+		/*  Bump to next win...  */
+		t1 = get_prev_window_in_stack_ring(t1);
+	}
+	for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+	{
+		if (NOT_GLOBALLY_ACTIVE(t->m, m))
+			continue;
+
+		if (IS_VIEWPORT_MOVED(t))
+		{
+			/* Clear double move blocker. */
+			SET_VIEWPORT_MOVED(t, 0);
+		}
+		/* If its an icon, and its sticking, autoplace it so that it
+		 * doesn't wind up on top of a stationary icon */
+		if (is_window_sticky_across_pages(t) &&
+		    IS_ICONIFIED(t) && !IS_ICON_MOVED(t) &&
+		    !IS_ICON_UNMAPPED(t))
+		{
+			AutoPlaceIcon(t, NULL, True);
+		}
+	}
+
+	return;
+}
+
 void MoveViewport(struct monitor *m, int newx, int newy, Bool grab)
 {
 	struct monitor *m_loop;
-	FvwmWindow *t, *t1;
-	int deltax,deltay;
-	int PageTop, PageLeft;
-	int PageBottom, PageRight;
-	int txl, txr, tyt, tyb;
+	position delta;
+	position page_tl;
+	position page_br;
 	bool global = (monitor_mode == MONITOR_TRACKING_G && !is_tracking_shared);
 
 	if (grab)
@@ -1418,18 +1596,18 @@ void MoveViewport(struct monitor *m, int newx, int newy, Bool grab)
 	{
 		newy = 0;
 	}
-	deltay = m->virtual_scr.Vy - newy;
-	deltax = m->virtual_scr.Vx - newx;
+	delta.y = m->virtual_scr.Vy - newy;
+	delta.x = m->virtual_scr.Vx - newx;
 	/*
 	  Identify the bounding rectangle that will be moved into
 	  the viewport.
 	*/
-	PageBottom    =  monitor_get_all_heights() - deltay - 1;
-	PageRight     =  monitor_get_all_widths()  - deltax - 1;
-	PageTop       =  0 - deltay;
-	PageLeft      =  0 - deltax;
+	page_br.y =  monitor_get_all_heights() - delta.y - 1;
+	page_br.x =  monitor_get_all_widths()  - delta.x - 1;
+	page_tl.y =  -delta.y;
+	page_tl.x =  -delta.x;
 
-	if (deltax || deltay)
+	if (delta.x || delta.y)
 	{
 		m->virtual_scr.prev_page_x = m->virtual_scr.Vx;
 		m->virtual_scr.prev_page_y = m->virtual_scr.Vy;
@@ -1446,159 +1624,9 @@ void MoveViewport(struct monitor *m, int newx, int newy, Bool grab)
 	 */
 	monitor_assign_virtual(m);
 
-	if (deltax || deltay)
+	if (delta.x || delta.y)
 	{
-		struct monitor	*mloop;
-
-		TAILQ_FOREACH(mloop, &monitor_q, entry) {
-			BroadcastPacket(
-				M_NEW_PAGE, 8,
-				(long)mloop->virtual_scr.Vx,
-				(long)mloop->virtual_scr.Vy,
-				(long)mloop->virtual_scr.CurrentDesk,
-				(long) monitor_get_all_widths(),
-				(long) monitor_get_all_heights(),
-				(long)((mloop->virtual_scr.VxMax / monitor_get_all_widths()) + 1),
-				(long)((mloop->virtual_scr.VyMax / monitor_get_all_heights()) + 1),
-				(long)mloop->si->rr_output);
-		}
-		/*
-		 * RBW - 11/13/1998      - new:  chase the chain
-		 * bidirectionally, all at once! The idea is to move the
-		 * windows that are moving out of the viewport from the bottom
-		 * of the stacking order up, to minimize the expose-redraw
-		 * overhead. Windows that will be moving into view will be
-		 * moved top down, for the same reason. Use the new
-		 * stacking-order chain, rather than the old last-focused
-		 * chain.
-		 *
-		 * domivogt (29-Nov-1999): It's faster to first map windows
-		 * top to bottom and then unmap windows bottom up.
-		 */
-		/* TA: 2020-01-21:  This change of skipping monitors will
-		 * break using 'Scroll' and __drag_viewport().  We need to
-		 * ensure we handle this case properly.
-		 */
-		t = get_next_window_in_stack_ring(&Scr.FvwmRoot);
-		while (t != &Scr.FvwmRoot)
-		{
-			if (NOT_GLOBALLY_ACTIVE(t->m, m)) {
-				/*  Bump to next win...  */
-				t = get_next_window_in_stack_ring(t);
-				continue;
-			}
-			/*
-			 * If the window is moving into the viewport...
-			 */
-			txl = t->g.frame.x;
-			tyt = t->g.frame.y;
-			txr = t->g.frame.x + t->g.frame.width - 1;
-			tyb = t->g.frame.y + t->g.frame.height - 1;
-			if (is_window_sticky_across_pages(t) &&
-			    !IS_VIEWPORT_MOVED(t))
-			{
-				/* the absolute position has changed */
-				t->g.normal.x -= deltax;
-				t->g.normal.y -= deltay;
-				t->g.max.x -= deltax;
-				t->g.max.y -= deltay;
-				/*  Block double move.  */
-				SET_VIEWPORT_MOVED(t, 1);
-			}
-			if ((txr >= PageLeft && txl <= PageRight
-			     && tyb >= PageTop && tyt <= PageBottom)
-			    && !IS_VIEWPORT_MOVED(t)
-			    && !IS_WINDOW_BEING_MOVED_OPAQUE(t))
-			{
-				/*  Block double move.  */
-				SET_VIEWPORT_MOVED(t, 1);
-				/* If the window is iconified, and sticky
-				 * Icons is set, then the window should
-				 * essentially be sticky */
-				if (!is_window_sticky_across_pages(t))
-				{
-					if (IS_ICONIFIED(t))
-					{
-						modify_icon_position(
-							t, deltax, deltay);
-						move_icon_to_position(t);
-						broadcast_icon_geometry(
-							t, False);
-					}
-					frame_setup_window(
-						t, t->g.frame.x + deltax,
-						t->g.frame.y + deltay,
-						t->g.frame.width,
-						t->g.frame.height, False);
-				}
-			}
-			/*  Bump to next win...  */
-			t = get_next_window_in_stack_ring(t);
-		}
-		t1 = get_prev_window_in_stack_ring(&Scr.FvwmRoot);
-		while (t1 != &Scr.FvwmRoot)
-		{
-			if (NOT_GLOBALLY_ACTIVE(t1->m, m)) {
-				/*  Bump to next win...  */
-				t1 = get_prev_window_in_stack_ring(t1);
-				continue;
-			}
-			/*
-			 *If the window is not moving into the viewport...
-			 */
-			SET_VIEWPORT_MOVED(t, 1);
-			txl = t1->g.frame.x;
-			tyt = t1->g.frame.y;
-			txr = t1->g.frame.x + t1->g.frame.width - 1;
-			tyb = t1->g.frame.y + t1->g.frame.height - 1;
-			if (! (txr >= PageLeft && txl <= PageRight
-			       && tyb >= PageTop && tyt <= PageBottom)
-			    && !IS_VIEWPORT_MOVED(t1)
-			    && !IS_WINDOW_BEING_MOVED_OPAQUE(t1))
-			{
-				/* If the window is iconified, and sticky
-				 * Icons is set, then the window should
-				 * essentially be sticky */
-				if (!is_window_sticky_across_pages(t1))
-				{
-					if (IS_ICONIFIED(t1))
-					{
-						modify_icon_position(
-							t1, deltax, deltay);
-						move_icon_to_position(t1);
-						broadcast_icon_geometry(
-							t1, False);
-					}
-					frame_setup_window(
-						t1, t1->g.frame.x + deltax,
-						t1->g.frame.y + deltay,
-						t1->g.frame.width,
-						t1->g.frame.height, False);
-				}
-			}
-			/*  Bump to next win...  */
-			t1 = get_prev_window_in_stack_ring(t1);
-		}
-		for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
-		{
-			if (NOT_GLOBALLY_ACTIVE(t->m, m))
-				continue;
-
-			if (IS_VIEWPORT_MOVED(t))
-			{
-				/* Clear double move blocker. */
-				SET_VIEWPORT_MOVED(t, 0);
-			}
-			/* If its an icon, and its sticking, autoplace it so
-			 * that it doesn't wind up on top a a stationary
-			 * icon */
-			if (is_window_sticky_across_pages(t) &&
-			    IS_ICONIFIED(t) && !IS_ICON_MOVED(t) &&
-			    !IS_ICON_UNMAPPED(t))
-			{
-				AutoPlaceIcon(t, NULL, True);
-			}
-		}
+		move_viewport_delta(m, delta, page_tl, page_br, 1);
 	}
 	checkPanFrames(m);
 
@@ -1640,6 +1668,8 @@ done:
 void goto_desk(int desk, struct monitor *m)
 {
 	struct monitor	*m2 = NULL;
+	bool has_run_globally = false;
+
 	/* RBW - the unmapping operations are now removed to their own
 	 * functions so they can also be used by the new GoToDeskAndPage
 	 * command. */
@@ -1684,17 +1714,54 @@ void goto_desk(int desk, struct monitor *m)
 		}
 
 		TAILQ_FOREACH(m2, &monitor_q, entry) {
+			/* In global mode, only do this once. */
+			if (monitor_mode == MONITOR_TRACKING_G &&
+			    has_run_globally)
+				break;
+
+			if (is_tracking_shared && m == m2) {
+			    BroadcastPacket(M_NEW_DESK, 2,
+				(long)m2->virtual_scr.CurrentDesk,
+				(long)m2->si->rr_output);
+
+				goto done;
+			}
+
+			/* If we're swapping a desktop between monitors for
+			 * shared mode, only do this when the is_swapping flag
+			 * is true, otherwise events would be raised for a
+			 * new_desk for monitors/desks which have not changed.
+			 */
+			if (is_tracking_shared) {
+			    if ((!m->virtual_scr.is_swapping ||
+			        !m2->virtual_scr.is_swapping) && m2 != m)
+					continue;
+
+			    BroadcastPacket(M_NEW_DESK, 2,
+				(long)m2->virtual_scr.CurrentDesk,
+				(long)m2->si->rr_output);
+
+				goto done;
+			}
 			if (m != m2) {
 				m2->Desktops = m->Desktops;
 				if (!is_tracking_shared) {
 					m2->virtual_scr.CurrentDesk =
 						m->virtual_scr.CurrentDesk;
 				}
-			}
 
-			BroadcastPacket(M_NEW_DESK, 2, (long)m2->virtual_scr.CurrentDesk,
+				BroadcastPacket(M_NEW_DESK, 2,
+					(long)m2->virtual_scr.CurrentDesk,
 					(long)m2->si->rr_output);
 
+				if (monitor_mode == MONITOR_TRACKING_G &&
+				    !has_run_globally) {
+					has_run_globally = true;
+					goto done;
+				}
+			}
+done:
+
 			/* FIXME: domivogt (22-Apr-2000): Fake a 'restack' for sticky
 			 * window upon desk change.  This is a workaround for a
 			 * problem in FvwmPager: The pager has a separate 'root'
@@ -1761,41 +1828,38 @@ void do_move_window_to_desk(FvwmWindow *fw, int desk)
 	return;
 }
 
-Bool get_page_arguments(FvwmWindow *fw, char *action, int *page_x, int *page_y, struct monitor **mret)
+Bool get_page_arguments(
+	FvwmWindow *fw, char *action, int *page_x, int *page_y,
+	struct monitor **mret)
 {
 	int val[2];
 	int suffix[2];
 	int mw, mh;
 	char *token;
-	char *taction, *action_cpy, *action_cpy_start;
+	char *taction;
+	char *next;
 	int wrapx;
 	int wrapy;
 	int limitdeskx;
 	int limitdesky;
-	struct monitor	*m, *m_use;
+	struct monitor	*m;
 
 	wrapx = 0;
 	wrapy = 0;
 	limitdeskx = 1;
 	limitdesky = 1;
 
-	m_use = (fw && fw->m) ? fw->m : monitor_get_current();
-	action_cpy = strdup(action);
-	action_cpy_start = action_cpy;
-	token = PeekToken(action_cpy, &action_cpy);
-	free(action_cpy_start);
-
-	if (token == NULL)
-		return (False);
-
+	token = PeekToken(action, &next);
 	m = monitor_resolve_name(token);
-	if (strcmp(m->si->name, token) != 0)
-		m = m_use;
+	if (m != NULL)
+	{
+		action = next;
+	}
 	else
-		PeekToken(action, &action);
-
-	if (mret != NULL)
-		*mret = m;
+	{
+		m = (fw && fw->m) ? fw->m : monitor_get_current();
+	}
+	*mret = m;
 
 	mw = monitor_get_all_widths();
 	mh = monitor_get_all_heights();
@@ -1850,7 +1914,7 @@ Bool get_page_arguments(FvwmWindow *fw, char *action, int *page_x, int *page_y,
 	if (GetSuffixedIntegerArguments(action, NULL, val, 2, "pw", suffix) !=
 	    2)
 	{
-		return 0;
+		return False;
 	}
 
 	if (suffix[0] == 1)
@@ -1978,7 +2042,8 @@ void parse_edge_leave_command(char *action, int type)
                 option = PeekToken(action, &action);
 
                 m = monitor_resolve_name(option);
-                if (strcmp(m->si->name, option) != 0) {
+                if (m == NULL)
+		{
                         fvwm_debug(__func__,
                                    "Invalid screen: %s", option);
                         return;
@@ -2202,14 +2267,11 @@ void CMD_EdgeResistance(F_CMD_ARGS)
 			   " instead:\n%s\n%s\n%s\n", cmd, stylecmd,
 			   stylecmd2);
 		execute_function(
-			cond_rc, exc, cmd,
-			FUNC_DONT_REPEAT | FUNC_DONT_EXPAND_COMMAND);
+			cond_rc, exc, cmd, pc, FUNC_DONT_EXPAND_COMMAND);
 		execute_function(
-			cond_rc, exc, stylecmd,
-			FUNC_DONT_REPEAT | FUNC_DONT_EXPAND_COMMAND);
+			cond_rc, exc, stylecmd, pc, FUNC_DONT_EXPAND_COMMAND);
 		execute_function(
-			cond_rc, exc, stylecmd2,
-			FUNC_DONT_REPEAT | FUNC_DONT_EXPAND_COMMAND);
+			cond_rc, exc, stylecmd2, pc, FUNC_DONT_EXPAND_COMMAND);
 	}
 	else
 	{
@@ -2224,14 +2286,25 @@ void CMD_EdgeResistance(F_CMD_ARGS)
 void CMD_DesktopConfiguration(F_CMD_ARGS)
 {
 	FvwmWindow	*t;
-	struct monitor	*m_loop, *m = monitor_get_current();
+	struct monitor	*m = monitor_get_current();
+	extern Bool	 Restarting;
 
 	if (action == NULL) {
 		fvwm_debug(__func__, "action is required");
 		return;
 	}
+	if ((Restarting && !Scr.flags.are_windows_captured) &&
+	    strcasecmp(action, "shared") == 0) {
+		fvwm_debug(__func__, "Restarting and in shared mode.");
+		if (!is_tracking_shared) {
+			is_tracking_shared = true;
+			monitor_mode = MONITOR_TRACKING_G;
+		}
+		goto update;
+	}
 
 	if (strcasecmp(action, "global") == 0) {
+		struct monitor	*m_loop;
 		/* If we're switching to global mode after coming out of per-monitor
 		 * mode, the desks won't be the same.  Fix this by switching
 		 * other monitor desks to be the same as the current monitor.
@@ -2247,7 +2320,8 @@ void CMD_DesktopConfiguration(F_CMD_ARGS)
 				m->virtual_scr.Vx / monitor_get_all_widths(),
 				m->virtual_scr.Vy / monitor_get_all_heights());
 
-			execute_function_override_window(NULL, NULL, cmd, 0, NULL);
+			execute_function_override_window(
+				NULL, NULL, cmd, NULL, 0, NULL);
 			free(cmd);
 		}
 		monitor_mode = MONITOR_TRACKING_G;
@@ -2383,7 +2457,7 @@ update:
 	raisePanFrames();
 
 	for (t = Scr.FvwmRoot.next; t; t = t->next)
-		UPDATE_FVWM_SCREEN(t);
+		update_fvwm_monitor(t);
 
 	BroadcastMonitorList(NULL);
 }
@@ -2439,19 +2513,20 @@ void CMD_DesktopSize(F_CMD_ARGS)
  */
 void CMD_GotoDesk(F_CMD_ARGS)
 {
-	struct monitor  *m_use = monitor_get_current(), *m, *m_loop;
-	char		*action_cpy, *action_cpy_start, *token;
+	struct monitor  *m, *m_loop;
+	char		*next, *token;
 	int		 new_desk;
 
-	action_cpy = strdup(action);
-	action_cpy_start = action_cpy;
-	token = PeekToken(action_cpy, &action_cpy);
-
+	token = PeekToken(action, &next);
 	m = monitor_resolve_name(token);
-	if (strcmp(m->si->name, token) != 0)
-		m = m_use;
+	if (m != NULL)
+	{
+		action = next;
+	}
 	else
-		PeekToken(action, &action);
+	{
+		m = monitor_get_current();
+	}
 
 	new_desk = GetDeskNumber(m, action, m->virtual_scr.CurrentDesk);
 
@@ -2474,22 +2549,18 @@ void CMD_GotoDesk(F_CMD_ARGS)
 				    this_desk_now);
 
 				m_loop->virtual_scr.is_swapping = true;
-				m->virtual_scr.is_swapping = true;
-
 				goto_desk(this_desk_now, m_loop);
-				goto_desk(new_desk, m);
-
 				m_loop->virtual_scr.is_swapping = false;
+
+				m->virtual_scr.is_swapping = true;
+				goto_desk(new_desk, m);
 				m->virtual_scr.is_swapping = false;
 
-				goto end;
+				return;
 			}
 		}
 	}
 	goto_desk(new_desk, m);
-end:
-	free(action_cpy_start);
-	return;
 }
 
 void CMD_Desk(F_CMD_ARGS)
@@ -2512,22 +2583,20 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
 {
 	int val[3];
 	Bool is_new_desk;
-	char *action_cpy, *action_cpy_start;
+	char *next;
 	char *token;
-	struct monitor  *m_use = monitor_get_current(), *m;
-
-	action_cpy = strdup(action);
-	action_cpy_start = action_cpy;
-	token = PeekToken(action_cpy, &action_cpy);
+	struct monitor  *m;
 
+	token = PeekToken(action, &next);
 	m = monitor_resolve_name(token);
-	if (strcmp(m->si->name, token) != 0)
-		m = m_use;
+	if (m != NULL)
+	{
+		action = next;
+	}
 	else
-		PeekToken(action, &action);
-
-	free(action_cpy_start);
-
+	{
+		m = monitor_get_current();
+	}
 	/* FIXME: monitor needs broadcast when global. */
 
 	if (MatchToken(action, "prev"))
@@ -2573,6 +2642,13 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
 		goto_desk(m->virtual_scr.CurrentDesk, m);
 		focus_grab_buttons_all();
 
+		/* If we're sharing desks between monitors, don't send further
+		 * NEW_DESK packets, as we've already handled this in the call
+		 * to goto_desk above.
+		 */
+		if (is_tracking_shared)
+			goto done;
+
 		BroadcastPacket(M_NEW_DESK, 2, (long)m->virtual_scr.CurrentDesk,
 			(long)m->si->rr_output);
 		/* FIXME: domivogt (22-Apr-2000): Fake a 'restack' for sticky
@@ -2591,6 +2667,7 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
 		BroadcastPacket(M_NEW_DESK, 2, (long)m->virtual_scr.CurrentDesk,
 				(long)m->si->rr_output);
 	}
+done:
 	BroadcastMonitorList(NULL);
 	EWMH_SetCurrentDesktop(m);
 
@@ -2658,8 +2735,8 @@ void CMD_MoveToDesk(F_CMD_ARGS)
 				xasprintf(&cmd,
 				    "MoveToPage %s $[w.pagex] $[w.pagey]",
 				    m->si->name);
-				execute_function_override_window(NULL, NULL,
-				    cmd, 0, fw);
+				execute_function_override_window(
+					NULL, NULL, cmd, NULL, 0, fw);
 				free(cmd);
 
 				return;
@@ -2675,20 +2752,18 @@ void CMD_Scroll(F_CMD_ARGS)
 	int x,y;
 	int val1, val2, val1_unit, val2_unit;
 	char *option;
+	char *next;
 	struct monitor  *m = monitor_get_current();
 
-	option = PeekToken(action, NULL);
+	option = PeekToken(action, &next);
 	if (StrEquals(option, "screen")) {
-		/* Skip literal 'screen' */
-		option = PeekToken(action, &action);
 		/* Actually get the screen value. */
-		option = PeekToken(action, &action);
-
+		option = PeekToken(next, &action);
 		m = monitor_resolve_name(option);
-		if (strcmp(m->si->name, option) != 0) {
-		fvwm_debug(__func__,
-			"Invalid screen: %s", option);
-		return;
+		if (m == NULL)
+		{
+			fvwm_debug(__func__, "Invalid screen: %s", option);
+			return;
 		}
 	}
 
@@ -2801,10 +2876,8 @@ store_desktop_cmd(int desk, char *name)
 	}
 
 	TAILQ_FOREACH(dc_loop, &desktop_cmd_q, entry) {
-		/* Update the name for an existing desktop, only if it
-		 * differs.
-		 */
-		if (dc_loop->desk == desk && (strcmp(dc_loop->name, name) != 0)) {
+		/* Update the name for an existing desktop. */
+		if (dc_loop->desk == desk) {
 			free(dc_loop->name);
 			dc_loop->name = fxstrdup(name);
 
@@ -2886,11 +2959,13 @@ desk_add_fw(FvwmWindow *fw)
 {
 	struct desktop_fw	*df, *df_loop;
 	struct desk_fvwmwin	*dfws;
-	int			 desk = monitor_get_current()->virtual_scr.CurrentDesk;
+	int			 desk;
 
 	if (fw == NULL)
 		return;
 
+	desk = fw->m->virtual_scr.CurrentDesk;
+
 	if (TAILQ_EMPTY(&desktop_fvwm_q))
 		TAILQ_INIT(&desktop_fvwm_q);
 
