@@ -12,8 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 /* ---------------------------- included header files ---------------------- */
@@ -379,12 +378,9 @@ int my_dither_depth_15_16_init(void)
 		return 0; /* fail */
 	}
 
-	Pcsi.red_dither =
-		(unsigned short *)safemalloc(4*4*256*sizeof(unsigned short));
-	Pcsi.green_dither =
-		(unsigned short *)safemalloc(4*4*256*sizeof(unsigned short));
-	Pcsi.blue_dither =
-		(unsigned short *)safemalloc(4*4*256*sizeof(unsigned short));
+	Pcsi.red_dither = fxmalloc(4*4*256*sizeof(unsigned short));
+	Pcsi.green_dither = fxmalloc(4*4*256*sizeof(unsigned short));
+	Pcsi.blue_dither = fxmalloc(4*4*256*sizeof(unsigned short));
 
 	for (y = 0; y < 4; y++)
 	{
@@ -693,7 +689,7 @@ void free_colors_in_table(
 		return;
 	}
 
-	p = (Pixel *)safemalloc(n*sizeof(Pixel));
+	p = fxmalloc(n * sizeof(Pixel));
 	for(i= 0; i < n; i++)
 	{
 		do_free = 1;
@@ -754,7 +750,7 @@ XColor *build_mapping_colors(int nr, int ng, int nb)
 	int r, g, b, i;
 	XColor *colors;
 
-	colors = (XColor *)safemalloc(nr*ng*nb * sizeof(XColor));
+	colors = fxmalloc(nr*ng*nb * sizeof(XColor));
 	i = 0;
 	for (r = 0; r < nr; r++)
 	{
@@ -785,7 +781,7 @@ static short *build_mapping_table(int nr, int ng, int nb, Bool use_named)
 	double dst;
 
 	colors_map = build_mapping_colors(nr, ng, nb);
-	Table = (short *)safemalloc((size+1) * sizeof(short));
+	Table = fxmalloc((size+1) * sizeof(short));
 	for(i=0; i<size; i++)
 	{
 		minind = 0;
@@ -907,7 +903,7 @@ PColor *alloc_color_cube(
 		end_grey = ngrey;
 	}
 
-	color_table = (PColor *)safemalloc((size+1) * sizeof(PColor));
+	color_table = fxmalloc((size+1) * sizeof(PColor));
 
 	i = 0;
 
@@ -1075,13 +1071,14 @@ PColor *alloc_named_ct(int *limit, Bool do_allocate)
 	XColor color;
 
 	*limit = (*limit > NColors)? NColors: *limit;
-	color_table = (PColor *)safemalloc((*limit+1) * sizeof(PColor));
+	color_table = fxmalloc((*limit+1) * sizeof(PColor));
 	for(i=0; i<*limit; i++)
 	{
 		rc=XParseColor(Pdpy, Pcmap, color_names[i], &color);
 		if (rc==0) {
-			fprintf(stderr,"color_to_rgb: can't parse color %s,"
-				" rc %d\n", color_names[i], rc);
+			fvwm_debug(__func__,
+				   "color_to_rgb: can't parse color %s,"
+				   " rc %d\n", color_names[i], rc);
 			free_table_colors(color_table, i);
 			free(color_table);
 			return NULL;
@@ -1248,14 +1245,15 @@ static void finish_ct_init(
 		{
 			ctt++;
 		}
-		env = safemalloc(PICTURE_TABLETYPE_LENGHT + 1);
+		/* TA:  FIXME!  Spelling!!! */
+		/* TA:  FIXME!  Use xasprintf() */
+		env = fxmalloc(PICTURE_TABLETYPE_LENGHT + 1);
 		sprintf(env, "%i", ctt);
 		flib_putenv("FVWM_COLORTABLE_TYPE", env);
 		free(env);
 		if (Pdepth <= 8)
 		{
-			Pac = (PColor *)safecalloc(
-				(1 << Pdepth), sizeof(PColor));
+			Pac = fxcalloc((1 << Pdepth), sizeof(PColor));
 		}
 	}
 
@@ -1754,8 +1752,8 @@ int PictureAllocColorTable(
 	finish_ct_init(call_type, cc_nbr-1, 0, 0, 0, 0, 1, 0);
 	if (Pct == NULL)
 	{
-		fprintf(stderr,
-			"[fvwm] ERR -- Cannot get Black and White. exiting!\n");
+		fvwm_debug(__func__,
+			   "[fvwm] ERR -- Cannot get Black and White. exiting!\n");
 		exit(2);
 	}
 	return PColorLimit;
@@ -1802,7 +1800,7 @@ Bool alloc_direct_colors(int *limit, Bool use_my_color_limit)
 	ng = 1 << Pcsi.green_prec;
 	nb = 1 << Pcsi.blue_prec;
 
-	colors = (XColor *)safemalloc(nb*sizeof(XColor));
+	colors = fxmalloc(nb*sizeof(XColor));
 	cf = DoRed|DoBlue|DoGreen;
 	for (r=0; r<nr; r++)
 	{
@@ -1842,7 +1840,7 @@ void init_static_colors_table(void)
 	int nbr_of_colors = min(256, (1 << Pdepth));
 
 	PColorLimit = nbr_of_colors;
-	Pct = (PColor *)safemalloc((nbr_of_colors+1) * sizeof(PColor));
+	Pct = fxmalloc((nbr_of_colors+1) * sizeof(PColor));
 	for (i = 0; i < nbr_of_colors; i++)
 	{
 		colors[i].pixel = Pct[i].color.pixel = i;
@@ -1879,10 +1877,10 @@ void print_colormap(Colormap cmap)
 	XQueryColors(Pdpy, cmap, colors, nbr_of_colors);
 	for (i = 0; i < nbr_of_colors; i++)
 	{
-		fprintf(stderr,"    rgb(%.3i): %.3i/%.3i/%.3i\n", i,
-			colors[i].red >> 8,
-			colors[i].green >> 8,
-			colors[i].blue >> 8);
+		fvwm_debug(__func__, "    rgb(%.3i): %.3i/%.3i/%.3i\n", i,
+			   colors[i].red >> 8,
+			   colors[i].green >> 8,
+			   colors[i].blue >> 8);
 	}
 }
 
@@ -1958,14 +1956,12 @@ PictureImageColorAllocator *PictureOpenImageColorAllocator(
 	PictureImageColorAllocator *pica;
 	Bool do_save_pixels = False;
 
-	pica = (PictureImageColorAllocator *)safemalloc(
-		sizeof(PictureImageColorAllocator));
+	pica = fxmalloc(sizeof(PictureImageColorAllocator));
 	if (Pdepth <= 8 && !do_not_save_pixels && (Pvisual->class & 1) &&
 	    ((PUseDynamicColors && Pct) || no_limit))
 	{
 		int s =  1 << Pdepth;
-		pica->pixels_table = (unsigned long *)safecalloc(
-			s, sizeof(unsigned long));
+		pica->pixels_table = fxcalloc(s, sizeof(unsigned long));
 		pica->pixels_table_size = s;
 		do_save_pixels = True;
 	}
@@ -2023,12 +2019,11 @@ void PictureCloseImageColorAllocator(
 		}
 		if (free_num)
 		{
-			free_pixels = (Pixel *)safemalloc(
-				free_num * sizeof(Pixel));
+			free_pixels = fxmalloc(free_num * sizeof(Pixel));
 		}
 		if (np && nalloc_pixels != NULL && alloc_pixels != NULL)
 		{
-			save_pixels = (Pixel *)safemalloc(np * sizeof(Pixel));
+			save_pixels = fxmalloc(np * sizeof(Pixel));
 		}
 		for(i = 0; i < pica->pixels_table_size; i++)
 		{
@@ -2149,15 +2144,16 @@ void PictureReduceColorName(char **my_color)
 
 	if (!XParseColor(Pdpy, Pcmap, *my_color, &rgb))
 	{
-		fprintf(stderr,"color_to_rgb: can't parse color %s\n",
-			*my_color);
+		fvwm_debug(__func__, "color_to_rgb: can't parse color %s\n",
+			   *my_color);
 	}
 	index = get_color_index(rgb.red,rgb.green,rgb.blue, False);
 	/* Finally: replace the color string by the newly determined color
 	 * string */
 	free(*my_color);                    /* free old color */
 	/* area for new color */
-	*my_color = safemalloc(8);
+	/* TA:  FIXME!  xasprintf() */
+	*my_color = fxmalloc(8);
 	sprintf(*my_color,"#%x%x%x",
 		Pct[index].color.red >> 8,
 		Pct[index].color.green >> 8,
@@ -2365,55 +2361,55 @@ void PicturePrintColorInfo(int verbose)
 {
 	unsigned long nbr_of_colors = 1 << Pdepth;
 
-	fprintf(stderr, "fvwm info on colors\n");
-	fprintf(stderr, "  Visual ID: 0x%x, Default?: %s, Class: ",
-		(int)(Pvisual->visualid),
-		(Pdefault)? "Yes":"No");
+	fvwm_debug(__func__, "fvwm info on colors\n");
+	fvwm_debug(__func__, "  Visual ID: 0x%x, Default?: %s, Class: ",
+		   (int)(Pvisual->visualid),
+		   (Pdefault)? "Yes":"No");
 	if (Pvisual->class == TrueColor)
 	{
-		fprintf(stderr,"TrueColor");
+		fvwm_debug(__func__, "TrueColor");
 	}
 	else if (Pvisual->class == PseudoColor)
 	{
-		fprintf(stderr,"PseudoColor");
+		fvwm_debug(__func__, "PseudoColor");
 	}
 	else if (Pvisual->class == DirectColor)
 	{
-		fprintf(stderr,"DirectColor");
+		fvwm_debug(__func__, "DirectColor");
 	}
 	else if (Pvisual->class == StaticColor)
 	{
-		fprintf(stderr,"StaticColor");
+		fvwm_debug(__func__, "StaticColor");
 	}
 	else if (Pvisual->class == GrayScale)
 	{
-		fprintf(stderr,"GrayScale");
+		fvwm_debug(__func__, "GrayScale");
 	}
 	else if (Pvisual->class == StaticGray)
 	{
-		fprintf(stderr,"StaticGray");
+		fvwm_debug(__func__, "StaticGray");
 	}
-	fprintf(stderr, "\n");
-	fprintf(stderr, "  Depth: %i, Number of colors: %lu",
-		Pdepth, (unsigned long)nbr_of_colors);
+	fvwm_debug(__func__, "\n");
+	fvwm_debug(__func__, "  Depth: %i, Number of colors: %lu",
+		   Pdepth, (unsigned long)nbr_of_colors);
 	if (Pct != NULL)
 	{
-		fprintf(stderr,"\n  Pallet with %i colors", PColorLimit);
+		fvwm_debug(__func__, "\n  Pallet with %i colors", PColorLimit);
 		if (Pvisual->class & 1)
 		{
-			fprintf(stderr,", Number of free colors: %i\n",
-				get_nbr_of_free_colors((1 << Pdepth)));
-			fprintf(stderr,
-				"  Auto Detected: %s, Strict: %s, Allocated: %s,"
-				" Dynamic: %s\n",
-				(Pcsi.pre_allocated_pallet)? "Yes":"No",
-				(PStrictColorLimit)? "Yes":"No",
-				(PAllocTable)? "Yes":"No",
-				(PUseDynamicColors)? "Yes":"No");
+			fvwm_debug(__func__, ", Number of free colors: %i\n",
+				   get_nbr_of_free_colors((1 << Pdepth)));
+			fvwm_debug(__func__,
+				   "  Auto Detected: %s, Strict: %s, Allocated: %s,"
+				   " Dynamic: %s\n",
+				   (Pcsi.pre_allocated_pallet)? "Yes":"No",
+				   (PStrictColorLimit)? "Yes":"No",
+				   (PAllocTable)? "Yes":"No",
+				   (PUseDynamicColors)? "Yes":"No");
 		}
 		else
 		{
-			fprintf(stderr," (default colormap)\n");
+			fvwm_debug(__func__, " (default colormap)\n");
 		}
 		if (PColorLimit <= 256)
 		{
@@ -2423,19 +2419,19 @@ void PicturePrintColorInfo(int verbose)
 
 			if (verbose)
 			{
-				fprintf(stderr,"  The fvwm colors table:\n");
+				fvwm_debug(__func__,
+					   "  The fvwm colors table:\n");
 			}
 			for (i = 0; i < PColorLimit; i++)
 			{
 				if (verbose)
 				{
-					fprintf(
-						stderr,
-						"    rgb:%.3i/%.3i/%.3i\t%lu\n",
-						Pct[i].color.red >> 8,
-						Pct[i].color.green >> 8,
-						Pct[i].color.blue >> 8,
-						Pct[i].alloc_count);
+					fvwm_debug(__func__,
+						   "    rgb:%.3i/%.3i/%.3i\t%lu\n",
+						   Pct[i].color.red >> 8,
+						   Pct[i].color.green >> 8,
+						   Pct[i].color.blue >> 8,
+						   Pct[i].alloc_count);
 				}
 				if (Pct[i].alloc_count)
 				{
@@ -2446,8 +2442,9 @@ void PicturePrintColorInfo(int verbose)
 			{
 				if (verbose)
 				{
-					fprintf(stderr,"  fvwm colors not in"
-						" the table:\n");
+					fvwm_debug(__func__,
+						   "  fvwm colors not in"
+						   " the table:\n");
 				}
 				for(i=0; i < nbr_of_colors; i++)
 				{
@@ -2469,35 +2466,36 @@ void PicturePrintColorInfo(int verbose)
 					count_alloc++;
 					if (verbose)
 					{
-						fprintf(
-							stderr,
-							"    rgb:"
-							"%.3i/%.3i/%.3i\t%lu\n",
-							Pac[i].color.red >> 8,
-							Pac[i].color.green >> 8,
-							Pac[i].color.blue >> 8,
-							Pac[i].alloc_count);
+						fvwm_debug(__func__,
+							   "    rgb:"
+							   "%.3i/%.3i/%.3i\t%lu\n",
+							   Pac[i].color.red >> 8,
+							   Pac[i].color.green >> 8,
+							   Pac[i].color.blue >> 8,
+							   Pac[i].alloc_count);
 					}
 				}
 				if (verbose && count_alloc == 0)
 				{
 					if (verbose)
 					{
-						fprintf(stderr,"    None\n");
+						fvwm_debug(__func__,
+							   "    None\n");
 					}
 				}
 			}
 			if (Pvisual->class & 1)
 			{
-				fprintf(stderr,
-					"  Number of colours used by fvwm:\n");
-				fprintf(stderr,
-					"    In the table: %i\n", count);
-				fprintf(
-					stderr, "    Out of the table: %i\n",
-					count_alloc);
-				fprintf(stderr,
-					"    Total: %i\n", count_alloc+count);
+				fvwm_debug(__func__,
+					   "  Number of colours used by fvwm:\n");
+				fvwm_debug(__func__,
+					   "    In the table: %i\n", count);
+				fvwm_debug(__func__,
+					   "    Out of the table: %i\n",
+					   count_alloc);
+				fvwm_debug(__func__,
+					   "    Total: %i\n",
+					   count_alloc+count);
 			}
 		}
 	}
@@ -2505,27 +2503,28 @@ void PicturePrintColorInfo(int verbose)
 	{
 		if (Pvisual->class == DirectColor)
 		{
-			fprintf(stderr, ", Pseudo Pallet with: %i colors\n",
-				(1 << Pcsi.red_prec)*(1 << Pcsi.green_prec)*
-				(1 << Pcsi.blue_prec));
+			fvwm_debug(__func__,
+				   ", Pseudo Pallet with: %i colors\n",
+				   (1 << Pcsi.red_prec)*(1 << Pcsi.green_prec)*
+				   (1 << Pcsi.blue_prec));
 		}
 		else
 		{
-			fprintf(stderr, ", No Pallet (static colors)\n");
+			fvwm_debug(__func__, ", No Pallet (static colors)\n");
 		}
-		fprintf(stderr, "  red: %i, green: %i, blue %i\n",
-			1 << Pcsi.red_prec, 1 << Pcsi.green_prec,
-			1 << Pcsi.blue_prec);
+		fvwm_debug(__func__, "  red: %i, green: %i, blue %i\n",
+			   1 << Pcsi.red_prec, 1 << Pcsi.green_prec,
+			   1 << Pcsi.blue_prec);
 		if (verbose && Pdepth <= 8)
 		{
 			if (Pvisual->class == DirectColor)
 			{
-				fprintf(stderr, "  Colormap:\n");
+				fvwm_debug(__func__, "  Colormap:\n");
 			}
 			else
 			{
-				fprintf(stderr,
-					"  Static Colormap used by fvwm:\n");
+				fvwm_debug(__func__,
+					   "  Static Colormap used by fvwm:\n");
 			}
 			print_colormap(Pcmap);
 		}
@@ -2533,7 +2532,7 @@ void PicturePrintColorInfo(int verbose)
 
 	if (Pdepth <= 8 && verbose >= 2)
 	{
-		fprintf(stderr,"\n  Default Colormap:\n");
+		fvwm_debug(__func__, "\n  Default Colormap:\n");
 		print_colormap(DefaultColormap(Pdpy,DefaultScreen(Pdpy)));
 	}
 }
