@@ -13,8 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 /* A set of functions for implementing a dummy sm. The code is based on xsm */
@@ -62,9 +61,9 @@ typedef struct
 
 /* ---------------------------- local variables ---------------------------- */
 
-static FIceAuthDataEntry *authDataEntries = NULL;
+static FIceAuthDataEntry *sauthDataEntries = NULL;
 static FIceIOErrorHandler prev_handler;
-static FIceListenObj *listenObjs;
+static FIceListenObj *slistenObjs;
 
 static char *addAuthFile = NULL;
 static char *remAuthFile = NULL;
@@ -163,7 +162,8 @@ unique_filename(char *path, char *prefix, int *pFd)
 {
 	char *tempFile;
 
-	tempFile = (char *)safemalloc(strlen(path) + strlen(prefix) + 8);
+	/* TA:  FIXME!  xasprintf() */
+	tempFile = fxmalloc(strlen(path) + strlen(prefix) + 8);
 	sprintf(tempFile, "%s/%sXXXXXX", path, prefix);
 	*pFd =  fvwm_mkstemp(tempFile);
 	if (*pFd == -1)
@@ -238,8 +238,7 @@ Status SetAuthentication(
 		goto bad;
 	}
 
-	*authDataEntries = (FIceAuthDataEntry *) safemalloc(
-		count * 2 * sizeof (FIceAuthDataEntry));
+	*authDataEntries = fxmalloc(count * 2 * sizeof (FIceAuthDataEntry));
 
 	for (i = 0; i < count * 2; i += 2)
 	{
@@ -274,7 +273,12 @@ Status SetAuthentication(
 	umask (original_umask);
 
 	sprintf (command, "iceauth source %s", addAuthFile);
-	system(command);
+	{
+		int n;
+
+		n = system(command);
+		(void)n;
+	}
 
 	unlink (addAuthFile);
 
@@ -328,7 +332,12 @@ void FreeAuthenticationData(int count, FIceAuthDataEntry *authDataEntries)
 	free ((char *) authDataEntries);
 
 	sprintf (command, "iceauth source %s", remAuthFile);
-	system(command);
+	{
+		int n;
+
+		n = system(command);
+		(void)n;
+	}
 
 	unlink (remAuthFile);
 
@@ -380,7 +389,7 @@ CloseListeners(void)
 		return;
 	}
 
-	FIceFreeListenObjs(numTransports, listenObjs);
+	FIceFreeListenObjs(numTransports, slistenObjs);
 }
 
 static
@@ -397,7 +406,7 @@ void ice_watch_fd(
 
 	if (opening)
 	{
-		fice_conn = (fsm_ice_conn_t *)safemalloc(sizeof(fsm_ice_conn_t));
+		fice_conn = fxmalloc(sizeof(fsm_ice_conn_t));
 		fice_conn->ice_conn = conn;
 		fice_conn->fd = FIceConnectionNumber(conn);
 		*watch_data = (FIcePointer) fice_conn;
@@ -431,11 +440,11 @@ Status RegisterClientProc(
 		return 0;
 	}
 #ifdef FVWM_DEBUG_FSM
-	fprintf (stderr,
-		 "[%s][RegisterClientProc] On FIceConn fd = %d, received "
-		 "REGISTER CLIENT [Previous Id = %s]\n", module_name,
-		 FIceConnectionNumber (client->ice_conn),
-		 previousId ? previousId : "NULL");
+	fvwm_debug(__func__,
+		   "[%s][RegisterClientProc] On FIceConn fd = %d, received "
+		   "REGISTER CLIENT [Previous Id = %s]\n", module_name,
+		   FIceConnectionNumber (client->ice_conn),
+		   previousId ? previousId : "NULL");
 #endif
 
 	/* ignore previousID!! (we are dummy) */
@@ -447,18 +456,17 @@ Status RegisterClientProc(
 		{
 			free(id);
 		}
-		fprintf(
-			stderr,
-			"[%s][RegisterClientProc] ERR -- fail to register "
-			"client", module_name);
+		fvwm_debug(__func__,
+			   "[%s][RegisterClientProc] ERR -- fail to register "
+			   "client", module_name);
 		return 0;
 	}
 
 #ifdef FVWM_DEBUG_FSM
-	fprintf (stderr,
-		 "[%s][RegisterClientProc] On FIceConn fd = %d, sent "
-		 "REGISTER CLIENT REPLY [Client Id = %s]\n",
-		 module_name, FIceConnectionNumber (client->ice_conn), id);
+	fvwm_debug(__func__,
+		   "[%s][RegisterClientProc] On FIceConn fd = %d, sent "
+		   "REGISTER CLIENT REPLY [Client Id = %s]\n",
+		   module_name, FIceConnectionNumber (client->ice_conn), id);
 #endif
 
 	client->clientId = id;
@@ -512,6 +520,7 @@ void SaveYourselfPhase2ReqProc(FSmsConn smsConn, FSmPointer managerData)
 		return;
 	}
 	client = (fsm_client_t *)managerData;
+	SUPPRESS_UNUSED_VAR_WARNING(client);
 	FSmsSaveYourselfPhase2(client->smsConn);
 }
 
@@ -525,6 +534,7 @@ SaveYourselfDoneProc(FSmsConn smsConn, FSmPointer managerData, Bool success)
 		return;
 	}
 	client = (fsm_client_t *) managerData;
+	SUPPRESS_UNUSED_VAR_WARNING(client);
 	FSmsSaveComplete(client->smsConn);
 }
 
@@ -538,15 +548,14 @@ CloseDownClient(fsm_client_t *client)
 	}
 
 #ifdef FVWM_DEBUG_FSM
-	fprintf(
-		stderr, "[%s][CloseDownClient] ICE Connection closed, "
-		"FIceConn fd = %d\n", module_name,
-		FIceConnectionNumber (client->ice_conn));
+	fvwm_debug(__func__, "[%s][CloseDownClient] ICE Connection closed, "
+		   "FIceConn fd = %d\n", module_name,
+		   FIceConnectionNumber (client->ice_conn));
 #endif
 
 	FSmsCleanUp(client->smsConn);
 	FIceSetShutdownNegotiation(client->ice_conn, False);
-	if (!FIceCloseConnection(client->ice_conn) != FIceClosedNow)
+	if ((!FIceCloseConnection(client->ice_conn)) != FIceClosedNow)
 	{
 		/* do not care */
 	}
@@ -623,7 +632,7 @@ NewClientProc(
 	    return 0;
     }
 
-    nc = (fsm_client_t *)safemalloc(sizeof (fsm_client_t));
+    nc = fxmalloc(sizeof (fsm_client_t));
     *maskRet = 0;
 
     nc->smsConn = smsConn;
@@ -633,10 +642,10 @@ NewClientProc(
     running_list = flist_append_obj(running_list, nc);
 
 #ifdef FVWM_DEBUG_FSM
-	fprintf(
-		stderr, "[%s][NewClientProc] On FIceConn fd = %d, client "
-		"set up session mngmt protocol\n", module_name,
-		FIceConnectionNumber (nc->ice_conn));
+	fvwm_debug(__func__,
+                   "[%s][NewClientProc] On FIceConn fd = %d, client "
+                   "set up session mngmt protocol\n", module_name,
+                   FIceConnectionNumber (nc->ice_conn));
 #endif
 
     /*
@@ -707,7 +716,7 @@ void CompletNewConnectionMsg(void)
 	while(l != NULL)
 	{
 		ice_conn = (FIceConn)l->object;
-		cstatus = FIceConnectionStatus(ice_conn);
+		cstatus = (FIceAcceptStatus)FIceConnectionStatus(ice_conn);
 		if (cstatus == (int)FIceConnectPending)
 		{
 			l = l->next;
@@ -723,10 +732,11 @@ void CompletNewConnectionMsg(void)
 			char *connstr;
 
 			connstr = FIceConnectionString (ice_conn);
-			fprintf(stderr, "[%s][NewConnection] ICE Connection "
-				"opened by client, FIceConn fd = %d, Accept at "
-				"networkId %s\n", module_name,
-				FIceConnectionNumber (ice_conn), connstr);
+			fvwm_debug(__func__,
+				   "[%s][NewConnection] ICE Connection "
+				   "opened by client, FIceConn fd = %d, Accept at "
+				   "networkId %s\n", module_name,
+				   FIceConnectionNumber (ice_conn), connstr);
 			free (connstr);
 #endif
 		}
@@ -743,15 +753,17 @@ void CompletNewConnectionMsg(void)
 #ifdef FVWM_DEBUG_FSM
 			if (cstatus == FIceConnectIOError)
 			{
-				fprintf(stderr, "[%s][NewConnection] IO error "
-					"opening ICE Connection!\n",
-					module_name);
+				fvwm_debug(__func__,
+					   "[%s][NewConnection] IO error "
+					   "opening ICE Connection!\n",
+					   module_name);
 			}
 			else
 			{
-				fprintf(stderr, "[%s][NewConnection] ICE "
-					"Connection rejected!\n",
-					module_name);
+				fvwm_debug(__func__,
+					   "[%s][NewConnection] ICE "
+					   "Connection rejected!\n",
+					   module_name);
 			}
 #endif
 		}
@@ -766,19 +778,19 @@ void NewConnectionMsg(int i)
 
 	if (!SessionSupport)
 	{
-		status = 0;
 		return;
 	}
 
-	ice_conn = FIceAcceptConnection(listenObjs[i], &status);
+	SUPPRESS_UNUSED_VAR_WARNING(status);
+	ice_conn = FIceAcceptConnection(slistenObjs[i], &status);
 #ifdef FVWM_DEBUG_FSM
-	fprintf(stderr, "[%s][NewConnection] %i\n", module_name, i);
+	fvwm_debug(__func__, "[%s][NewConnection] %i\n", module_name, i);
 #endif
 	if (!ice_conn)
 	{
 #ifdef FVWM_DEBUG_FSM
-		fprintf(stderr, "[%s][NewConnection] "
-			"FIceAcceptConnection failed\n", module_name);
+		fvwm_debug(__func__, "[%s][NewConnection] "
+			   "FIceAcceptConnection failed\n", module_name);
 #endif
 	}
 	else
@@ -801,7 +813,8 @@ void ProcessIceMsg(fsm_ice_conn_t *fic)
 	}
 
 #ifdef FVWM_DEBUG_FSM
-	fprintf(stderr, "[%s][ProcessIceMsg] %i\n", module_name, (int)fic->fd);
+	fvwm_debug(__func__, "[%s][ProcessIceMsg] %i\n", module_name,
+		   (int)fic->fd);
 #endif
 
 	status = FIceProcessMessages(fic->ice_conn, NULL, NULL);
@@ -812,8 +825,9 @@ void ProcessIceMsg(fsm_ice_conn_t *fic)
 		int found = 0;
 
 #ifdef FVWM_DEBUG_FSM
-		fprintf(stderr, "[%s][ProcessIceMsg] IO error on connection\n",
-			module_name);
+		fvwm_debug(__func__,
+			   "[%s][ProcessIceMsg] IO error on connection\n",
+			   module_name);
 #endif
 
 		for (cl = running_list; cl; cl = cl->next)
@@ -837,7 +851,7 @@ void ProcessIceMsg(fsm_ice_conn_t *fic)
 			 */
 
 			FIceSetShutdownNegotiation (fic->ice_conn, False);
-			if (!FIceCloseConnection(fic->ice_conn) != FIceClosedNow)
+			if ((!FIceCloseConnection(fic->ice_conn)) != FIceClosedNow)
 			{
 				/* do not care */
 			}
@@ -964,9 +978,8 @@ void set_session_manager(Display *dpy, Window window, char *sm)
 	}
 
 #ifdef FVWM_DEBUG_FSM
-	fprintf(
-		stderr, "[%s][fsm_init] Proxy %s window 0x%lx\n",
-		module_name, (sm)? "On":"Off", client_leader);
+	fvwm_debug(__func__, "[%s][fsm_init] Proxy %s window 0x%lx\n",
+		   module_name, (sm)? "On":"Off", client_leader);
 
 #endif
 
@@ -1004,80 +1017,76 @@ int fsm_init(char *module)
 		ice_watch_fd(0, NULL, 0, NULL);
 		NewClientProc(0, NULL, 0, NULL, NULL);
 #ifdef FVWM_DEBUG_FSM
-		fprintf(
-			stderr, "[%s][fsm_init] No Session Support\n",
-			module_name);
+		fvwm_debug(__func__, "[%s][fsm_init] No Session Support\n",
+			   module_name);
 #endif
 		return 0;
 	}
 
 	if (fsm_init_succeed)
 	{
-		fprintf(
-			stderr, "[%s][fsm_init] <<ERROR>> -- "
-			"Session already Initialized!\n", module_name);
+		fvwm_debug(__func__, "[%s][fsm_init] <<ERROR>> -- "
+			   "Session already Initialized!\n", module_name);
 		return 1;
 	}
 	module_name = module;
 	InstallIOErrorHandler ();
 
 #ifdef FVWM_DEBUG_FSM
-	fprintf(stderr, "[%s][fsm_init] FSmsInitialize\n", module_name);
+	fvwm_debug(__func__, "[%s][fsm_init] FSmsInitialize\n", module_name);
 #endif
 	if (!FSmsInitialize(
 		    module, "1.0", NewClientProc, NULL, HostBasedAuthProc, 256,
 		    errormsg))
 	{
-		fprintf(
-			stderr, "[%s][fsm_init] <<ERROR>> -- "
-			"FSmsInitialize failed: %s\n",
-			module_name, errormsg);
+		fvwm_debug(__func__, "[%s][fsm_init] <<ERROR>> -- "
+			   "FSmsInitialize failed: %s\n",
+			   module_name, errormsg);
 		return 0;
 	}
 
 	if (!FIceListenForConnections (
-		    &numTransports, &listenObjs, 256, errormsg))
+		    &numTransports, &slistenObjs, 256, errormsg))
 	{
-		fprintf(
-			stderr, "[%s][fsm_init] <<ERROR>> -- "
-			"FIceListenForConnections failed:\n"
-			"%s\n", module_name, errormsg);
+		fvwm_debug(__func__, "[%s][fsm_init] <<ERROR>> -- "
+			   "FIceListenForConnections failed:\n"
+			   "%s\n", module_name, errormsg);
 		return 0;
 	}
 
 	atexit(CloseListeners);
 
-	if (!SetAuthentication(numTransports, listenObjs, &authDataEntries))
+	if (!SetAuthentication(numTransports, slistenObjs, &sauthDataEntries))
 	{
-	    fprintf(
-		    stderr, "[%s][fsm_init] <<ERROR>> -- "
-		    "Could not set authorization\n", module_name);
+	    fvwm_debug(__func__, "[%s][fsm_init] <<ERROR>> -- "
+		       "Could not set authorization\n", module_name);
 	    return 0;
 	}
 
 	if (FIceAddConnectionWatch(&ice_watch_fd, NULL) == 0)
 	{
-		fprintf(stderr,
-			"[%s][fsm_init] <<ERROR>> -- "
-			"FIceAddConnectionWatch failed\n", module_name);
+		fvwm_debug(__func__,
+			   "[%s][fsm_init] <<ERROR>> -- "
+			   "FIceAddConnectionWatch failed\n", module_name);
 		return 0;
 	}
 
-	ice_fd = (int *)safemalloc(sizeof(int) * numTransports + 1);
+	ice_fd = fxmalloc(sizeof(int) * numTransports + 1);
 	for (i = 0; i < numTransports; i++)
 	{
-		ice_fd[i] = FIceGetListenConnectionNumber(listenObjs[i]);
+		ice_fd[i] = FIceGetListenConnectionNumber(slistenObjs[i]);
 	}
 
-	networkIds = FIceComposeNetworkIdList(numTransports, listenObjs);
-	p = (char *)safemalloc(
-		16 + strlen(networkIds) + 1);
+	networkIds = FIceComposeNetworkIdList(numTransports, slistenObjs);
+	/* TA:  FIXME!  xasprintf() */
+	p = fxmalloc(16 + strlen(networkIds) + 1);
 	sprintf(p, "SESSION_MANAGER=%s", networkIds);
 	putenv(p);
 
 #ifdef FVWM_DEBUG_FSM
-	fprintf(stderr,"[%s][fsm_init] OK: %i\n", module_name, numTransports);
-	fprintf(stderr,"\t%s\n", p);
+	fvwm_debug(__func__, "[%s][fsm_init] OK: %i\n", module_name,
+		   numTransports);
+	fvwm_debug(__func__, "\t%s\n", p);
 #endif
 
 	fsm_init_succeed = True;
@@ -1190,6 +1199,5 @@ void fsm_close(void)
 	{
 		return;
 	}
-	FreeAuthenticationData(numTransports, authDataEntries);
+	FreeAuthenticationData(numTransports, sauthDataEntries);
 }
-
