@@ -5,12 +5,6 @@
  * This module, and the entire GoodStuff program, and the concept for
  * interfacing this module to the Window Manager, are all original work
  * by Robert Nation
- *
- * Copyright 1993, Robert Nation. No guarantees or warantees or anything
- * are provided or implied in any way whatsoever. Use this program at your
- * own risk. Permission to use this program for any purpose is given,
- * as long as the copyright is kept intact.
- *
  */
 
 /* This program is free software; you can redistribute it and/or modify
@@ -24,8 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 /* ------------------------------- includes -------------------------------- */
@@ -42,16 +35,8 @@
 #include "libs/ftime.h"
 #include <sys/stat.h>
 
-#include <X11/keysym.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/Xproto.h>
-#include <X11/Xatom.h>
-#include <X11/Intrinsic.h>
-
 #include "libs/defaults.h"
 #include "libs/fvwmlib.h"
-#include "libs/FScreen.h"
 #include "libs/FShape.h"
 #include "libs/FRenderInit.h"
 #include "libs/Grab.h"
@@ -71,6 +56,7 @@
 #include "libs/wild.h"
 #include "libs/WinMagic.h"
 #include "libs/XError.h"
+#include "libs/FEvent.h"
 
 #include "FvwmButtons.h"
 #include "misc.h" /* ConstrainSize() */
@@ -123,7 +109,6 @@ void CreateUberButtonWindow(button_info *, int, int);
 int My_FNextEvent(Display *dpy, XEvent *event);
 void process_message(unsigned long type, unsigned long *body);
 extern void send_clientmessage(Display *disp, Window w, Atom a, Time timestamp);
-void parse_message_line(char *line);
 void CheckForHangon(unsigned long *);
 static Window GetRealGeometry(
 	Display *, Window, int *, int *, unsigned int *, unsigned int *,
@@ -132,8 +117,6 @@ static void GetPanelGeometry(
 	Bool get_big, button_info *b, int lb, int tb, int rb, int bb,
 	int *x, int *y, int *w, int *h);
 void swallow(unsigned long *);
-void AddButtonAction(button_info *, int, char *);
-char *GetButtonAction(button_info *, int);
 static void update_root_transparency(XEvent *Event);
 static void change_colorset(int colorset, XEvent *Event);
 
@@ -199,22 +182,10 @@ Bool do_allow_bad_access = False;
 Bool was_bad_access = False;
 Bool swallowed = False;
 Window swallower_win = 0;
+char windowname[128];
 
 /* ------------------------------ Misc functions ----------------------------*/
 
-#ifdef DEBUG
-char *mymalloc(int length)
-{
-	int i = length;
-	char *p = safemalloc(length);
-	while (i)
-	{
-		p[--i] = 255;
-	}
-	return p;
-}
-#endif
-
 /**
 *** Some fancy routines straight out of the manual :-) Used in DeadPipe.
 **/
@@ -276,7 +247,6 @@ static void DeadPipeCleanup(void)
 
 	signal(SIGPIPE, SIG_IGN);/* Xsync may cause SIGPIPE */
 
-	MyXGrabServer(Dpy); /* We don't want interference right now */
 	while (NextButton(&ub, &b, &button, 0))
 	{
 		swin = SwallowedWindow(b);
@@ -284,9 +254,9 @@ static void DeadPipeCleanup(void)
 		if ((buttonSwallowCount(b) == 3) && swin)
 		{
 #ifdef DEBUG_HANGON
-			fprintf(stderr,
-				"%s: Button 0x%06x window 0x%x (\"%s\") is ",
-				MyName, (ushort)b, (ushort)swin, b->hangon);
+			fvwm_debug(__func__,
+				   "%s: Button 0x%06x window 0x%x (\"%s\") is ",
+				   MyName, (ushort)b, (ushort)swin, b->hangon);
 #endif
 			if (!IsThereADestroyEvent(b))
 			{ /* Has someone destroyed it? */
@@ -296,7 +266,8 @@ static void DeadPipeCleanup(void)
 					{
 						XKillClient(Dpy, swin);
 #ifdef DEBUG_HANGON
-						fprintf(stderr, "now killed\n");
+						fvwm_debug(__func__,
+							   "now killed\n");
 #endif
 					}
 					else
@@ -306,15 +277,16 @@ static void DeadPipeCleanup(void)
 							_XA_WM_DEL_WIN,
 							CurrentTime);
 #ifdef DEBUG_HANGON
-						fprintf(stderr,
-							"now deleted\n");
+						fvwm_debug(__func__,
+							   "now deleted\n");
 #endif
 					}
 				}
 				else
 				{
 #ifdef DEBUG_HANGON
-					fprintf(stderr, "now unswallowed\n");
+					fvwm_debug(__func__,
+						   "now unswallowed\n");
 #endif
 					if (b->swallow & b_FvwmModule)
 					{
@@ -342,13 +314,13 @@ static void DeadPipeCleanup(void)
 #ifdef DEBUG_HANGON
 			else
 			{
-				fprintf(stderr, "already handled\n");
+				fvwm_debug(__func__, "already handled\n");
 			}
 #endif
 		}
 	}
+
 	XSync(Dpy, 0);
-	MyXUngrabServer(Dpy); /* We're through */
 
 	fsm_close();
 	/* Hey, we have to free the pictures too! */
@@ -386,14 +358,16 @@ void SetButtonSize(button_info *ub, int w, int h)
 
 	if (!ub || !(ub->flags.b_Container))
 	{
-		fprintf(stderr,
-			"%s: BUG: Tried to set size of noncontainer\n", MyName);
+		fvwm_debug(__func__,
+			   "%s: BUG: Tried to set size of noncontainer\n",
+			   MyName);
 		exit(2);
 	}
 	if (ub->c->num_rows == 0 || ub->c->num_columns == 0)
 	{
-		fprintf(stderr,
-			"%s: BUG: Set size when rows/cols was unset\n", MyName);
+		fvwm_debug(__func__,
+			   "%s: BUG: Set size when rows/cols was unset\n",
+			   MyName);
 		exit(2);
 	}
 
@@ -438,7 +412,8 @@ void AddButtonAction(button_info *b, int n, char *action)
 
 	if (!b || n < 0 || n > NUMBER_OF_EXTENDED_MOUSE_BUTTONS || !action)
 	{
-		fprintf(stderr, "%s: BUG: AddButtonAction failed\n", MyName);
+		fvwm_debug(__func__, "%s: BUG: AddButtonAction failed\n",
+			   MyName);
 		exit(2);
 	}
 	if (b->flags.b_Action)
@@ -451,7 +426,7 @@ void AddButtonAction(button_info *b, int n, char *action)
 	else
 	{
 		int i;
-		b->action = (char**)mymalloc(
+		b->action = fxmalloc(
 			(NUMBER_OF_EXTENDED_MOUSE_BUTTONS + 1) * sizeof(char*));
 		for (i = 0; i <= NUMBER_OF_EXTENDED_MOUSE_BUTTONS;
 			b->action[i++] = NULL)
@@ -484,7 +459,7 @@ void AddButtonAction(button_info *b, int n, char *action)
 			break;
 		}
 	}
-	t = (char *)mymalloc(l + 1);
+	t = fxmalloc(l + 1);
 	memmove(t, action, l);
 	t[l] = 0;
 	b->action[n] = t;
@@ -648,9 +623,9 @@ int main(int argc, char **argv)
 
 	if (argc<6 || argc>10)
 	{
-		fprintf(stderr,
-			"%s version %s should only be executed by fvwm!\n",
-			MyName, VERSION);
+		fvwm_debug(__func__,
+			   "%s version %s should only be executed by fvwm!\n",
+			   MyName, VERSION);
 		exit(1);
 	}
 
@@ -686,13 +661,13 @@ int main(int argc, char **argv)
 		/* There is a naming argument here! */
 		{
 			free(MyName);
-			MyName = safestrdup(argv[i]);
+			MyName = fxstrdup(argv[i]);
 			has_name = 1;
 		}
 		else if (!has_file)
 		/* There is a config file here! */
 		{
-			config_file = safestrdup(argv[i]);
+			config_file = fxstrdup(argv[i]);
 			has_file = 1;
 		}
 	}
@@ -701,9 +676,9 @@ int main(int argc, char **argv)
 	fd[1] = atoi(argv[2]);
 	if (!(Dpy = XOpenDisplay(NULL)))
 	{
-		fprintf(stderr,
-			"%s: Can't open display %s", MyName,
-			XDisplayName(NULL));
+		fvwm_debug(__func__,
+			   "%s: Can't open display %s", MyName,
+			   XDisplayName(NULL));
 		exit (1);
 	}
 	flib_init_graphics(Dpy);
@@ -715,13 +690,14 @@ int main(int argc, char **argv)
 	Root = RootWindow(Dpy, screen);
 	if (Root == None)
 	{
-		fprintf(stderr, "%s: Screen %d is not valid\n", MyName, screen);
+		fvwm_debug(__func__, "%s: Screen %d is not valid\n", MyName,
+			   screen);
 		exit(1);
 	}
 
 	oldErrorHandler = XSetErrorHandler(myErrorHandler);
 
-	UberButton = (button_info*)mymalloc(sizeof(button_info));
+	UberButton = fxmalloc(sizeof(button_info));
 	memset(UberButton, 0, sizeof(button_info));
 	UberButton->BWidth = 1;
 	UberButton->BHeight = 1;
@@ -731,7 +707,7 @@ int main(int argc, char **argv)
 	dph = DisplayHeight(Dpy, screen);
 
 #ifdef DEBUG_INIT
-	fprintf(stderr, "%s: Parsing...", MyName);
+	fvwm_debug(__func__, "%s: Parsing...", MyName);
 #endif
 
 	UberButton->title   = MyName;
@@ -752,10 +728,10 @@ int main(int argc, char **argv)
 		{
 			if (b->flags.b_ActiveIcon || b->flags.b_ActiveTitle)
 			{
-				fprintf(stderr,
-					"%s: Must specify ActiveColorset when "
-					"using ActiveIcon or ActiveTitle with "
-					"\"Pixmap none\".\n", MyName);
+				fvwm_debug(__func__,
+					   "%s: Must specify ActiveColorset when "
+					   "using ActiveIcon or ActiveTitle with "
+					   "\"Pixmap none\".\n", MyName);
 				exit(0);
 			}
 		}
@@ -772,19 +748,20 @@ int main(int argc, char **argv)
 	/* Don't quit if only a subpanel is empty */
 	if (UberButton->c->num_buttons == 0)
 	{
-		fprintf(stderr, "%s: No buttons defined. Quitting\n", MyName);
+		fvwm_debug(__func__, "%s: No buttons defined. Quitting\n",
+			   MyName);
 		exit(0);
 	}
 
 #ifdef DEBUG_INIT
-	fprintf(stderr, "OK\n%s: Shuffling...", MyName);
+	fvwm_debug(__func__, "OK\n%s: Shuffling...", MyName);
 #endif
 
 	ShuffleButtons(UberButton);
 	NumberButtons(UberButton);
 
 #ifdef DEBUG_INIT
-	fprintf(stderr, "OK\n%s: Creating Main Window ...", MyName);
+	fvwm_debug(__func__, "OK\n%s: Creating Main Window ...", MyName);
 #endif
 
 	xswa.colormap = Pcmap;
@@ -795,7 +772,7 @@ int main(int argc, char **argv)
 		CWColormap | CWBackPixmap | CWBorderPixel, &xswa);
 
 #ifdef DEBUG_INIT
-	fprintf(stderr, "OK\n%s: Loading data...\n", MyName);
+	fvwm_debug(__func__, "OK\n%s: Loading data...\n", MyName);
 #endif
 
 	/* Load fonts and icons, calculate max buttonsize */
@@ -812,7 +789,7 @@ int main(int argc, char **argv)
 	}
 
 #ifdef DEBUG_INIT
-	fprintf(stderr, "%s: Configuring main window...", MyName);
+	fvwm_debug(__func__, "%s: Configuring main window...", MyName);
 #endif
 
 	CreateUberButtonWindow(UberButton, maxx, maxy);
@@ -821,7 +798,8 @@ int main(int argc, char **argv)
 		Dpy, MyWindow, &root, &x, &y, (unsigned int *)&Width,
 		(unsigned int *)&Height, (unsigned int *)&border_width, &depth))
 	{
-		fprintf(stderr, "%s: Failed to get window geometry\n", MyName);
+		fvwm_debug(__func__, "%s: Failed to get window geometry\n",
+			   MyName);
 		exit(0);
 	}
 	SetButtonSize(UberButton, Width, Height);
@@ -844,7 +822,7 @@ int main(int argc, char **argv)
 	}
 
 #ifdef DEBUG_INIT
-	fprintf(stderr, "OK\n%s: Mapping windows...", MyName);
+	fvwm_debug(__func__, "OK\n%s: Mapping windows...", MyName);
 #endif
 
 	XMapSubwindows(Dpy, MyWindow);
@@ -861,8 +839,13 @@ int main(int argc, char **argv)
 	 * indent buttons correctly */
 	SendText(fd, "Send_WindowList", 0);
 
+	if (UberButton->c->flags.b_WindowName == 1)
+	{
+		change_window_name(windowname);
+	}
+
 #ifdef DEBUG_INIT
-	fprintf(stderr, "OK\n%s: Startup complete\n", MyName);
+	fvwm_debug(__func__, "OK\n%s: Startup complete\n", MyName);
 #endif
 
 	/*
@@ -976,7 +959,7 @@ void ButtonPressProcess (button_info *b, char **act)
 					if(i2-i>1)
 					{
 						b->flags.b_Hangon = 1;
-						b->hangon = mymalloc(i2-i);
+						b->hangon = fxmalloc(i2-i);
 						strncpy(
 							b->hangon, &(*act)[i+1],
 							i2-i-1);
@@ -987,7 +970,7 @@ void ButtonPressProcess (button_info *b, char **act)
 				else
 					i2=i;
 
-				tmp.name = mymalloc(strlen(*act)+1);
+				tmp.name = fxmalloc(strlen(*act)+1);
 				strcpy(tmp.name, "Exec ");
 				while (
 					(*act)[i2]!=0 &&
@@ -996,13 +979,6 @@ void ButtonPressProcess (button_info *b, char **act)
 					i2++;
 				}
 				strcat(tmp.name ,&(*act)[i2]);
-				if (is_transient)
-				{
-					/* delete the window before continuing
-					 */
-					XDestroyWindow(Dpy, MyWindow);
-					XSync(Dpy, 0);
-				}
 				SendText(fd,tmp.name,0);
 				if (is_transient)
 				{
@@ -1021,13 +997,6 @@ void ButtonPressProcess (button_info *b, char **act)
 				SaveButtons(UberButton);
 			else
 			{
-				if (is_transient)
-				{
-					/* delete the window before continuing
-					 */
-					XDestroyWindow(Dpy, MyWindow);
-					XSync(Dpy, 0);
-				}
 				SendText(fd,*act,0);
 				if (is_transient)
 				{
@@ -1182,11 +1151,13 @@ void Loop(void)
 	unsigned int depth, tw, th, border_width;
 	Window root;
 
+	fev_sanitise_configure_notify(&Event.xconfigure);
 	while (FCheckTypedWindowEvent(Dpy, MyWindow, ConfigureNotify, &event))
 	{
 	  if (!event.xconfigure.send_event &&
 	      Event.xconfigure.window != MyWindow)
 	    continue;
+	  fev_sanitise_configure_notify(&event.xconfigure);
 	  Event.xconfigure.x = event.xconfigure.x;
 	  Event.xconfigure.y = event.xconfigure.y;
 	  Event.xconfigure.send_event = True;
@@ -1434,7 +1405,7 @@ void Loop(void)
 		    (!(buttonSwallow(b)&b_NoHints)))
 	    {
 	      long supp;
-	      if (!XGetWMNormalHints(Dpy, swin, b->hints, &supp))
+	      if (!FGetWMNormalHints(Dpy, swin, b->hints, &supp))
 		b->hints->flags = 0;
 	      MakeButton(b);
 	      if (FShapesSupported)
@@ -1457,7 +1428,7 @@ void Loop(void)
 	button = -1;
 	while (NextButton(&ub, &b, &button, 0))
 	{
-	  if (b->flags.b_Panel && Event.xany.window == b->PanelWin)
+	  if (b->flags.b_Panel && Event.xdestroywindow.window == b->PanelWin)
 	  {
 	    /* A panel has been unmapped, update the button */
 	    b->newflags.panel_mapped = (Event.type == MapNotify);
@@ -1473,12 +1444,12 @@ void Loop(void)
 	{
 	  Window swin = SwallowedWindow(b);
 
-	  if ((buttonSwallowCount(b) == 3) && Event.xany.window == swin)
+	  if ((buttonSwallowCount(b) == 3) && Event.xdestroywindow.window == swin)
 	  {
 #ifdef DEBUG_HANGON
-	    fprintf(stderr,
-		    "%s: Button 0x%06x lost its window 0x%x (\"%s\")",
-		    MyName, (ushort)b, (ushort)swin, b->hangon);
+	    fvwm_debug(__func__,
+		       "%s: Button 0x%06x lost its window 0x%x (\"%s\")",
+		       MyName, (ushort)b, (ushort)swin, b->hangon);
 #endif
 	    b->swallow &= ~b_Count;
 	    if (b->flags.b_Panel)
@@ -1490,7 +1461,7 @@ void Loop(void)
 	      char *p;
 
 #ifdef DEBUG_HANGON
-	      fprintf(stderr, ", respawning\n");
+	      fvwm_debug(__func__, ", respawning\n");
 #endif
 	      if (b->newflags.is_panel && is_transient)
 	      {
@@ -1527,7 +1498,7 @@ void Loop(void)
 		    !b->newflags.is_panel)
 	    {
 #ifdef DEBUG_HANGON
-	      fprintf(stderr, ", waiting for respawned window\n");
+	      fvwm_debug(__func__, ", waiting for respawned window\n");
 #endif
 	      b->swallow |= 1;
 	      b->flags.b_Swallow = 1;
@@ -1544,7 +1515,7 @@ void Loop(void)
 	      b->flags.b_Panel = 0;
 	      RedrawButton(b, DRAW_FORCE, NULL);
 #ifdef DEBUG_HANGON
-	      fprintf(stderr, "\n");
+	      fvwm_debug(__func__, "\n");
 #endif
 	    }
 	    break;
@@ -1556,9 +1527,9 @@ void Loop(void)
 
 			default:
 #ifdef DEBUG_EVENTS
-				fprintf(stderr,
-					"%s: Event fell through unhandled\n",
-					MyName);
+				fvwm_debug(__func__,
+					   "%s: Event fell through unhandled\n",
+					   MyName);
 #endif
 				break;
 			} /* switch */
@@ -1577,7 +1548,6 @@ void RedrawWindow(void)
 	XEvent dummy;
 	button_info *ub, *b;
 	static Bool initial_redraw = True;
-	Bool clear_buttons;
 
 	if (ready < 1)
 	{
@@ -1593,12 +1563,10 @@ void RedrawWindow(void)
 	if (initial_redraw == False)
 	{
 		XClearWindow(Dpy, MyWindow);
-		clear_buttons = False;
 	}
 	else
 	{
 		initial_redraw = False;
-		clear_buttons = True;
 	}
 
 	button = -1;
@@ -1646,7 +1614,8 @@ void RecursiveLoadData(button_info *b, int *maxx, int *maxy)
     return;
 
 #ifdef DEBUG_LOADDATA
-  fprintf(stderr, "%s: Loading: Button 0x%06x: colors", MyName, (ushort)b);
+  fvwm_debug(__func__, "%s: Loading: Button 0x%06x: colors", MyName,
+             (ushort)b);
 #endif
 
 
@@ -1724,7 +1693,7 @@ void RecursiveLoadData(button_info *b, int *maxx, int *maxy)
     {
        b->c->colorset = -1;
 #ifdef DEBUG_LOADDATA
-      fprintf(stderr, ", colors2");
+      fvwm_debug(__func__, ", colors2");
 #endif
       if (b->c->flags.b_Fore)
 	b->c->fc = GetColor(b->c->fore);
@@ -1748,7 +1717,7 @@ void RecursiveLoadData(button_info *b, int *maxx, int *maxy)
   if (b->flags.b_Font)
   {
 #ifdef DEBUG_LOADDATA
-    fprintf(stderr, ", font \"%s\"", b->font_string);
+    fvwm_debug(__func__, ", font \"%s\"", b->font_string);
 #endif
 
     if (strncasecmp(b->font_string, "none", 4) == 0)
@@ -1764,7 +1733,7 @@ void RecursiveLoadData(button_info *b, int *maxx, int *maxy)
   if (b->flags.b_Container && b->c->flags.b_Font)
   {
 #ifdef DEBUG_LOADDATA
-    fprintf(stderr, ", font2 \"%s\"", b->c->font_string);
+    fvwm_debug(__func__, ", font2 \"%s\"", b->c->font_string);
 #endif
     if (strncasecmp(b->c->font_string, "none", 4) == 0)
     {
@@ -1780,7 +1749,7 @@ void RecursiveLoadData(button_info *b, int *maxx, int *maxy)
   if (b->flags.b_Container && b->c->num_buttons)
   {
 #ifdef DEBUG_LOADDATA
-    fprintf(stderr, ", entering container\n");
+    fvwm_debug(__func__, ", entering container\n");
 #endif
     for (i = 0; i < b->c->num_buttons; i++)
       if (b->c->buttons[i])
@@ -1792,7 +1761,8 @@ void RecursiveLoadData(button_info *b, int *maxx, int *maxy)
       y = b->c->miny;
     }
 #ifdef DEBUG_LOADDATA
-    fprintf(stderr, "%s: Loading: Back to container 0x%06x", MyName, (ushort)b);
+    fvwm_debug(__func__, "%s: Loading: Back to container 0x%06x", MyName,
+               (ushort)b);
 #endif
 
     x *= b->c->num_columns;
@@ -1818,7 +1788,7 @@ void RecursiveLoadData(button_info *b, int *maxx, int *maxy)
 	  LoadIconFile(b->icon_file, &b->icon, buttonColorset(b)))
   {
 #ifdef DEBUG_LOADDATA
-    fprintf(stderr, ", icon \"%s\"", b->icon_file);
+    fvwm_debug(__func__, ", icon \"%s\"", b->icon_file);
 #endif
     ix = b->icon->width;
     iy = b->icon->height;
@@ -1831,7 +1801,7 @@ void RecursiveLoadData(button_info *b, int *maxx, int *maxy)
       LoadIconFile(b->active_icon_file, &b->activeicon, buttonColorset(b)))
   {
 #ifdef DEBUG_LOADDATA
-    fprintf(stderr, ", active icon \"%s\"", b->active_icon_file);
+    fvwm_debug(__func__, ", active icon \"%s\"", b->active_icon_file);
 #endif
 
     hix = b->activeicon->width;
@@ -1849,7 +1819,7 @@ void RecursiveLoadData(button_info *b, int *maxx, int *maxy)
       LoadIconFile(b->press_icon_file, &b->pressicon, buttonColorset(b)))
   {
 #ifdef DEBUG_LOADDATA
-    fprintf(stderr, ", press icon \"%s\"", b->press_icon_file);
+    fvwm_debug(__func__, ", press icon \"%s\"", b->press_icon_file);
 #endif
 
     pix = b->pressicon->width;
@@ -1866,7 +1836,7 @@ void RecursiveLoadData(button_info *b, int *maxx, int *maxy)
   if (b->flags.b_Title && (Ffont = buttonFont(b)))
   {
 #ifdef DEBUG_LOADDATA
-    fprintf(stderr, ", title \"%s\"", b->title);
+    fvwm_debug(__func__, ", title \"%s\"", b->title);
 #endif
     if (buttonJustify(b)&b_Horizontal)
     {
@@ -1884,7 +1854,7 @@ void RecursiveLoadData(button_info *b, int *maxx, int *maxy)
   if (b->flags.b_ActiveTitle && (Ffont = buttonFont(b)))
   {
 #ifdef DEBUG_LOADDATA
-    fprintf(stderr, ", title \"%s\"", b->title);
+    fvwm_debug(__func__, ", title \"%s\"", b->title);
 #endif
     if (buttonJustify(b) & b_Horizontal)
     {
@@ -1908,7 +1878,7 @@ void RecursiveLoadData(button_info *b, int *maxx, int *maxy)
   if (b->flags.b_PressTitle && (Ffont = buttonFont(b)))
   {
 #ifdef DEBUG_LOADDATA
-    fprintf(stderr, ", title \"%s\"", b->title);
+    fvwm_debug(__func__, ", title \"%s\"", b->title);
 #endif
     if (buttonJustify(b) & b_Horizontal)
     {
@@ -1946,7 +1916,7 @@ void RecursiveLoadData(button_info *b, int *maxx, int *maxy)
   *maxx = max(x, *maxx);
   *maxy = max(y, *maxy);
 #ifdef DEBUG_LOADDATA
-  fprintf(stderr, ", size %ux%u, done\n", x, y);
+  fvwm_debug(__func__, ", size %ux%u, done\n", x, y);
 #endif
 }
 
@@ -1977,7 +1947,7 @@ static void HandlePanelPress(button_info *b)
 
   /* force StaticGravity on window */
   mysizehints.flags = 0;
-  XGetWMNormalHints(Dpy, b->PanelWin, &mysizehints, &supplied);
+  FGetWMNormalHints(Dpy, b->PanelWin, &mysizehints, &supplied);
   mysizehints.flags |= PWinGravity;
   mysizehints.win_gravity = StaticGravity;
   XSetWMNormalHints(Dpy, b->PanelWin, &mysizehints);
@@ -1999,7 +1969,7 @@ static void HandlePanelPress(button_info *b)
 
     /* now request mapping in the void and wait until it gets mapped */
     mysizehints.flags = 0;
-    XGetWMNormalHints(Dpy, b->PanelWin, &mysizehints, &supplied);
+    FGetWMNormalHints(Dpy, b->PanelWin, &mysizehints, &supplied);
     mysizehints.flags |= USPosition;
     /* hack to prevent mapping panels on wrong screen with StartsOnScreen */
     FScreenMangleScreenIntoUSPosHints(FSCREEN_XYPOS, &mysizehints);
@@ -2154,13 +2124,13 @@ void CreateUberButtonWindow(button_info *ub, int maxx, int maxy)
 	}
 
 #ifdef DEBUG_INIT
-	fprintf(stderr, "making atoms...");
+	fvwm_debug(__func__, "making atoms...");
 #endif
 
 	_XA_WM_DEL_WIN = XInternAtom(Dpy, "WM_DELETE_WINDOW", 0);
 
 #ifdef DEBUG_INIT
-	fprintf(stderr, "sizing...");
+	fvwm_debug(__func__, "sizing...");
 #endif
 
 	mysizehints.flags = PWinGravity | PBaseSize;
@@ -2172,7 +2142,7 @@ void CreateUberButtonWindow(button_info *ub, int maxx, int maxy)
 	if (w>-1) /* from geometry */
 	{
 #ifdef DEBUG_INIT
-		fprintf(stderr, "constraining (w=%i)...", w);
+		fvwm_debug(__func__, "constraining (w=%i)...", w);
 #endif
 		ConstrainSize(&mysizehints, &w, &h);
 		mysizehints.width = w;
@@ -2181,7 +2151,7 @@ void CreateUberButtonWindow(button_info *ub, int maxx, int maxy)
 	}
 
 #ifdef DEBUG_INIT
-	fprintf(stderr, "gravity...");
+	fvwm_debug(__func__, "gravity...");
 #endif
 	wx = 0;
 	wy = 0;
@@ -2220,17 +2190,17 @@ void CreateUberButtonWindow(button_info *ub, int maxx, int maxy)
 #ifdef DEBUG_INIT
 	if (mysizehints.flags&USPosition)
 	{
-		fprintf(stderr,
-			"create(%i,%i,%u,%u,1,%u,%u)...",
-			wx, wy, mysizehints.width, mysizehints.height,
-			(ushort)fore_pix, (ushort)back_pix);
+		fvwm_debug(__func__,
+			   "create(%i,%i,%u,%u,1,%u,%u)...",
+			   wx, wy, mysizehints.width, mysizehints.height,
+			   (ushort)fore_pix, (ushort)back_pix);
 	}
 	else
 	{
-		fprintf(stderr,
-			"create(-,-,%u,%u,1,%u,%u)...",
-			mysizehints.width, mysizehints.height,
-		(ushort)fore_pix, (ushort)back_pix);
+		fvwm_debug(__func__,
+			   "create(-,-,%u,%u,1,%u,%u)...",
+			   mysizehints.width, mysizehints.height,
+			   (ushort)fore_pix, (ushort)back_pix);
 	}
 #endif
 
@@ -2245,7 +2215,7 @@ void CreateUberButtonWindow(button_info *ub, int maxx, int maxy)
 	}
 
 #ifdef DEBUG_INIT
-	fprintf(stderr, "colors...");
+	fvwm_debug(__func__, "colors...");
 #endif
 	if (Pdepth < 2)
 	{
@@ -2264,12 +2234,12 @@ void CreateUberButtonWindow(button_info *ub, int maxx, int maxy)
 
 
 #ifdef DEBUG_INIT
-	fprintf(stderr, "properties...");
+	fvwm_debug(__func__, "properties...");
 #endif
 	XSetWMProtocols(Dpy, MyWindow, &_XA_WM_DEL_WIN, 1);
 
-	myclasshints.res_name = safestrdup(MyName);
-	myclasshints.res_class = safestrdup("FvwmButtons");
+	myclasshints.res_name = fxstrdup(MyName);
+	myclasshints.res_class = fxstrdup("FvwmButtons");
 
 	{
 		XTextProperty mynametext;
@@ -2277,9 +2247,9 @@ void CreateUberButtonWindow(button_info *ub, int maxx, int maxy)
 		list[0] = MyName;
 		if (!XStringListToTextProperty(list, 1, &mynametext))
 		{
-			fprintf(stderr,
-				"%s: Failed to convert name to XText\n",
-				MyName);
+			fvwm_debug(__func__,
+				   "%s: Failed to convert name to XText\n",
+				   MyName);
 			exit(1);
 		}
 		XSetWMProperties(
@@ -2291,7 +2261,7 @@ void CreateUberButtonWindow(button_info *ub, int maxx, int maxy)
 	XSelectInput(Dpy, MyWindow, MW_EVENTS);
 
 #ifdef DEBUG_INIT
-	fprintf(stderr, "GC...");
+	fvwm_debug(__func__, "GC...");
 #endif
 	gcm = GCForeground|GCBackground;
 	gcv.foreground = fore_pix;
@@ -2369,8 +2339,9 @@ void DebugEvents(XEvent *event)
 		"ClientMessage",
 		"MappingNotify"
 	};
-	fprintf(stderr, "%s: Received %s event from window 0x%x\n",
-		MyName, event_names[event->type], (ushort)event->xany.window);
+	fvwm_debug(__func__, "%s: Received %s event from window 0x%x\n",
+		   MyName, event_names[event->type],
+		   (ushort)event->xany.window);
 }
 #endif
 
@@ -2481,11 +2452,11 @@ void SpawnSome(void)
 			if (b->spawn)
 			{
 #ifdef DEBUG_HANGON
-				fprintf(stderr,
-					"%s: Button 0x%06x did not find a "
-					"\"%s\" window, %s",
-					MyName, (ushort)b, b->hangon,
-					"spawning own\n");
+				fvwm_debug(__func__,
+					   "%s: Button 0x%06x did not find a "
+					   "\"%s\" window, %s",
+					   MyName, (ushort)b, b->hangon,
+					   "spawning own\n");
 #endif
 				p = module_expand_action(
 					Dpy, screen, b->spawn, NULL,
@@ -2732,11 +2703,6 @@ static void handle_config_info_packet(unsigned long *body)
 		colorset = LoadColorset(tline);
 		change_colorset(colorset, NULL);
 	}
-	else if (StrEquals(token, XINERAMA_CONFIG_STRING))
-	{
-		FScreenConfigureModule(tline);
-	}
-	return;
 }
 
 
@@ -2904,11 +2870,13 @@ void CheckForHangon(unsigned long *body)
 						&b->bw, &d);
 
 #ifdef DEBUG_HANGON
-				fprintf(stderr,
-					"%s: Button 0x%06x %s 0x%lx \"%s\", "
-					"parent 0x%lx\n", MyName, (ushort)b,
-					"will swallow window", body[0], cbody,
-					b->IconWinParent);
+				fvwm_debug(__func__,
+					   "%s: Button 0x%06x %s 0x%lx \"%s\", "
+					   "parent 0x%lx\n", MyName,
+					   (ushort)b,
+					   "will swallow window", body[0],
+					   cbody,
+					   b->IconWinParent);
 #endif
 
 				if (buttonSwallow(b)&b_UseOld)
@@ -3440,7 +3408,8 @@ void exec_swallow(char *action, button_info *b)
 		len = 45 + strlen(my_sm_env) + strlen(orig_sm_env);
 	}
 
-	cmd = safemalloc(len + strlen(action));
+	/* TA:  FIXME!  xasprintf() */
+	cmd = fxmalloc(len + strlen(action));
 	sprintf(
 		cmd,
 		"FSMExecFuncWithSessionManagment \"%s\" \"%s\" \"%s\"",
@@ -3448,3 +3417,22 @@ void exec_swallow(char *action, button_info *b)
 	SendText(fd, cmd, 0);
 	free(cmd);
 }
+
+/*
+ *  Change the window name displayed in the title bar.
+ *  Helper function borrowed from FvwmIdent for
+ *  WindowName FvwmButtons functionality
+ */
+void change_window_name(char *str)
+{
+	XTextProperty name;
+
+	if (XStringListToTextProperty(&str, 1, &name) == 0)
+	{
+		fprintf(stderr,"FvwmButtons: cannot allocate window name");
+		return;
+	}
+	XSetWMName(Dpy, MyWindow, &name);
+	XSetWMIconName(Dpy, MyWindow, &name);
+	XFree(name.value);
+}
