@@ -5,12 +5,6 @@
  * This module, and the entire GoodStuff program, and the concept for
  * interfacing this module to the Window Manager, are all original work
  * by Robert Nation
- *
- * Copyright 1993, Robert Nation. No guarantees or warantees or anything
- * are provided or implied in any way whatsoever. Use this program at your
- * own risk. Permission to use this program for any purpose is given,
- * as long as the copyright is kept intact.
- *
  */
 
 /* This program is free software; you can redistribute it and/or modify
@@ -24,8 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 #include "config.h"
@@ -33,11 +26,7 @@
 #include <ctype.h>
 #include <stdio.h>
 
-#include <X11/Xlib.h>
-#include <X11/Intrinsic.h>
-
 #include "libs/fvwmlib.h"
-#include "libs/FScreen.h"
 #include "libs/Module.h"
 #include "libs/Colorset.h"
 #include "libs/Parse.h"
@@ -145,15 +134,15 @@ static int ParseBack(char **ss)
 		{
 		case 0: /* Icon */
 			r = 1;
-			fprintf(stderr,
-				"%s: Back(Icon) not supported yet\n",
-				MyName);
+			fvwm_debug(__func__,
+				   "%s: Back(Icon) not supported yet\n",
+				   MyName);
 			break;
 		default:
 			t = seekright(&s);
-			fprintf(stderr,
-				"%s: Illegal back option \"%s\"\n",
-				MyName, (t) ? t : "");
+			fvwm_debug(__func__,
+				   "%s: Illegal back option \"%s\"\n",
+				   MyName, (t) ? t : "");
 			if (t)
 			{
 				free(t);
@@ -207,9 +196,9 @@ static void ParseBoxSize(char **ss, flags_type *flags)
 	default:
 		flags->b_SizeFixed = 0;
 		flags->b_SizeSmart = 0;
-		fprintf(stderr,
-			"%s: Illegal boxsize option \"%s\"\n",
-			MyName, s);
+		fvwm_debug(__func__,
+			   "%s: Illegal boxsize option \"%s\"\n",
+			   MyName, s);
 		break;
 	}
 	return;
@@ -266,9 +255,9 @@ static void ParseTitle(char **ss, byte *flags, byte *mask)
 			break;
 		default:
 			t = seekright(&s);
-			fprintf(stderr,
-				"%s: Illegal title option \"%s\"\n",
-				MyName, (t) ? t : "");
+			fvwm_debug(__func__,
+				   "%s: Illegal title option \"%s\"\n",
+				   MyName, (t) ? t : "");
 			if (t)
 			{
 				free(t);
@@ -379,9 +368,10 @@ static void ParseSwallow(
 			break;
 		default:
 			t = seekright(&s);
-			fprintf(stderr,
-				"%s: Illegal Swallow option \"%s\"\n", MyName,
-				(t) ? t : "");
+			fvwm_debug(__func__,
+				   "%s: Illegal Swallow option \"%s\"\n",
+				   MyName,
+				   (t) ? t : "");
 			if (t)
 			{
 				free(t);
@@ -403,7 +393,8 @@ static void ParsePanel(
 	char **ss, unsigned int *flags, unsigned int *mask, char *direction,
 	int *steps, int *delay, panel_flags_type *panel_flags,
 	int *indicator_size, int *rela_x, int *rela_y, char *position,
-	char *context)
+	char *context, int *indicator_sunkraise, char *indicator_in_out)
+
 {
 	char *swallowopts[] =
 	{
@@ -431,6 +422,8 @@ static void ParsePanel(
 
 	char *t, *s = *ss;
 	int n;
+	char *instr = "in";
+	char *outstr = "out";
 
 	while (*s && *s != ')')
 	{
@@ -522,7 +515,34 @@ static void ParsePanel(
 			n = 0;
 			(*panel_flags).panel_indicator = 1;
 			*indicator_size = 0;
-			sscanf(s, "%d%n", indicator_size, &n);
+			sscanf(s, "%d %s%n", indicator_size, indicator_in_out, &n);
+
+			/* Remove comma from %s of sscanf */
+			indicator_in_out[strcspn(indicator_in_out, ",")] = '\0';
+
+			if (*indicator_in_out == *instr)
+			{
+			        *indicator_sunkraise = 2;
+			}
+			if (*indicator_in_out == *outstr)
+			{
+			        *indicator_sunkraise = 1;
+			}
+
+			/* Not defined - parse indicator size without in/out after it */
+			if (*indicator_sunkraise < 1 || *indicator_sunkraise > 2)
+			{
+			        *indicator_sunkraise = 1;
+			        s = trimleft(s);
+			        n = n - 1;
+			        sscanf(s, "%d%n", indicator_size, &n);
+			        if (*indicator_size < 0 || *indicator_size > 100)
+			        {
+			                *indicator_size = 0;
+			        }
+			}
+
+
 			if (*indicator_size < 0 || *indicator_size > 100)
 			{
 				*indicator_size = 0;
@@ -599,10 +619,11 @@ static void ParsePanel(
 					s--;
 					if (t)
 					{
-						fprintf(stderr,
-							"%s: Illegal Panel "
-							"position option %s\n",
-							MyName,	(t) ? t : "");
+						fvwm_debug(__func__,
+							   "%s: Illegal Panel "
+							   "position option %s\n",
+							   MyName,
+							   	(t) ? t : "");
 						free(t);
 					}
 				}
@@ -610,9 +631,10 @@ static void ParsePanel(
 			break;
 		default:
 			t = seekright(&s);
-			fprintf(stderr,
-				"%s: Illegal Panel option \"%s\"\n", MyName,
-				(t) ? t : "");
+			fvwm_debug(__func__,
+				   "%s: Illegal Panel option \"%s\"\n",
+				   MyName,
+				   (t) ? t : "");
 			if (t)
 			{
 				free(t);
@@ -725,7 +747,7 @@ static void ParseContainer(char **ss,button_info *b)
 	b->c->flags.b_Padding = 1;
       }
       else
-	fprintf(stderr,"%s: Illegal padding argument\n",MyName);
+	fvwm_debug(__func__, "%s: Illegal padding argument\n",MyName);
       break;
     case 7: /* Title - flags */
       s = trimleft(s);
@@ -742,9 +764,9 @@ static void ParseContainer(char **ss,button_info *b)
       else
       {
 	char *temp;
-	fprintf(stderr,
-		"%s: Illegal title in container options\n",
-		MyName);
+	fvwm_debug(__func__,
+                   "%s: Illegal title in container options\n",
+                   MyName);
 	temp = seekright(&s);
 	if (temp)
 	{
@@ -759,8 +781,8 @@ static void ParseContainer(char **ss,button_info *b)
 	s = trimleft(s);
 	if (b->c->flags.b_Swallow || b->c->flags.b_Panel)
 	{
-	  fprintf(stderr, "%s: Multiple Swallow or Panel options are not"
-		" allowed in a single button", MyName);
+	  fvwm_debug(__func__, "%s: Multiple Swallow or Panel options are not"
+                     " allowed in a single button", MyName);
 	  failed = True;
 	}
 	else if (*s == '(' && s++)
@@ -775,9 +797,9 @@ static void ParseContainer(char **ss,button_info *b)
 	}
 	else
 	{
-	  fprintf(stderr,
-		  "%s: Illegal swallow or panel in container options\n",
-		  MyName);
+	  fvwm_debug(__func__,
+                     "%s: Illegal swallow or panel in container options\n",
+                     MyName);
 	  failed = True;
 	}
 	if (failed)
@@ -806,7 +828,7 @@ static void ParseContainer(char **ss,button_info *b)
 	s = o;
       }
       else
-	fprintf(stderr,"%s: Illegal size arguments\n",MyName);
+	fvwm_debug(__func__, "%s: Illegal size arguments\n",MyName);
       break;
 
     case 11: /* BoxSize */
@@ -830,7 +852,8 @@ static void ParseContainer(char **ss,button_info *b)
 
     default:
       t = seekright(&s);
-      fprintf(stderr,"%s: Illegal container option \"%s\"\n",MyName, (t)?t:"");
+      fvwm_debug(__func__, "%s: Illegal container option \"%s\"\n",MyName,
+                 (t)?t:"");
       if (t)
 	free(t);
     }
@@ -1002,7 +1025,7 @@ static void ParseButton(button_info **uberb, char *s)
 					}
 				}
 				t = seekright(&s);
-				if (t && *t && (t[0] != '-' || t[1] != 0))
+				if (t && *t)
 				{
 					if (b->title)
 					{
@@ -1010,17 +1033,17 @@ static void ParseButton(button_info **uberb, char *s)
 					}
 					b->title = t;
 #ifdef DEBUG_PARSER
-					fprintf(stderr,
-						"PARSE: Title \"%s\"\n",
-						b->title);
+					fvwm_debug(__func__,
+						   "PARSE: Title \"%s\"\n",
+						   b->title);
 #endif
 					b->flags.b_Title = 1;
 				}
 				else
 				{
-					fprintf(stderr,
-						"%s: Missing title argument\n",
-						MyName);
+					fvwm_debug(__func__,
+						   "%s: Missing title argument\n",
+						   MyName);
 					if (t)
 					{
 						free(t);
@@ -1036,13 +1059,13 @@ static void ParseButton(button_info **uberb, char *s)
 				{
 					if (b->flags.b_Swallow)
 					{
-						fprintf(stderr,
-							"%s: a button can not "
-							"have an icon and a "
-							"swallowed window at "
-							"the same time. "
-							"Ignoring icon\n",
-							MyName);
+						fvwm_debug(__func__,
+							   "%s: a button can not "
+							   "have an icon and a "
+							   "swallowed window at "
+							   "the same time. "
+							   "Ignoring icon\n",
+							   MyName);
 					}
 					else
 					{
@@ -1056,9 +1079,9 @@ static void ParseButton(button_info **uberb, char *s)
 				}
 				else
 				{
-					fprintf(stderr,
-						"%s: Missing Icon argument\n",
-						MyName);
+					fvwm_debug(__func__,
+						   "%s: Missing Icon argument\n",
+						   MyName);
 					if (t)
 					{
 						free(t);
@@ -1078,9 +1101,9 @@ static void ParseButton(button_info **uberb, char *s)
 				}
 				else
 				{
-					fprintf(stderr,
-						"%s: Illegal frame argument\n",
-						MyName);
+					fvwm_debug(__func__,
+						   "%s: Illegal frame argument\n",
+						   MyName);
 				}
 				break;
 
@@ -1102,9 +1125,9 @@ static void ParseButton(button_info **uberb, char *s)
 				}
 				else
 				{
-					fprintf(stderr,
-						"%s: Illegal padding "
-						"argument\n", MyName);
+					fvwm_debug(__func__,
+						   "%s: Illegal padding "
+						   "argument\n", MyName);
 				}
 				break;
 
@@ -1155,7 +1178,9 @@ static void ParseButton(button_info **uberb, char *s)
 							&b->relative_x,
 							&b->relative_y,
 							&b->slide_position,
-							&b->slide_context);
+							&b->slide_context,
+							&b->indicator_sunkraise,
+							&b->indicator_in_out);
 					}
 				}
 				t = seekright(&s);
@@ -1171,13 +1196,13 @@ static void ParseButton(button_info **uberb, char *s)
 					{
 						if (b->flags.b_Icon)
 						{
-							fprintf(stderr,
-							"%s: a button can not "
-							"have an icon and a "
-							"swallowed window at "
-							"the same time. "
-							"Ignoring icon\n",
-							MyName);
+							fvwm_debug(__func__,
+								   "%s: a button can not "
+								   "have an icon and a "
+								   "swallowed window at "
+								   "the same time. "
+								   "Ignoring icon\n",
+								   MyName);
 							b->flags.b_Icon = 0;
 						}
 
@@ -1194,8 +1219,8 @@ static void ParseButton(button_info **uberb, char *s)
 					b->swallow |= 1;
 					if (!(b->swallow & b_NoHints))
 					{
-						b->hints = (XSizeHints*)
-						mymalloc(sizeof(XSizeHints));
+						b->hints = fxmalloc(
+							sizeof(XSizeHints));
 					}
 					if (o)
 					{
@@ -1226,9 +1251,9 @@ static void ParseButton(button_info **uberb, char *s)
 				}
 				else
 				{
-					fprintf(stderr,
-						"%s: Missing swallow "
-						"argument\n", MyName);
+					fvwm_debug(__func__,
+						   "%s: Missing swallow "
+						   "argument\n", MyName);
 					if (t)
 					{
 						free(t);
@@ -1286,7 +1311,9 @@ static void ParseButton(button_info **uberb, char *s)
 				free(b);
 				if (!ub->parent)
 				{
-					fprintf(stderr,"%s: Unmatched END in config file\n",MyName);
+					fvwm_debug(__func__,
+						   "%s: Unmatched END in config file\n",
+						   MyName);
 					exit(1);
 				}
 				if (ub->parent->c->flags.b_Colorset ||
@@ -1317,9 +1344,9 @@ static void ParseButton(button_info **uberb, char *s)
 				}
 				else
 				{
-					fprintf(stderr,
-						"%s: Illegal size arguments\n",
-						MyName);
+					fvwm_debug(__func__,
+						   "%s: Illegal size arguments\n",
+						   MyName);
 				}
 				break;
 
@@ -1363,9 +1390,9 @@ static void ParseButton(button_info **uberb, char *s)
 					s++;
 					if (strncasecmp(s, "mouse", 5) != 0)
 					{
-						fprintf(stderr,
-							"%s: Couldn't parse "
-							"action\n", MyName);
+						fvwm_debug(__func__,
+							   "%s: Couldn't parse "
+							   "action\n", MyName);
 					}
 					s += 5;
 					i = strtol(s, &t, 10);
@@ -1407,9 +1434,9 @@ static void ParseButton(button_info **uberb, char *s)
 				}
 				else
 				{
-					fprintf(stderr,
-						"%s: Missing action argument\n",
-						MyName);
+					fvwm_debug(__func__,
+						   "%s: Missing action argument\n",
+						   MyName);
 				}
 				break;
 
@@ -1437,17 +1464,18 @@ static void ParseButton(button_info **uberb, char *s)
 					}
 			        	else
 					{
-						fprintf(stderr,
-							"%s: Incorrect id '%s' "
-							"ignored\n", MyName, t);
+						fvwm_debug(__func__,
+							   "%s: Incorrect id '%s' "
+							   "ignored\n",
+							   MyName, t);
 					}
 					free(t);
 				}
 				else
 				{
-					fprintf(stderr,
-						"%s: Missing id argument\n",
-						MyName);
+					fvwm_debug(__func__,
+						   "%s: Missing id argument\n",
+						   MyName);
 				}
 				break;
 
@@ -1458,13 +1486,13 @@ static void ParseButton(button_info **uberb, char *s)
 				{
 					if (b->flags.b_Swallow)
 					{
-						fprintf(stderr,
-							"%s: a button can not "
-							"have a ActiveIcon and "
-							"a swallowed window at "
-							"the same time. "
-							"Ignoring ActiveIcon.\n",
-							MyName);
+						fvwm_debug(__func__,
+							   "%s: a button can not "
+							   "have a ActiveIcon and "
+							   "a swallowed window at "
+							   "the same time. "
+							   "Ignoring ActiveIcon.\n",
+							   MyName);
 					}
 					else
 					{
@@ -1478,9 +1506,9 @@ static void ParseButton(button_info **uberb, char *s)
 				}
 				else
 				{
-					fprintf(stderr,
-						"%s: Missing ActiveIcon "
-						"argument\n", MyName);
+					fvwm_debug(__func__,
+						   "%s: Missing ActiveIcon "
+						   "argument\n", MyName);
 					if (t)
 					{
 						free(t);
@@ -1493,9 +1521,10 @@ static void ParseButton(button_info **uberb, char *s)
 				s = trimleft(s);
 				if (*s == '(')
 				{
-					fprintf(stderr,
-						"%s: justification not allowed "
-						"for ActiveTitle.\n", MyName);
+					fvwm_debug(__func__,
+						   "%s: justification not allowed "
+						   "for ActiveTitle.\n",
+						   MyName);
 				}
 				t = seekright(&s);
 				if (t && *t && (t[0] != '-' || t[1] != 0))
@@ -1506,17 +1535,17 @@ static void ParseButton(button_info **uberb, char *s)
 					}
 					b->activeTitle = t;
 #ifdef DEBUG_PARSER
-					fprintf(stderr,
-						"PARSE: ActiveTitle \"%s\"\n",
-						b->activeTitle);
+					fvwm_debug(__func__,
+						   "PARSE: ActiveTitle \"%s\"\n",
+						   b->activeTitle);
 #endif
 					b->flags.b_ActiveTitle = 1;
 				}
 				else
 				{
-					fprintf(stderr,
-						"%s: Missing ActiveTitle "
-						"argument\n", MyName);
+					fvwm_debug(__func__,
+						   "%s: Missing ActiveTitle "
+						   "argument\n", MyName);
 					if (t)
 					{
 						free(t);
@@ -1531,13 +1560,13 @@ static void ParseButton(button_info **uberb, char *s)
 				{
 					if (b->flags.b_Swallow)
 					{
-						fprintf(stderr,
-							"%s: a button can not "
-							"have a PressIcon and "
-							"a swallowed window at "
-							"the same time. "
-							"Ignoring PressIcon.\n",
-							MyName);
+						fvwm_debug(__func__,
+							   "%s: a button can not "
+							   "have a PressIcon and "
+							   "a swallowed window at "
+							   "the same time. "
+							   "Ignoring PressIcon.\n",
+							   MyName);
 					}
 					else
 					{
@@ -1551,9 +1580,9 @@ static void ParseButton(button_info **uberb, char *s)
 				}
 				else
 				{
-					fprintf(stderr,
-						"%s: Missing PressIcon "
-						"argument\n", MyName);
+					fvwm_debug(__func__,
+						   "%s: Missing PressIcon "
+						   "argument\n", MyName);
 					if (t)
 					{
 						free(t);
@@ -1566,9 +1595,10 @@ static void ParseButton(button_info **uberb, char *s)
 				s = trimleft(s);
 				if (*s == '(')
 				{
-					fprintf(stderr,
-						"%s: justification not allowed "
-						"for PressTitle.\n", MyName);
+					fvwm_debug(__func__,
+						   "%s: justification not allowed "
+						   "for PressTitle.\n",
+						   MyName);
 				}
 				t = seekright(&s);
 				if (t && *t && (t[0] != '-' || t[1] != 0))
@@ -1579,17 +1609,17 @@ static void ParseButton(button_info **uberb, char *s)
 					}
 					b->pressTitle = t;
 #ifdef DEBUG_PARSER
-					fprintf(stderr,
-						"PARSE: PressTitle \"%s\"\n",
-						b->pressTitle);
+					fvwm_debug(__func__,
+						   "PARSE: PressTitle \"%s\"\n",
+						   b->pressTitle);
 #endif
 					b->flags.b_PressTitle = 1;
 				}
 				else
 				{
-					fprintf(stderr,
-						"%s: Missing PressTitle "
-						"argument\n", MyName);
+					fvwm_debug(__func__,
+						   "%s: Missing PressTitle "
+						   "argument\n", MyName);
 					if (t)
 					{
 						free(t);
@@ -1635,9 +1665,9 @@ static void ParseButton(button_info **uberb, char *s)
 			/* --------------- --------------- */
 			default:
 				t = seekright(&s);
-				fprintf(stderr,
-					"%s: Illegal button option \"%s\"\n",
-					MyName, (t) ? t : "");
+				fvwm_debug(__func__,
+					   "%s: Illegal button option \"%s\"\n",
+					   MyName, (t) ? t : "");
 				if (t)
 				{
 					free(t);
@@ -1707,9 +1737,9 @@ static void ParseButton(button_info **uberb, char *s)
 	{
 		if (b->flags.b_Swallow || b->flags.b_Panel)
 		{
-			fprintf(stderr,
-				"%s: Illegal with both old and new swallow!\n",
-				MyName);
+			fvwm_debug(__func__,
+				   "%s: Illegal with both old and new swallow!\n",
+				   MyName);
 			exit(1);
 		}
 		s += 7;
@@ -1727,7 +1757,7 @@ static void ParseButton(button_info **uberb, char *s)
 		b->hangon = seekright(&s);
 		if (!b->hangon)
 		{
-			b->hangon = safestrdup("");
+			b->hangon = fxstrdup("");
 		}
 		if (tolower(*s) == 's')
 		{
@@ -1743,7 +1773,7 @@ static void ParseButton(button_info **uberb, char *s)
 		s = trimleft(s);
 		if (!(b->swallow & b_NoHints))
 		{
-			b->hints = (XSizeHints*)mymalloc(sizeof(XSizeHints));
+			b->hints = fxmalloc(sizeof(XSizeHints));
 		}
 		if (*s)
 		{
@@ -1751,7 +1781,7 @@ static void ParseButton(button_info **uberb, char *s)
 			{
 				SendText(fd, s, 0);
 			}
-			b->spawn = safestrdup(s);
+			b->spawn = fxstrdup(s);
 		}
 	}
 	else if (*s)
@@ -1784,6 +1814,7 @@ static void ParseConfigLine(button_info **ubb, char *s)
 		"colorset",
 		"activecolorset",
 		"presscolorset",
+		"windowname",
 		NULL
 	};
 	int i, j, k;
@@ -1932,6 +1963,23 @@ static void ParseConfigLine(button_info **ubb, char *s)
 		}
 		break;
 
+	case 15: /* WindowName */
+	{
+		i = sscanf(s, "%126[^\n]", windowname);
+		if (i > 0)
+		{
+			ub->c->flags.b_WindowName = 1;
+		}
+		else
+		{
+			ub->c->flags.b_WindowName = 0;
+		}
+#ifdef DEBUG_PARSER
+		fprintf(stderr, "%s\n", windowname);
+#endif
+		break;
+	}
+
 	default:
 		s = trimleft(s);
 		ParseButton(ubb, s);
@@ -1950,9 +1998,9 @@ static void ParseConfigFile(button_info *ub)
 	int l;
 	if (!f)
 	{
-		fprintf(stderr,
-			"%s: Couldn't open config file %s\n", MyName,
-			config_file);
+		fvwm_debug(__func__,
+			   "%s: Couldn't open config file %s\n", MyName,
+			   config_file);
 		return;
 	}
 
@@ -1962,7 +2010,10 @@ static void ParseConfigFile(button_info *ub)
 		while ((l = strlen(s)) < sizeof(s)
 			&& l >= 2 && s[l - 1] == '\n' && s[l - 2] == '\\')
 		{
-			fgets(s + l - 2, sizeof(s) - l, f);
+			char *p;
+
+			p = fgets(s + l - 2, sizeof(s) - l, f);
+			(void)p;
 		}
 
 		/* And comments: */
@@ -2054,11 +2105,10 @@ void ParseConfiguration(button_info *ub)
 		NULL, /* filled out below */
 		"imagepath",
 		"colorset",
-		XINERAMA_CONFIG_STRING,
 		NULL
 	};
 
-	items[0] = mymalloc(strlen(MyName) + 2);
+	items[0] = fxmalloc(strlen(MyName) + 2);
 	sprintf(items[0], "*%s", MyName);
 
 	/* send config lines with MyName */
@@ -2088,10 +2138,6 @@ void ParseConfiguration(button_info *ub)
 			/* store colorset sent by fvwm */
 			LoadColorset(rest);
 			break;
-		case 3:
-			/* Xinerama state */
-			FScreenConfigureModule(rest);
-			break;
 		}
 		GetConfigLine(fd,&s);
 	}
