@@ -10,8 +10,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 #include "config.h"
@@ -19,6 +18,7 @@
 #include "libs/fvwmlib.h"
 #include "libs/FShape.h"
 #include "libs/Module.h"
+#include "libs/FEvent.h"
 #include "FvwmIconMan.h"
 #include "x.h"
 #include "xmanager.h"
@@ -339,8 +339,9 @@ static ManGeometry *query_geometry(WinManager *man)
   {
     g.x = off_x;
     g.y = off_y;
-    fprintf(stderr, "%s: query_geometry: failed to get frame attributes.\n",
-	    MyName);
+    fvwm_debug(__func__,
+               "%s: query_geometry: failed to get frame attributes.\n",
+               MyName);
   }
   if (XGetWindowAttributes(theDisplay, man->theWindow, &win_attr))
   {
@@ -351,8 +352,9 @@ static ManGeometry *query_geometry(WinManager *man)
   {
     g.width = 1;
     g.height = 1;
-    fprintf(stderr, "%s: query_geometry: failed to get window attributes.\n",
-	    MyName);
+    fvwm_debug(__func__,
+               "%s: query_geometry: failed to get window attributes.\n",
+               MyName);
   }
 
   return &g;
@@ -366,7 +368,7 @@ static void fix_manager_size(WinManager *man, int w, int h)
   if (man->geometry.dir & GROW_FIXED)
     return;
 
-  XGetWMNormalHints(theDisplay, man->theWindow, &size, &mask);
+  FGetWMNormalHints(theDisplay, man->theWindow, &size, &mask);
   size.min_width = w;
   size.max_width = w;
   size.min_height = h;
@@ -374,14 +376,33 @@ static void fix_manager_size(WinManager *man, int w, int h)
   XSetWMNormalHints(theDisplay, man->theWindow, &size);
 }
 
+static void release_manager_size(WinManager *man)
+{
+  XSizeHints size;
+  long mask;
+
+  if (man->geometry.dir & GROW_FIXED)
+    return;
+
+  FGetWMNormalHints(theDisplay, man->theWindow, &size, &mask);
+  size.min_width = 1;
+  size.max_width = 65535;
+  size.min_height = 1;
+  size.max_height = 65535;
+  XSetWMNormalHints(theDisplay, man->theWindow, &size);
+}
+
 /* Like XMoveResizeWindow(), but can move in arbitary directions */
 static void resize_window(WinManager *man)
 {
+  XWindowAttributes attribs;
   ManGeometry *g;
-  int x_changed, y_changed, dir;
+  int x_changed, y_changed, dir, counter = 20000;
 
   dir = man->geometry.dir;
-  fix_manager_size(man, man->geometry.width, man->geometry.height);
+  /* Remove any size limits so that the window can be resized without
+   * warnings. */
+  release_manager_size(man);
 
   if ((dir & GROW_DOWN) && (dir & GROW_RIGHT)) {
     XResizeWindow(theDisplay, man->theWindow, man->geometry.width,
@@ -421,6 +442,17 @@ static void resize_window(WinManager *man)
     }
     MyXUngrabServer(theDisplay);
   }
+  /* Wait until the window has resized -- there could be a timing issue and
+   * we don't want to warn about incorrect sizehints when this is still
+   * happening.
+   */
+  while (counter && (attribs.width != man->geometry.width ||
+	 attribs.height != man->geometry.height)) {
+		if (!XGetWindowAttributes(theDisplay, man->theWindow, &attribs))
+			break;
+		counter--;
+  }
+  fix_manager_size(man, man->geometry.width, man->geometry.height);
 }
 
 static char *make_display_string(WinData *win, char *format, int len)
@@ -633,12 +665,10 @@ static void set_num_buttons(ButtonArray *buttons, int n)
 
   if (n > buttons->num_buttons) {
     buttons->dirty_flags |= NUM_BUTTONS_CHANGED;
-    buttons->buttons = (Button **)saferealloc((void *)buttons->buttons,
-					   n * sizeof(Button *));
+    buttons->buttons = fxrealloc((void *)buttons->buttons, n, sizeof(Button *));
 
     for (i = buttons->num_buttons; i < n; i++) {
-      buttons->buttons[i] = (Button *)safemalloc(sizeof(Button));
-      memset(buttons->buttons[i], 0, sizeof(Button));
+      buttons->buttons[i] = fxcalloc(1, sizeof(Button));
       buttons->buttons[i]->drawn_state.display_string = NULL;
       buttons->buttons[i]->index = i;
     }
@@ -835,7 +865,15 @@ void set_win_iconified(WinData *win, int iconified)
 		win->button->drawn_state.dirty_flags |= ICON_STATE_CHANGED;
 	}
 	win->iconified = iconified;
-	if (!iconified)
+	if (iconified)
+	{
+		win->state = ICON_CONTEXT;
+		if (globals.select_win == win)
+		{
+			add_win_state(win, SELECT_CONTEXT);
+		}
+	}
+	else
 	{
 		win->state = PLAIN_CONTEXT;
 		if (globals.select_win == win)
@@ -880,6 +918,11 @@ void add_win_state(WinData *win, int flag)
 		{
 			win->state = FOCUS_SELECT_CONTEXT;
 		}
+		else if (win->state == ICON_CONTEXT ||
+			 win->state == ICON_SELECT_CONTEXT)
+		{
+			win->state = ICON_SELECT_CONTEXT;
+		}
 		else
 		{
 			win->state = SELECT_CONTEXT;
@@ -894,18 +937,23 @@ void add_win_state(WinData *win, int flag)
 	{
 		win->button->drawn_state.dirty_flags |= STATE_CHANGED;
 	}
-	ConsoleDebug(X11, "add_win_state: %s 0x%x\n", win->titlename, flag);
 }
 
 /* this is "broken" */
 void del_win_state(WinData *win, int flag)
 {
-
 	if (flag == FOCUS_CONTEXT)
 	{
 		if (win->state == FOCUS_SELECT_CONTEXT)
 		{
-			win->state = SELECT_CONTEXT;
+			if (win->iconified)
+			{
+				win->state = ICON_SELECT_CONTEXT;
+			}
+			else
+			{
+				win->state = SELECT_CONTEXT;
+			}
 		}
 		else if (win->state == FOCUS_CONTEXT)
 		{
@@ -929,16 +977,13 @@ void del_win_state(WinData *win, int flag)
 		{
 			win->state = FOCUS_CONTEXT;
 		}
+		else if (win->state == ICON_SELECT_CONTEXT)
+		{
+			win->state = ICON_CONTEXT;
+		}
 		else if (win->state == SELECT_CONTEXT)
 		{
-			if (win->iconified)
-			{
-				win->state = ICON_CONTEXT;
-			}
-			else
-			{
-				win->state = PLAIN_CONTEXT;
-			}
+			win->state = PLAIN_CONTEXT;
 		}
 		else
 		{
@@ -954,7 +999,6 @@ void del_win_state(WinData *win, int flag)
 	{
 		win->button->drawn_state.dirty_flags |= STATE_CHANGED;
 	}
-	ConsoleDebug(X11, "del_win_state: %s 0x%x\n", win->titlename, flag);
 }
 
 void set_win_displaystring(WinData *win)
@@ -1083,7 +1127,9 @@ static void clear_empty_region(WinManager *man)
 		rects[0].x, rects[0].y, rects[0].width, rects[0].height,
 		rects[1].x, rects[1].y, rects[1].width, rects[1].height);
 
-  if (CSET_IS_TRANSPARENT_PR_PURE(man->colorsets[DEFAULT]))
+  int cset = man->colorsets[DEFAULT];
+
+  if (CSET_IS_TRANSPARENT_PR_PURE(cset))
   {
     for(n=0; n < num_rects; n++)
     {
@@ -1095,6 +1141,19 @@ static void clear_empty_region(WinManager *man)
 	    }
     }
   }
+  else if (cset >= 0)
+  {
+    for(n=0; n < num_rects; n++)
+    {
+	    if (rects[n].width > 0 && rects[n].height > 0)
+	    {
+		    SetRectangleBackground(theDisplay, man->theWindow,
+			    rects[n].x, rects[n].y, rects[n].width,
+			    rects[n].height, &Colorset[cset], Pdepth,
+			    man->backContext[DEFAULT]);
+	    }
+    }
+  }
   else
   {
     XFillRectangles(theDisplay, man->theWindow,
@@ -1372,8 +1431,16 @@ static void draw_button_background(
 {
 	int cset = man->colorsets[button_state];
 
-	if (CSET_IS_TRANSPARENT_PR_PURE(cset) ||
-	    man->backContext[button_state] == None)
+	if (cset >= 0)
+	{
+		SetRectangleBackground(
+			theDisplay, man->theWindow,
+			bounding.x, bounding.y,
+			bounding.width, bounding.height,
+			&Colorset[cset], Pdepth,
+			man->backContext[button_state]);
+	}
+	else if (man->backContext[button_state] == None)
 	{
 		XClearArea(
 			theDisplay, man->theWindow,
@@ -1381,14 +1448,6 @@ static void draw_button_background(
 			bounding.width, bounding.height,
 			False);
 	}
-	else if (CSET_IS_TRANSPARENT_PR_TINT(cset))
-	{
-		SetRectangleBackground(
-			theDisplay, man->theWindow,
-			bounding.x, bounding.y,
-			bounding.width, bounding.height,
-			&Colorset[cset], Pdepth, man->backContext[button_state]);
-	}
 	else
 	{
 		XFillRectangle(
@@ -1826,7 +1885,9 @@ static void draw_button(WinManager *man, int button, int force)
 					man, button_state, &g, context1,
 					context2);
 			}
-			else if (button_state & SELECT_CONTEXT)
+			else if (button_state == SELECT_CONTEXT ||
+				 button_state == FOCUS_SELECT_CONTEXT ||
+				 button_state == ICON_SELECT_CONTEXT)
 			{
 				XDrawRectangle(
 					theDisplay, man->theWindow,
@@ -1962,7 +2023,7 @@ void draw_manager(WinManager *man)
   if (!man->window_up)
     return;
   ConsoleDebug(X11, "Drawing Manager: %s\n", man->titlename);
-  redraw_all = man->dirty_flags & REDRAW_MANAGER;
+  redraw_all = (man->dirty_flags & REDRAW_MANAGER);
 
   if (!man->flags.is_shaded && man->flags.needs_resize_after_unshade) {
     ConsoleDebug(X11, "\tresizing manager\n");
@@ -2347,7 +2408,14 @@ void delete_windows_button(WinData *win)
 		  selected_index);
     move_highlight(man, man->buttons.buttons[selected_index]);
   }
-  win->state = PLAIN_CONTEXT;
+  if (win->iconified)
+  {
+    win->state = ICON_CONTEXT;
+  }
+  else
+  {
+    win->state = PLAIN_CONTEXT;
+  }
 }
 
 void resort_windows_button(WinData *win)
@@ -2473,7 +2541,7 @@ void man_exposed(WinManager *man, XEvent *theEvent)
 				bp[i]->drawn_state.ex = max(r1.x,r2.x);
 				bp[i]->drawn_state.ey = max(r1.y,r2.y);
 				bp[i]->drawn_state.ew =
-					min(r1.x+r1.width,r2.x+r2.width) - 
+					min(r1.x+r1.width,r2.x+r2.width) -
 						max(r1.x,r2.x);
 				bp[i]->drawn_state.eh =
 					min(r1.y+r1.height, r2.y+r2.height) -
