@@ -16,8 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see: <http://www.gnu.org/licenses/>
  */
 
 #include "config.h"
@@ -26,6 +25,7 @@
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
+#include "libs/FEvent.h"
 #include "libs/fvwmsignal.h"
 #include "libs/Picture.h"
 #include "libs/PictureGraphics.h"
@@ -40,17 +40,18 @@
 #include "libs/Parse.h"
 #include "libs/Strings.h"
 #include "libs/XError.h"
+#include "libs/Cursor.h"
 
 #ifdef MEMDEBUG                 /* For debugging */
 #include <unchecked.h>
 #endif
 
-/* Variables utilise par l'analyseur syntaxique */
+/* Variables used by syntax interpreter */
 extern ScriptProp *scriptprop;
-extern int nbobj;                       /* Nombre d'objets */
-extern int numligne;                    /* Numero de ligne */
-extern TabObj *tabobj;                  /* Tableau d'objets, limite=1000 */
-extern char **TabVVar;                  /* Tableau des variables du sript */
+extern int nbobj;                       /* Number of objects */
+extern int numligne;                    /* Line Number */
+extern TabObj *tabobj;                  /* Object array, limit=1000 */
+extern char **TabVVar;                  /* Script variables array */
 extern int TabIdObj[1001];
 extern Bloc **TabIObj;
 extern CaseObj *TabCObj;
@@ -58,15 +59,15 @@ extern CaseObj *TabCObj;
 extern int __bounds_debug_no_checking;
 #endif
 
-/* Constante de couleurs utilise dans le tableau TabColor */
+/* Color constants used in TabColor array */
 #define back 0
 #define fore 1
 #define shad 2
 #define hili 3
 
 
-/* Variables globales */
-char *ScriptName;       /* Nom du fichier contenat le script decrivant le GUI */
+/* Global Variables */
+char *ScriptName;       /* Filename for GUI script */
 char *ScriptBaseName;
 char *ScriptPath = "";
 ModuleArgs *module;
@@ -82,8 +83,8 @@ extern void (*TabCom[25]) (int NbArg,long *TabArg);
 Display *dpy;
 int screen;
 Window  Root;
-X11base *x11base;               /* Pour le serveur X */
-TypeBuffSend BuffSend;          /* Pour les communication entre script */
+X11base *x11base;               /* For X server */
+TypeBuffSend BuffSend;          /* For inter script communications */
 int grab_server = 0;
 struct XObj *tabxobj[1000];
 char *Scrapt;
@@ -92,12 +93,13 @@ static Atom wm_del_win;
 char *imagePath = NULL;
 static Bool is_dead_pipe = False;
 KeySym shift_tab_ks;  /* shift-Tab keysym */
-char *LastString = NULL;  /* last string send by a SendString ot Key cmd */
+char *LastString = NULL;  /* last string sent by a SendString ot Key cmd */
 Binding *BindingsList;
 
 extern void InitCom(void);
 static void TryToFind(char *filename);
 
+char *default_cursor;
 
 /* Exit procedure - called whenever we call exit(), or when main() ends */
 static void
@@ -106,7 +108,6 @@ ShutdownX(void)
   int i;
   static XEvent event;
   fd_set in_fdset;
-  extern int x_fd;
   Atom MyAtom;
   int NbEssai=0;
   struct timeval tv;
@@ -122,15 +123,15 @@ ShutdownX(void)
   if (is_dead_pipe)
     return;
 
-  /* On cache la fenetre */
+  /* We hide the window */
   XUnmapWindow(dpy,x11base->win);
   XFlush(dpy);
 
-  /* Le script ne possede plus la propriete */
+  /* Script doesn't own the property anymore */
   MyAtom = XInternAtom(dpy, x11base->TabScriptId[1], False);
   XSetSelectionOwner(dpy, MyAtom, x11base->root, CurrentTime);
 
-  /* On verifie si tous les messages ont ete envoyes */
+  /* We check whether all messages have been sent */
   while(!isTerminated && (BuffSend.NbMsg > 0) && (NbEssai < 10000) )
   {
     tv.tv_sec = 1;
@@ -150,9 +151,9 @@ ShutdownX(void)
   }
   XFlush(dpy);
 
-  /* Attente de deux secondes afin d'etre sur que tous */
-  /* les messages soient arrives a destination         */
-  /* On quitte proprement le serveur X */
+  /* Wait for 2 seconds, to be sure all */
+  /* messages are received */
+  /* Exiting cleanly the X server */
   for (i=0; i<nbobj; i++)
     tabxobj[i]->DestroyObj(tabxobj[i]);
   XFlush(dpy);
@@ -169,17 +170,18 @@ void Debug(void)
   for (j=1;j<=nbobj;j++)
     for (i=0; i<=TabCObj[TabIdObj[j]].NbCase; i++)
     {
-      /* Execution du bloc d'instruction */
-      fprintf(stderr,"Id de l'objet %d\n",TabIdObj[j]);
-      fprintf(stderr,"Nb Instruction %d\n",TabIObj[TabIdObj[j]][i].NbInstr);
+      /* Execution of instruction block */
+      fvwm_debug(__func__, "Id de l'objet %d\n",TabIdObj[j]);
+      fvwm_debug(__func__, "Nb Instruction %d\n",
+                 TabIObj[TabIdObj[j]][i].NbInstr);
     }
 
 }
 
-/* Lecture du fichier contenant le script */
+extern FILE *yyin;
+/* Script file reading */
 void ReadConfig (char *ScriptName)
 {
-  extern FILE *yyin;
   char s[FILENAME_MAX];
 
   /* This is legacy support.  The first try is the command line file name
@@ -198,11 +200,11 @@ void ReadConfig (char *ScriptName)
 	    module->name,s);
     exit(1);
   }
-  /* On ne redefini pas yyout qui est la sortie standard */
+  /* We don't define yyout again, it's the standard output */
 
-  /* Application de l'analyseur syntaxique et lexical */
+  /* Applying syntaxic and lexical interpreter */
   yyparse();
-  /* Fermeture du script */
+  /* Closing the script */
 
   fclose(yyin);
 }
@@ -215,7 +217,6 @@ void ReadConfig (char *ScriptName)
    The extern "yyin" indicates success or failure.
 */
 static void TryToFind(char *filename) {
-  extern FILE *yyin;
   char path[FILENAME_MAX];
 
   if (filename[0] == '/') {             /* if absolute path */
@@ -231,7 +232,7 @@ static void TryToFind(char *filename) {
   return;
 }
 
-/* Quitter par l'option Delete du bouton de la fenetre */
+/* Exiting with Delete button */
 RETSIGTYPE DeadPipe(int nonsense)
 {
   is_dead_pipe = True;
@@ -239,7 +240,7 @@ RETSIGTYPE DeadPipe(int nonsense)
   SIGNAL_RETURN;
 }
 
-/* Lecture du fichier system.fvwmrc ou .fvwmrc */
+/* Reading system.fvwmrc or .fvwmrc file */
 void ParseOptions(void)
 {
   char *tline;
@@ -281,10 +282,11 @@ void ParseOptions(void)
 	CopyStringWithQuotes(&x11base->font,&tline[22]);
       else if (strncasecmp(tline,"Colorset",8) == 0)
 	LoadColorset(&tline[8]);
-      else if (strncasecmp(tline, XINERAMA_CONFIG_STRING,
-			   sizeof(XINERAMA_CONFIG_STRING) - 1) == 0)
-	FScreenConfigureModule(
-	  tline + sizeof(XINERAMA_CONFIG_STRING) - 1);
+      else if (strncasecmp(tline,"*FvwmScriptDefaultCursor",24) == 0)
+      {
+	CopyString(&x11base->cursor,&tline[24]);
+	default_cursor = x11base->cursor;
+      }
     }
 
     GetConfigLine(fd,&tline);
@@ -306,14 +308,14 @@ static int myErrorHandler(Display *dpy, XErrorEvent *event)
   return 0;
 }
 
-/* Procedure d'initialisation du serveur X et des variables globales*/
+/* Initialize X server and global variables */
 void Xinit(int IsFather)
 {
   char *name;
   Atom myatom;
   int i = 16;
 
-  /* Connextion au serveur X */
+  /* Connection to X server */
 #ifdef MEMDEBUG
   __bounds_debug_no_checking=True;
 #endif
@@ -321,8 +323,8 @@ void Xinit(int IsFather)
   dpy=XOpenDisplay(NULL);
   if (dpy == NULL)
   {
-    fprintf(stderr,"[%s]: <<ERROR>> Can't open display %s",
-	    ScriptName, XDisplayName(NULL));
+    fvwm_debug(__func__, "[%s]: <<ERROR>> Can't open display %s",
+               ScriptName, XDisplayName(NULL));
     exit(1);
   }
   screen = DefaultScreen(dpy);
@@ -337,7 +339,7 @@ void Xinit(int IsFather)
 
   if (IsFather)
   {
-    name = (char*)safecalloc(sizeof(char),strlen("FvwmScript")+5);
+    name = fxcalloc(sizeof(char), strlen("FvwmScript") + 5);
     do
     {
       sprintf(name,"%c%xFvwmScript",161,i);
@@ -358,7 +360,7 @@ void Xinit(int IsFather)
 }
 
 /*
- * Lecture d'un icone
+ * Reading an icon
  */
 void LoadIcon(struct XObj *xobj)
 {
@@ -406,9 +408,9 @@ void LoadIcon(struct XObj *xobj)
 		&xobj->icon_h, &depth, &xobj->nalloc_pixels,
 		&xobj->alloc_pixels, 0, fpa))
 	{
-		fprintf(stderr,"[%s][LoadIcon]: <<WARNING>> Unable to "
-			"load pixmap %s\n",
-			ScriptName,xobj->icon);
+		fvwm_debug(__func__, "[%s][LoadIcon]: <<WARNING>> Unable to "
+			   "load pixmap %s\n",
+			   ScriptName,xobj->icon);
 		free(path);
 		return;
 	}
@@ -434,7 +436,7 @@ void LoadIcon(struct XObj *xobj)
 }
 
 
-/* Ouvre une fenetre pour l'affichage du GUI */
+/* Opening a GUI window */
 void OpenWindow (void)
 {
   XTextProperty Name;
@@ -443,8 +445,9 @@ void OpenWindow (void)
   unsigned long mask;
   XClassHint classHints;
   XSetWindowAttributes Attr;
+  int xcnum;
 
-  /* Allocation des couleurs */
+  /* Allocation for colors */
   if (x11base->colorset >= 0) {
     x11base->TabColor[fore] = Colorset[x11base->colorset].fg;
     x11base->TabColor[back] = Colorset[x11base->colorset].bg;
@@ -457,12 +460,22 @@ void OpenWindow (void)
     x11base->TabColor[hili] = GetColor(x11base->hilicolor);
   }
 
-  /* Definition des caracteristiques de la fentre */
+  /* Window properties definition */
   mask = CWBackPixel | CWBorderPixel | CWColormap;
   Attr.background_pixel = x11base->TabColor[back];
   Attr.border_pixel = 0;
   Attr.colormap = Pcmap;
 
+  xcnum = fvwmCursorNameToIndex(default_cursor);
+  if (xcnum != -1) {
+    Attr.cursor = XCreateFontCursor(dpy, xcnum);
+    mask |= CWCursor;
+  } else {
+    /* Bad name or value, clear the config slot. */
+    x11base->cursor = NULL;
+  }
+  free(default_cursor);
+
   x11base->win=XCreateWindow(
     dpy, x11base->root, x11base->size.x,x11base->size.y, x11base->size.width,
     x11base->size.height, 0, Pdepth, InputOutput, Pvisual, mask, &Attr);
@@ -472,16 +485,17 @@ void OpenWindow (void)
       dpy, x11base->win, x11base->size.width, x11base->size.height,
       &Colorset[x11base->colorset], Pdepth, x11base->gc, True);
 
-  /* Choix des evts recus par la fenetre */
+  /* Select which events are received by the window */
   XSelectInput(dpy,x11base->win, KeyPressMask|ButtonPressMask|
 	       ExposureMask|ButtonReleaseMask|EnterWindowMask|LeaveWindowMask|
 	       ButtonMotionMask|StructureNotifyMask);
   XSelectInput(dpy, x11base->root, PropertyChangeMask);
 
-  /* Specification des parametres utilises par le gestionnaire de fenetre */
+  /* Specify which parameters are used by the window manager */
   if (XStringListToTextProperty(&x11base->title,1,&Name) == 0)
-    fprintf(stderr,"[%s][OpenWindow]: <<WARNING>> Can't use icon name\n",
-	    ScriptName);
+    fvwm_debug(__func__,
+               "[%s][OpenWindow]: <<WARNING>> Can't use icon name\n",
+               ScriptName);
   IndicNorm = XAllocSizeHints();
   if (x11base->size.x!=-1)
   {
@@ -502,14 +516,14 @@ void OpenWindow (void)
   IndicWM->initial_state = NormalState;
   IndicWM->flags = InputHint|StateHint;
 
-  classHints.res_name = safestrdup(ScriptBaseName);
-  classHints.res_class = safestrdup(module->name);
+  classHints.res_name = fxstrdup(ScriptBaseName);
+  classHints.res_class = fxstrdup(module->name);
 
   XSetWMProperties(dpy, x11base->win, &Name,
 		   &Name, NULL, 0, IndicNorm, IndicWM, &classHints);
-  Scrapt = (char*)safecalloc(sizeof(char),1);
+  Scrapt = fxcalloc(sizeof(char), 1);
 
-  /* Construction des atomes pour la communication inter-application */
+  /* Build atoms for inter application communication */
   propriete = XInternAtom(dpy,"Prop_selection",False);
   wm_del_win = XInternAtom(dpy,"WM_DELETE_WINDOW",False);
   XSetWMProtocols(dpy,x11base->win,&wm_del_win,1);
@@ -519,8 +533,8 @@ void OpenWindow (void)
 }
 
 /*
- * Execution d'une sequence d'instruction
- */
+ * Executing an instruction sequence
+*/
 void ExecBloc(Bloc *bloc)
 {
   int i;
@@ -533,7 +547,7 @@ void ExecBloc(Bloc *bloc)
 }
 
 
-/* Construction de l'interface graphique */
+/* Building the GUI */
 void BuildGUI(int IsFather)
 {
   int i;
@@ -585,14 +599,14 @@ void BuildGUI(int IsFather)
   x11base->size.height = scriptprop->height;
   x11base->title = scriptprop->titlewin;
 
-  /* Initialisation de la fenetre */
+  /* Window initialization */
   OpenWindow();
 
-  /* Parcour de tous les objets graphiques */
+  /* Go through every graphical object */
   nbobj++;
   for (i=0; i<nbobj; i++)
   {
-    tabxobj[i] = (struct XObj*)safecalloc(1,sizeof(struct XObj));
+    tabxobj[i] = fxcalloc(1, sizeof(struct XObj));
     tabxobj[i]->id = (*tabobj)[i].id;
     tabxobj[i]->x = (*tabobj)[i].x;
     tabxobj[i]->y = (*tabobj)[i].y;
@@ -613,32 +627,32 @@ void BuildGUI(int IsFather)
     tabxobj[i]->swallow = (*tabobj)[i].swallow;
 
     if ((*tabobj)[i].title == NULL)
-      tabxobj[i]->title = (char*)safecalloc(1,200);
+      tabxobj[i]->title = fxcalloc(1, 200);
     else
       tabxobj[i]->title = (*tabobj)[i].title;
 
     if ((*tabobj)[i].font == NULL && x11base->font != NULL)
-      tabxobj[i]->font = (char*)safestrdup(x11base->font);
+      tabxobj[i]->font = fxstrdup(x11base->font);
     else
       tabxobj[i]->font = (*tabobj)[i].font;
 
     if ((*tabobj)[i].forecolor == NULL)
-      tabxobj[i]->forecolor = (char*)safestrdup(x11base->forecolor);
+      tabxobj[i]->forecolor = fxstrdup(x11base->forecolor);
     else
       tabxobj[i]->forecolor = (*tabobj)[i].forecolor;
 
     if ((*tabobj)[i].backcolor == NULL)
-      tabxobj[i]->backcolor = (char*)safestrdup(x11base->backcolor);
+      tabxobj[i]->backcolor = fxstrdup(x11base->backcolor);
     else
       tabxobj[i]->backcolor = (*tabobj)[i].backcolor;
 
     if ((*tabobj)[i].shadcolor == NULL)
-      tabxobj[i]->shadcolor = (char*)safestrdup(x11base->shadcolor);
+      tabxobj[i]->shadcolor = fxstrdup(x11base->shadcolor);
     else
       tabxobj[i]->shadcolor = (*tabobj)[i].shadcolor;
 
     if ((*tabobj)[i].hilicolor == NULL)
-      tabxobj[i]->hilicolor = safestrdup(x11base->hilicolor);
+      tabxobj[i]->hilicolor = fxstrdup(x11base->hilicolor);
     else
       tabxobj[i]->hilicolor = (*tabobj)[i].hilicolor;
 
@@ -656,17 +670,17 @@ void BuildGUI(int IsFather)
     tabxobj[i]->iconPixmap = None;
     tabxobj[i]->icon_maskPixmap = None;
 
-    LoadIcon(tabxobj[i]);                  /* Chargement de l'icone du widget */
+    LoadIcon(tabxobj[i]);                  /* Loading the widget icon */
 
     tabxobj[i]->InitObj(tabxobj[i]);
   }
 
-  /* Enregistrement du bloc de taches periodic */
+  /* Registering the periodic task block */
   x11base->periodictasks = scriptprop->periodictasks;
   /* the QuitFunc */
   x11base->quitfunc = scriptprop->quitfunc;
 
-  /*Si un bloc d'initialisation du script existe, on l'execute ici */
+  /* If an initialization block exists for the script, it's executed here */
   if (scriptprop->initbloc != NULL)
   {
     ExecBloc(scriptprop->initbloc);
@@ -683,7 +697,7 @@ void BuildGUI(int IsFather)
 }
 
 /*
- * Fonction de traitement des msg entre objets
+ * Managing messages between objects
  */
 void SendMsg(struct XObj *xobj,int TypeMsg)
 {
@@ -692,13 +706,13 @@ void SendMsg(struct XObj *xobj,int TypeMsg)
   for (i=0; i <= TabCObj[TabIdObj[xobj->id]].NbCase; i++)
     if (TabCObj[TabIdObj[xobj->id]].LstCase[i] == TypeMsg)
     {
-      /* Execution du bloc d'instruction */
+      /* Executing the instruction block */
       ExecBloc(&TabIObj[TabIdObj[xobj->id]][i]);
     }
 }
 
 /*
- * Fonction de traitement des msg entre objets
+ * Managing messages between objects
  */
 void SendMsgAndString(char *action, char *type)
 {
@@ -712,8 +726,9 @@ void SendMsgAndString(char *action, char *type)
   if(StrEquals(type,"SendString") || StrEquals(type,"CheckBinding")){
     if (GetIntegerArguments(action, NULL, val, 2) == 2){
       if (val[0] > 1000 || val[0] < 1){
-	fprintf(stderr,"[%s][%s]: <<WARNING>> Widget id out of range: %i\n",
-		ScriptName,type,val[0]);
+	fvwm_debug(__func__,
+                   "[%s][%s]: <<WARNING>> Widget id out of range: %i\n",
+                   ScriptName,type,val[0]);
 	return;
       }
 
@@ -731,14 +746,14 @@ void SendMsgAndString(char *action, char *type)
 
 	SendMsg(tabxobj[i],val[1]);
       }else{
-	fprintf(stderr,"[%s][%s]: <<WARNING>> no Widget %i\n",
-		ScriptName,type,val[0]);
+	fvwm_debug(__func__, "[%s][%s]: <<WARNING>> no Widget %i\n",
+                   ScriptName,type,val[0]);
       }
       if (token)
 	free(token);
     }else{
-      fprintf(stderr,"[%s][%s]: <<WARNING>> Syntax Error: %s\n",
-	      ScriptName,type,action);
+      fvwm_debug(__func__, "[%s][%s]: <<WARNING>> Syntax Error: %s\n",
+                 ScriptName,type,action);
     }
   }else{
 
@@ -754,9 +769,8 @@ void SendMsgAndString(char *action, char *type)
 
       }else{
 	if(XFetchName(dpy, x11base->win, &windowName) == 0){
-		fprintf(
-			stderr,"[%s][SendMsgAndString]: <<ERROR>> "
-			"Can't find the title of a window\n", module->name);
+		fvwm_debug(__func__, "[%s][SendMsgAndString]: <<ERROR>> "
+                           "Can't find the title of a window\n", module->name);
 	}else{
 	  if(StrEquals(arg2,windowName)){
 	    XChangeProperty(
@@ -768,9 +782,9 @@ void SendMsgAndString(char *action, char *type)
     }
     else
     {
-	    fprintf(
-		    stderr,"[%s][SendMsgAndString]: <<ERROR>> "
-		    "Unknown SendToModule command: %s\n", module->name, type);
+	    fvwm_debug(__func__, "[%s][SendMsgAndString]: <<ERROR>> "
+                       "Unknown SendToModule command: %s\n", module->name,
+                       type);
     }
 
     if(arg1) free(arg1);
@@ -780,7 +794,7 @@ void SendMsgAndString(char *action, char *type)
 }
 
 /*
- * Appeler lors d'une demande de selection
+ * Called by a select request
  */
 void SendMsgToScript(XEvent event)
 {
@@ -805,7 +819,7 @@ void SendMsgToScript(XEvent event)
     evnt_sel.xselection.selection = event.xselectionrequest.selection;
     evnt_sel.xselection.target = Receiver;
     evnt_sel.xselection.time = event.xselectionrequest.time;
-    /* On a trouve le recepteur */
+    /* Receiver has been found */
     if (event.xselectionrequest.target == Receiver)
     {
       evnt_sel.xselection.property = event.xselectionrequest.property;
@@ -821,13 +835,13 @@ void SendMsgToScript(XEvent event)
       free(BuffSend.TabMsg[i].Msg);
       if (BuffSend.NbMsg > 0)
       {
-	memmove(&BuffSend.TabMsg[i],
+memmove(&BuffSend.TabMsg[i],
 		&BuffSend.TabMsg[i+1],(BuffSend.NbMsg-i)*sizeof(TypeName));
       }
     }
     else
     {
-      /* Cas ou le recepteur demande un message et qu'il n'y en a pas */
+      /* If receiver asks for a message, but there is none */
       evnt_sel.xselection.property = None;
     }
     FSendEvent(dpy,evnt_sel.xselection.requestor, False, 0, &evnt_sel);
@@ -954,9 +968,11 @@ void ReadXServer (void)
     {
       Bool moved = False;
 
+      fev_sanitise_configure_notify(&event.xconfigure);
       moved = event.xconfigure.send_event;
       while (FCheckTypedEvent(dpy, ConfigureNotify, &event))
       {
+	fev_sanitise_configure_notify(&event.xconfigure);
 	/* check for movement */
 	if (event.xconfigure.send_event)
 	  moved = True;
@@ -968,20 +984,19 @@ void ReadXServer (void)
     }
     break;
     case KeyPress:
-      /* Touche presse dans un objet / Key press in an object */
+      /* Key pressed in an object */
       isTab = 0;
       find = False;
       XLookupString(&event.xkey, (char *)buf, sizeof(buf), &ks, NULL);
       event.xkey.keycode =
-	XKeysymToKeycode(dpy,XKeycodeToKeysym(dpy,event.xkey.keycode,0));
+	XKeysymToKeycode(dpy,fvwm_KeycodeToKeysym(dpy,event.xkey.keycode,0,0));
 
       /* check for bindings defined by the Key instruction */
 	  tmp.res_class = tmp.res_name = "root";
       if ((action = CheckBinding(
-		   BindingsList, STROKE_ARG(0) event.xkey.keycode,
+		   BindingsList, event.xkey.keycode,
 		   event.xkey.state, LockMask, C_WINDOW, BIND_KEYPRESS,
-		   &tmp, tmp.res_class)) !=
-	  NULL)
+		   &tmp, tmp.res_class)) != NULL)
       {
 	SendMsgAndString(action, "CheckBinding");
 	break;
@@ -998,7 +1013,7 @@ void ReadXServer (void)
 	isTab = 2;
       if (event.xkey.subwindow!=0)
       {
-	/* Envoi de l'evt à l'objet */
+	/* Sending event to object */
 	for (i=0; i<nbobj; i++)
 	{
 	  if (tabxobj[i]->win == event.xkey.subwindow)
@@ -1038,7 +1053,7 @@ void ReadXServer (void)
       }
       break;
     case ButtonPress:
-      /* Clique dans quel fenetre? */
+      /* Which window is clicked in? */
       if (event.xbutton.subwindow != 0)
 	for (i=0; i<nbobj; i++)
 	  if (tabxobj[i]->win == event.xbutton.subwindow) {
@@ -1115,7 +1130,7 @@ void ReadXServer (void)
       else break;
       if (i > 0)
       {
-	Scrapt = (char*)saferealloc((void*)Scrapt,sizeof(char)*(i+1));
+	Scrapt = fxrealloc((void*)Scrapt,sizeof(char)*(i+1), sizeof(Scrapt));
 	Scrapt = strcpy(Scrapt,octet);
       }
       break;
@@ -1228,11 +1243,7 @@ void MainLoop (void)
 	      }
 	    }
 	  }
-	  else if (StrEquals(token, XINERAMA_CONFIG_STRING)) {
-	    FScreenConfigureModule(line);
-	  }
-	  if (token)
-	    free(token);
+	  free(token);
 	}
 	else if (packet->type == MX_PROPERTY_CHANGE)
 	{
@@ -1297,7 +1308,7 @@ void MainLoop (void)
 
     if (!isTerminated && x11base->periodictasks!=NULL && delta >= 1000000)
     {
-      /* Execution des taches periodics */
+      /* Executing periodic tasks */
       last_periodic.tv_sec = now.tv_sec;
       last_periodic.tv_usec = now.tv_usec;
       ExecBloc(x11base->periodictasks);
@@ -1311,21 +1322,25 @@ void ReadFvwmScriptArg(int argc, char **argv,int IsFather)
   int i;
   Atom myatom;
 
-  BuffSend.NbMsg=0;                     /* Aucun message dans le buffer */
+  BuffSend.NbMsg=0;             /* No message in buffer */
 
   for (i=2; i<98; i++)
     x11base->TabScriptId[i]=NULL;
 
-  if (IsFather)                 /* Cas du pere */
+  if (IsFather)                 /* Father case */
   {
     myatom = XInternAtom(dpy,x11base->TabScriptId[1],True);
     XSetSelectionOwner(dpy, myatom, x11base->win, CurrentTime);
   }
   else
-  {                             /* Cas du fils */
-    x11base->TabScriptId[0] = (char*)safecalloc(sizeof(char),strlen(argv[7]));
-    x11base->TabScriptId[0] = strncpy(x11base->TabScriptId[0],argv[7],
-				    strlen(argv[7])-2);
+  {                             /* Son case */
+    char *a7_cpy = fxstrdup(argv[7]);
+
+    a7_cpy[strlen(a7_cpy) - 2] = '\0';
+
+    free(x11base->TabScriptId[0]);
+    xasprintf(&x11base->TabScriptId[0], "%s", a7_cpy);
+    free(a7_cpy);
     x11base->TabScriptId[1] = argv[7];
     myatom=XInternAtom(dpy, x11base->TabScriptId[1], True);
     XSetSelectionOwner(dpy, myatom, x11base->win, CurrentTime);
@@ -1353,17 +1368,17 @@ int main (int argc, char **argv)
   module = ParseModuleArgs(argc,argv,0); /* no alias */
   if (module == NULL)
   {
-    fprintf(stderr,"FvwmScript must be started by Fvwm.\n");
+    fvwm_debug(__func__, "FvwmScript must be started by Fvwm.\n");
     exit(1);
   }
 
   if (module->user_argc == 0)
   {
-    fprintf(stderr,"FvwmScript requires the script's name or path.\n");
+    fvwm_debug(__func__, "FvwmScript requires the script's name or path.\n");
     exit(1);
   }
 
-  /* On determine si le script a un pere */
+  /* Check whether the script has a father */
   if (module->user_argc >= 2)
     IsFather = (module->user_argv[1][0] != (char)161);
   else
@@ -1379,30 +1394,30 @@ int main (int argc, char **argv)
 		 M_MAP|  M_RES_NAME| M_RES_CLASS| M_CONFIG_INFO|
 		 M_END_CONFIG_INFO| M_WINDOW_NAME | M_SENDCONFIG);
   SetMessageMask(fd, MX_PROPERTY_CHANGE);
-  /* Enregistrement des arguments du script */
-  x11base = (X11base*) safecalloc(1,sizeof(X11base));
+  /* Saving the script arguments */
+  x11base = fxcalloc(1, sizeof(X11base));
   x11base->TabArg[0] = module->name;
   for (i=2-IsFather; i< module->user_argc; i++)
     x11base->TabArg[i-1+IsFather] = module->user_argv[i];
-  /* Couleurs et fontes par defaut */
+  /* Default colors and fonts */
   x11base->font = NULL;
-  x11base->forecolor = safestrdup("black");
-  x11base->backcolor = safestrdup("grey85");
-  x11base->shadcolor = safestrdup("grey55");
-  x11base->hilicolor = safestrdup("grey100");
+  x11base->forecolor = fxstrdup("black");
+  x11base->backcolor = fxstrdup("grey85");
+  x11base->shadcolor = fxstrdup("grey55");
+  x11base->hilicolor = fxstrdup("grey100");
   x11base->colorset = -1;
   x11base->swallowed = False;
 
-  /* Initialisation du serveur X et de la fenetre */
+  /* Initializing X server and window */
   Xinit(IsFather);
 
   ParseOptions();
 
   SendText(fd,"Send_WindowList",0);
 
-  ReadConfig(ScriptName);       /* Lecture et analyse du script */
+  ReadConfig(ScriptName);       /* Read and interpret the script */
 
-  /* Fonction d'initialisation de TabCom et TabFunc   */
+  /* Initializing TabCom and TabFunc   */
   InitCom();
 
 #ifdef HAVE_SIGACTION
@@ -1452,10 +1467,10 @@ int main (int argc, char **argv)
   /* Compute the Shift-Tab keysym */
   {
     KeyCode tab = XKeysymToKeycode(dpy, XK_Tab);
-    shift_tab_ks = XKeycodeToKeysym(dpy, tab, ShiftMask);
+    shift_tab_ks = fvwm_KeycodeToKeysym(dpy, tab, ShiftMask, 0);
   }
 
-  /* Construction des boutons et de la fenetre */
+  /* Building buttons and window */
   BuildGUI(IsFather);
 
   ReadFvwmScriptArg(argc,argv,IsFather);
